//谷歌、高德 腾讯都属于同一坐标系
let GpsH = {
    // 经纬度转换常量
    PI: 3.1415926535897932384626,
    XPI: 3.14159265358979324 * 3000.0 / 180.0,
    A: 6378245.0,
    EE: 0.00669342162296594323,
    // GPSCorrect
    /**
     * 原始坐标转谷歌坐标
     * @param wgLat
     * @param wgLon
     * @return
     */
     oriToGoogle  (wgLon, wgLat) {
        let latlng = [];
        let dLat =  this.transformLat(wgLon - 105.0, wgLat - 35.0);
        let dLon =  this.transformLon(wgLon - 105.0, wgLat - 35.0);
        let radLat = wgLat / 180.0 * this.PI;
        let magic = Math.sin(radLat);
        magic = 1 - this.EE * magic * magic;
        let sqrtMagic = Math.sqrt(magic);
        dLat = (dLat * 180.0) / ((this.A * (1 - this.EE)) / (magic * sqrtMagic) * this.PI);
        dLon = (dLon * 180.0) / (this.A / sqrtMagic * Math.cos(radLat) * this.PI);
        latlng[1] = wgLat + dLat;
        latlng[0] = wgLon + dLon;
        return latlng;
    },

    /**
     * 谷歌坐标转百度坐标
     * @param gg_lat
     * @param gg_lon
     * @return
     */
     GoogleToBaidu:function(gg_lon, gg_lat) {
        let tmpData = gg_lat;
        gg_lat = gg_lon;
        gg_lon = tmpData;
        let y = gg_lon;
        let x = gg_lat;
        let z = Math.sqrt(x * x + y * y) + 0.00002 * Math.sin(y * this.XPI);
        let theta = Math.atan2(y, x) + 0.000003 * Math.cos(x * this.XPI);
        let bd_lon = z * Math.cos(theta) + 0.0065;
        let bd_lat = z * Math.sin(theta) + 0.006;
        let tmp = [];
        tmp[1] = bd_lat;
        tmp[0] = bd_lon;
        return tmp;
    },

     oriToBaidu:function(lon, lat) {
        /**原始坐标转谷歌坐标*/
        let latlng =  this.oriToGoogle(lon, lat);
        let bdlatlng =  this.GoogleToBaidu(latlng[0], latlng[1]);
        return bdlatlng;
    },


    /**
     * 纬度转换
     * @param x
     * @param y
     * @return
     */
     transformLat:function(x, y) {
        let ret = -100.0 + 2.0 * x + 3.0 * y + 0.2 * y * y + 0.1 * x * y + 0.2 * Math.sqrt(Math.abs(x));
        ret += (20.0 * Math.sin(6.0 * x * this.PI) + 20.0 * Math.sin(2.0 * x * this.PI)) * 2.0 / 3.0;
        ret += (20.0 * Math.sin(y * this.PI) + 40.0 * Math.sin(y / 3.0 * this.PI)) * 2.0 / 3.0;
        ret += (160.0 * Math.sin(y / 12.0 * this.PI) + 320 * Math.sin(y * this.PI / 30.0)) * 2.0 / 3.0;
        return ret;
    },

    /**
     * 经度转换
     * @param x
     * @param y
     * @return
     */
     transformLon:function(x, y) {
        let ret = 300.0 + x + 2.0 * y + 0.1 * x * x + 0.1 * x * y + 0.1 * Math.sqrt(Math.abs(x));
        ret += (20.0 * Math.sin(6.0 * x * this.PI) + 20.0 * Math.sin(2.0 * x * this.PI)) * 2.0 / 3.0;
        ret += (20.0 * Math.sin(x * this.PI) + 40.0 * Math.sin(x / 3.0 * this.PI)) * 2.0 / 3.0;
        ret += (150.0 * Math.sin(x / 12.0 * this.PI) + 300.0 * Math.sin(x / 30.0 * this.PI)) * 2.0 / 3.0;
        return ret;
    },

    // GPSReverseCorrect
    /**
     * 谷歌转原始
     * @param wgLat
     * @param wgLon
     * @return
     */
     googleToOri:function(wgLon, wgLat) {
        let latlng = [];

        let dLat =  this.transformLat(wgLon - 105.0, wgLat - 35.0);
        let dLon =  this.transformLon(wgLon - 105.0, wgLat - 35.0);
        let radLat = wgLat / 180.0 * this.PI;
        let magic = Math.sin(radLat);
        magic = 1 - this.EE * magic * magic;
        let sqrtMagic = Math.sqrt(magic);
        dLat = (dLat * 180.0) / ((this.A * (1 - this.EE)) / (magic * sqrtMagic) * this.PI);
        dLon = (dLon * 180.0) / (this.A / sqrtMagic * Math.cos(radLat) * this.PI);
        latlng[1] = wgLat - dLat;
        latlng[0] = wgLon - dLon;

        return latlng;
    },

    /**
     * 百度转谷歌
     * @param gg_lat
     * @param gg_lon
     * @return
     */
     baiduToGoogle:function(gg_lon, gg_lat) {
        let tmpData = gg_lon;
        gg_lon = gg_lat;
        gg_lat = tmpData;
        let y = gg_lon;
        let x = gg_lat;

        let z = Math.sqrt(x * x + y * y) - 0.00002 * Math.sin(y * this.XPI);
        let theta = Math.atan2(y, x) - 0.000003 * Math.cos(x * this.XPI);
        let bd_lon = z * Math.cos(theta) - 0.0065;
        let bd_lat = z * Math.sin(theta) - 0.006;
        let tmp = [];
        tmp[1] = bd_lat;
        tmp[0] = bd_lon;
        return tmp;
    },

    /**
     * 百度转原始
     * @param lat
     * @param lon
     * @return
     */
     baiduToOri:function(lon, lat) {
        /**百度转谷歌*/
        let bdlatlng =  this.baiduToGoogle(lon, lat);
        /**谷歌坐标转原始坐标*/
        let latlng =  this.googleToOri(bdlatlng[0], bdlatlng[1]);
        return latlng;
    },


    // 原有代码
    /**
     * 获取经纬度之间的距离
     * @param {any} lat1 纬度1
     * @param {any} lng1 经度1
     * @param {any} lat2 纬度2
     * @param {any} lng2 经度2
     * @returns {any} d (km)
     */
    EARTH_RADIUS: 6371,//km

    Distance:function(lat1, lng1, lat2, lng2) {

        var p1 = this.LonLatToWorld(lat1, lng1,);
        var p2 = this.LonLatToWorld(lat2, lng2);
        var angle = Math.acos(this.CalcVCos(p1.x, p1.y, p1.z, p2.x, p2.y, p2.z));
        var dis = this.EARTH_RADIUS * angle
        return dis;
    },

    CalcVCos:function(x1, y1, z1, x2, y2, z2) {
        var l1 = Math.sqrt(x1 * x1 + y1 * y1 + z1 * z1);
        var l2 = Math.sqrt(x2 * x2 + y2 * y2 + z2 * z2);
        var result = (x1 * x2 + y1 * y2 + z1 * z2) / (l1 * l2);
        if (result > 0.99999999999999 && result < 1) result = 1;
        if (result < -1) result = -1;
        if (result > 1) result = 1;
        return result;

    },
    /**
     * 地理坐标转空间坐标
     * @param {any} lng
     * @param {any} lat
     */
    LonLatToWorld:function(lat, lng) {
        //弧度计算
        var rLat = lat * Math.PI / 180.0;
        var rLng = lng * Math.PI / 180.0;
        var radiusCosLat = this.EARTH_RADIUS * Math.cos(rLat);
        var x = radiusCosLat * Math.sin(rLng);
        var y = (this.EARTH_RADIUS * Math.sin(rLat));
        var z = (radiusCosLat * Math.cos(rLng));
        z = -z;
        return {x, y, z};
    },


    // backup
    // GPSCorrect
    /**
     * 原始坐标转谷歌坐标
     * @param wgLat
     * @param wgLon
     * @return
     */
     oriToGoogle_backup:function(wgLat, wgLon) {
        let latlng = [];
        let dLat =  this.transformLat(wgLon - 105.0, wgLat - 35.0);
        let dLon =  this.transformLon(wgLon - 105.0, wgLat - 35.0);
        let radLat = wgLat / 180.0 * this.PI;
        let magic = Math.sin(radLat);
        magic = 1 - this.EE * magic * magic;
        let sqrtMagic = Math.sqrt(magic);
        dLat = (dLat * 180.0) / ((this.A * (1 - this.EE)) / (magic * sqrtMagic) * this.PI);
        dLon = (dLon * 180.0) / (this.A / sqrtMagic * Math.cos(radLat) * this.PI);
        latlng[0] = wgLat + dLat;
        latlng[1] = wgLon + dLon;
        return latlng;
    },
    /**
     * 谷歌坐标转百度坐标
     * @param gg_lat
     * @param gg_lon
     * @return
     */
     GoogleToBaidu_backup:function(gg_lat, gg_lon) {
        let x = gg_lon;
        let y = gg_lat;
        let z = Math.sqrt(x * x + y * y) + 0.00002 * Math.sin(y * this.XPI);
        let theta = Math.atan2(y, x) + 0.000003 * Math.cos(x * this.XPI);
        let bd_lon = z * Math.cos(theta) + 0.0065;
        let bd_lat = z * Math.sin(theta) + 0.006;
        let tmp = [];
        tmp[0] = bd_lat;
        tmp[1] = bd_lon;
        return tmp;
    },

     oriToBaidu_backup:function(lat, lon) {
        /**原始坐标转谷歌坐标*/
        let latlng =  this.oriToGoogle_backup(lat, lon);
        let bdlatlng =  this.GoogleToBaidu_backup(latlng[0], latlng[1]);
        return bdlatlng;
    },

    // GPSReverseCorrect
    /**
     * 谷歌转原始
     * @param wgLat
     * @param wgLon
     * @return
     */
     googleToOri_backup:function(wgLat, wgLon) {
        let latlng = [];
        let dLat =  this.transformLat(wgLon - 105.0, wgLat - 35.0);
        let dLon =  this.transformLon(wgLon - 105.0, wgLat - 35.0);
        let radLat = wgLat / 180.0 * this.PI;
        let magic = Math.sin(radLat);
        magic = 1 - this.EE * magic * magic;
        let sqrtMagic = Math.sqrt(magic);
        dLat = (dLat * 180.0) / ((this.A * (1 - this.EE)) / (magic * sqrtMagic) * this.PI);
        dLon = (dLon * 180.0) / (this.A / sqrtMagic * Math.cos(radLat) * this.PI);
        latlng[0] = wgLat - dLat;
        latlng[1] = wgLon - dLon;
        return latlng;
    },
    /**
     * 百度转谷歌
     * @param gg_lat
     * @param gg_lon
     * @return
     */
     baiduToGoogle_backup:function(gg_lat, gg_lon) {
        let x = gg_lon;
        let y = gg_lat;
        let z = Math.sqrt(x * x + y * y) - 0.00002 * Math.sin(y * this.XPI);
        let theta = Math.atan2(y, x) - 0.000003 * Math.cos(x * this.XPI);
        let bd_lon = z * Math.cos(theta) - 0.0065;
        let bd_lat = z * Math.sin(theta) - 0.006;
        let tmp = [];
        tmp[0] = bd_lat;
        tmp[1] = bd_lon;
        return tmp;
    },
    /**
     * 百度转原始
     * @param lat
     * @param lon
     * @return
     */
     baiduToOri_backup:function(lat, lon) {
        /**百度转谷歌*/
        let bdlatlng =  this.baiduToGoogle_backup(lat, lon);
        /**谷歌坐标转原始坐标*/
        let latlng =  this.googleToOri_backup(bdlatlng[0], bdlatlng[1]);
        return latlng;
    },

}
export default GpsH; 
