
import core from '../core';
import mapFunc from './map';
import {collIcon,markItem,markLayer} from './markLayer.js';
const fillColor = "#1791fc"; //填充色 默认值"#1791fc"
const fillOpacity = 0.35;//填充透明度 默认值0.35
const strokeWeight = 2;//线宽 默认值2
const strokeColor = "#1791fc";//线颜色 默认值"#1791fc"
const strokeOpacity = 0.9;//线透明度 默认值0.9
//地图实例配置对象
const instanceConfig = {
  protocol: 'https',
  host: "webapi.amap.com",
  callback: 'amapOnload',
  key: 'ecd528345198c8dc6f5e399beabe9b42',
  plugin: [
    'AMap.Autocomplete',
    'AMap.PlaceSearch',
    'AMap.Scale',
    'AMap.OverView',
    'AMap.ToolBar',
    'AMap.MapType',
    'AMap.ControlBar',
    'AMap.PolyEditor',
    'AMap.CircleEditor',
    //定位
    'AMap.Geolocation',
    //地理编码
    'AMap.Geocoder',
    //行政区查询
    'AMap.DistrictSearch',
    //自动驾车
    'AMap.Driving',
    //可拖拽
    'AMap.DragRoute',
    //鼠标工具插件
    'AMap.MouseTool',
    //矩形编辑
    'AMap.RectangleEditor',
    //测距
    'AMap.RangingTool',
    //点聚合
    'MarkerClusterer',
    //自定义图层
    "AMap.CustomLayer",
    //轨迹纠偏服务
    "AMap.GraspRoad",
    //热力图
    "AMap.Heatmap",
    "AMap.PlaceSearch",
    //简易行政区图
    "DistrictLayer"
  ],
  // 默认高德 sdk 版本为 1.4.4
  v: '1.4.14',
  uiVersion: '1.0.11'
};


let obj = {
  amap: {},
  geocoder: null,
  mousetool: {},
  rangtool: {},
  dragroute: null,
  satellite: {},
  roadNet: {},
  traffic: {},
  mapTypeDic: {},
  mLayerDic: {},
  scaleDic: {},
  controlBarDic: {},
  overViewDic: {},
  toolBarDic: {},
  patchIOS11Geo() {
    // ios环境切换到使用远程https定位
    if (AMap.UA.ios && document.location.protocol !== 'https:') {
      // 使用远程定位，见 remogeo.js
      var remoGeo = new RemoGeoLocation();
      // 替换方法
      navigator.geolocation.getCurrentPosition = function () {
        return remoGeo.getCurrentPosition.apply(remoGeo, arguments);
      };
      // 替换方法
      navigator.geolocation.watchPosition = function () {
        return remoGeo.watchPosition.apply(remoGeo, arguments);
      };
    }
  },
  create(data) {
    if (data.host) instanceConfig.host = data.host;
    if (data.protocol) instanceConfig.protocol = data.protocol;
    if (data.gdKey) instanceConfig.key = data.gdKey;
    this.loadAMapScript((resData) => {
      if (resData.Code !== 0) {
        console.error("加载Js Api 失败");
        return;
      }
      var $this = this;
      this.district = new AMap.DistrictSearch({
        // 返回行政区边界坐标等具体信息
        extensions: 'all',
        // 设置查询行政区级别为 区，
      });
      if (this.amap[data.id]) {
        delete this.amap[data.id]
        this.clearMapData(data.id)
    }
      let mapData = {
        animateEnable: false
      };
      if (data.mapStyle) mapData.mapStyle = data.mapStyle;
      if (data.viewMode) mapData.viewMode = '3D';
      if (data.pitch) mapData.pitch = data.pitch;
      if (!data.pitchEnable) mapData.pitchEnable = false;
      if (!data.rotateEnable) mapData.rotateEnable = false;
      if (data.zooms) mapData.zooms = data.zooms;
      if (data.mask && data.mask.keywords) {
        let mapMask = [];
        this.keywords = data.mask.keywords;
        this.index = 0;
        let keyMask = {};
        this.drawMaskMap(mapMask, keyMask, this.keywords[0], (mask, keyMask) => {
          mapData.mask = mask;
          var retData = {
            keyMask: keyMask,
            mask: mask
          };
          this.initMapObj(data, mapData, retData);
        });
      } else {
        this.initMapObj(data, mapData);
      }
    });
  },
  clearMapData(id) {
      delete  this.mousetool[id]
      delete this.rangtool[id]
      delete this.satellite[id]
      delete this.mapTypeDic[id]
      delete this.scaleDic[id]
      delete this.controlBarDic[id]
      delete this.overViewDic[id]
      delete this.toolBarDic[id]
  },
  //添加地图引擎script标签
  loadAMapScript(callback) {
    this.loadUIAMap((res) => {
      if (res.Code === 0) {
        if (window.AMap && window.AMap.Map) {
          if (callback) callback({ Code: 0 });
          return;
        };
        const script = document.createElement('script');
        script.type = 'text/javascript';
        script.async = true;
        script.defer = true;
        script.src = this._getScriptSrc();
        //异步高德地图JS API 加载回调
        if (!window['amapOnload']) {
          window['amapOnload'] = () => {
            initAMapUI();
            if (callback) callback({ Code: 0 });
          };
        }
        document.head.appendChild(script);
      } else {
        callback({ Code: -1 });
        console.error("加载AMapUI出了问题！");
      }
    });

  },
  loadUIAMap(callback) {
    if (!instanceConfig.uiVersion || window.AMapUI) { callback({ Code: 0 }); return; };
    const UIScript = document.createElement('script');
    const [versionMain, versionSub, versionDetail] = instanceConfig.uiVersion.split('.');
    if (versionMain === undefined || versionSub === undefined) {
      console.error('amap ui version is not correct, please check! version: ', instanceConfig.uiVersion);
      return;
    }
    let src = `${instanceConfig.protocol}://${instanceConfig.host}/ui/${versionMain}.${versionSub}/main-async.js`;
    if (versionDetail) src += `?v=${versionMain}.${versionSub}.${versionDetail}`;
    UIScript.src = src;
    UIScript.type = 'text/javascript';
    UIScript.async = true;
    document.head.appendChild(UIScript);
    UIScript.onload = () => {
      callback({ Code: 0 });
    };
    UIScript.onerror = () => {
      callback({ Code: -1 });
    };
  },
  _getScriptSrc() {
    // amap plugin prefix reg
    const amap_prefix_reg = /^AMap./;

    const config = instanceConfig;
    const paramKeys = ['v', 'key', 'plugin', 'callback'];

    // check 'AMap.' prefix
    if (config.plugin && config.plugin.length > 0) {
      // push default types
      config.plugin.push('Autocomplete', 'PlaceSearch', 'PolyEditor', 'CircleEditor');

      config.plugin = config.plugin.map(item => {
        return (amap_prefix_reg.test(item)) ? item : 'AMap.' + item;
      });
    }

    const params = Object.keys(config)
      .filter(k => ~paramKeys.indexOf(k))
      .filter(k => config[k] != null)
      .filter(k => {
        return !Array.isArray(config[k]) ||
          (Array.isArray(config[k]) && config[k].length > 0);
      })
      .map(k => {
        let v = config[k];
        if (Array.isArray(v)) return { key: k, value: v.join(',') };
        return { key: k, value: v };
      })
      .map(entry => `${entry.key}=${entry.value}`)
      .join('&');
    return `${instanceConfig.protocol}://${instanceConfig.host}/maps?${params}`;
  },
  //地图初始化时需要执行的
  initMapObj(data, mapData, area) {
    this.amap[data.id] = new AMap.Map(data.id, mapData);
    this.satellite[data.id] = new AMap.TileLayer.Satellite();
    this.roadNet[data.id] = new AMap.TileLayer.RoadNet();
    this.traffic[data.id] = new AMap.TileLayer.Traffic();
    this.traffic[data.id].setMap(this.amap[data.id]);
    this.roadNet[data.id].setMap(this.amap[data.id]);
    this.satellite[data.id].setMap(this.amap[data.id]);
    this.roadNet[data.id].hide();
    this.traffic[data.id].hide();
    this.satellite[data.id].hide();
    //地图缩放监听
    // 在图面添加比例尺控件，展示地图在当前层级和纬度下的比例尺
    this.scaleDic[data.id] = new AMap.Scale();
    this.amap[data.id].addControl(this.scaleDic[data.id]);
    if (data.scale) {
      this.scaleDic[data.id].show();
    } else {
      this.scaleDic[data.id].hide();
    }
    // 在图面添加鹰眼控件，在地图右下角显示地图的缩略图
    this.overViewDic[data.id] = new AMap.OverView();
    this.amap[data.id].addControl(this.overViewDic[data.id] );
    if (data.overview) {
      this.overViewDic[data.id].show();
    } else {
      this.overViewDic[data.id].hide();
    }
    this.toolBarDic[data.id] = new AMap.ToolBar({
      offset: new AMap.Pixel(10, 60),
    });
    this.amap[data.id].addControl(this.toolBarDic[data.id]);
    // 在图面添加类别切换控件，实现默认图层与卫星图、实施交通图层之间切换的控制
    if (data.toolbar) {
      // 在图面添加工具条控件，工具条控件集成了缩放、平移、定位等功能按钮在内的组合控件
      this.toolBarDic[data.id].show();
    } else {
      this.toolBarDic[data.id].hide();
    }

    this.mapTypeDic[data.id] = new AMap.MapType();
    this.amap[data.id].addControl(this.mapTypeDic[data.id]);
    if (data.maptype) {
      this.mapTypeDic[data.id].show();
    } else {
      this.mapTypeDic[data.id].hide();
    }

    this.controlBarDic[data.id] = new AMap.ControlBar();
    if (data.controlBar) {
      this.amap[data.id].addControl(this.controlBarDic[data.id]);
      this.controlBarDic[data.id].show();
    } else {
      //this.amap[data.id].removeControl(this.controlBarDic[data.id]);
    }
    //地理编码
    this.geocoder = new AMap.Geocoder();

    if (data.callback) {
      data.callback(area)
    }
  },
  drawMaskMap(mask, keyMask, key, callback) {
    this.district.search(key, (status, info) => {
      if (status == "complete") {
        let bounds = info.districtList[0].boundaries;
        keyMask[key] = bounds;
        for (let j = 0; j < bounds.length; j++) {
          mask.push([bounds[j]]);
        }
      }
      let next = this.keywords[this.index++];
      if (next) {
        this.drawMaskMap(mask, keyMask, next, callback);
      } else {
        if (callback) callback(mask, keyMask);
      }
    });
  },
  showToolBar(mapId) {
    if (this.toolBarDic[mapId]) {
      this.toolBarDic[mapId].show();
    } else {
      console.error("toolBar未初始化");
    }
  },
  showMapType(mapId) {
    if (this.mapTypeDic[mapId]) {
      this.mapTypeDic[mapId].show();
    } else {
      console.error("mapType未初始化");
    }
  },
  hideToolBar(mapId) {
    if (this.toolBarDic[mapId]) {
      this.toolBarDic[mapId].hide();
    } else {
      console.error("toolBar未初始化");
    }
  },
  hideMapType(mapId) {
    if (this.mapTypeDic[mapId]) {
      this.mapTypeDic[mapId].hide();
    } else {
      console.error("mapType未初始化");
    }
  },
  showOverView(mapId) {
    if (this.overViewDic[mapId]) {
      this.overViewDic[mapId].show();
    } else {
      console.error("overView未初始化");
    }
  },
  showScale(mapId) {
    if (this.scaleDic[mapId]) {
      this.scaleDic[mapId].show();
    } else {
      console.error("scale未初始化");
    }
  },
  hideOverView(mapId) {
    if (this.overViewDic[mapId]) {
      this.overViewDic[mapId].hide();
    } else {
      console.error("overView未初始化");
    }
  },
  hideScale(mapId) {
    if (this.scaleDic[mapId]) {
      this.scaleDic[mapId].hide();
    } else {
      console.error("scale未初始化");
    }
  },
  //设置地图风格
  setMapStyle(style, id) {
    var map = this.getMapObj({ id: id }).currentObj;
    let styleStr = 'amap://styles/' + style;
    map.setMapStyle(styleStr);
  },
  setFeatures(arr, id) {
    var map = this.getMapObj({ id: id }).currentObj;
    map.setFeatures(arr);
  },
  getFeatures(id) {
    var map = this.getMapObj({ id: id }).currentObj;
    return map.getFeatures();
  },
  //设置俯仰角,3D视图有效
  setPitch(Number, id) {
    var map = this.getMapObj({ id: id }).currentObj;
    map.setPitch(Number);
  },
  zoomIn(id) {
    var map = this.getMapObj({ id: id }).currentObj;
    map.zoomIn();
  },
  zoomOut(id) {
    var map = this.getMapObj({ id: id }).currentObj;
    map.zoomOut();
  },
  //卫星图层
  satelliteShow(data) {
    this.satellite[data.id].show();
  },
  //路网图层
  roadNetShow(data) {
    this.roadNet[data.id].show();
  },
  //路况图层
  trafficShow(data) {
    this.traffic[data.id].show();
  },
  //卫星图层
  satelliteHide(data) {
    this.satellite[data.id].hide();
  },
  //路网图层
  roadNetHide(data) {
    this.roadNet[data.id].hide();
  },
  //路况图层
  trafficHide(data) {
    this.traffic[data.id].hide();
  },
  //获取两点间距离
  distance(data) {
    var p = [];
    for (var i = 0; i < data.length; i++) {
      if (!data[i].lng || !data[i].lat) {
        p.push(data[i]);
      } else {
        p.push([data[i].lng, data[i].lat]);
      }
    }
    var dis = AMap.GeometryUtil.distance(p[0], p[1]);
    return dis;
  },
  //行政区查询实例
  districtSearch(data) {
    var inst = {};

    var district = new AMap.DistrictSearch({
      // 返回行政区边界坐标等具体信息
      extensions: 'all',
      // 设置查询行政区级别为 区
      level: data.level ? data.level : 'district',
      subdistrict: data.subdistrict ? data.subdistrict : '1',
      showbiz: data.showbiz !== undefined ? data.showbiz : true,
    });
    inst.obj = district;
    inst.search = function (keywords, callback) {
      inst.obj.search(keywords, (status, res) => {
        var resInfo = {};
        if (status == "complete") {
          resInfo.Info = res.Info;
          resInfo.districtList = res.districtList;
        } else {
          resInfo.Info = "";
          resInfo.districtList = null;
        }
        callback(status, resInfo);
      })
    };
    inst.setLevel = function (level) {
      //设置关键字对应的行政区级别或商圈，可选值：
      // country：国家
      //province：省 / 直辖市
      // city：市
      //district：区 / 县
      //biz_area：商圈
      inst.obj.setLevel(level);
    }
    inst.setSubdistrict = function (district) {
      //设置下级行政区级数（行政区级别包括：国家、省/直辖市、市、区/县4个级别），商圈为区/县下一级，默认值：1
      //可选值：0、1、2、3
      //0：不返回下级行政区；
      //1：返回下一级行政区；
      //2：返回下两级行政区；
      //3：返回下三级行政区
      inst.obj.setSubdistrict(district);
    }
    return inst;
  },
  //数组path处理
  getPoints(data) {
    var lnglat = [];
    if (core.isNull(data.length) && data.currentObj) {
      return [data.currentObj];
    }
    for (let i = 0; i < data.length; i++) {
      if (Array.isArray(data[i])) {
        lnglat.push([data[i][0], data[i][1]]);
      } else if (typeof (data[i]) == "object") {
        if (data[i].currentObj) {
          lnglat.push(data[i].currentObj);
        } else {
          lnglat.push(data[i]);
        }
      } else if (typeof (data[i]) != "object") {
        lnglat = [[data[0], data[1]]];
        break;
      }
    }
    return lnglat;
  },
  setPoints(arr) {
    var poi = []
    for (var i = 0; i < arr.length; i++) {
      poi.push({
        currentObj: arr[i],
        lng: arr[i].lng,
        lat: arr[i].lat,
      })
    }
    return poi;
  },
  CreatePolyEditor(data) {
    let polygon = null;
    let obj = null;
    var map = this.getMapObj(data).currentObj;
    if (data.shape) {
      if (data.shape.currentObj) polygon = data.shape.currentObj;
      else polygon = data.shape;
      let polyEditor = new AMap.PolyEditor(map, polygon);
     obj = { currrentObj: polyEditor };
      obj.open = () => {
        obj.currrentObj.open();
      }
      obj.close = () => {
        obj.currrentObj.close();
      }
      obj.currrentObj.on("addnode", (res) => {
        if (data.addnodeFn) {
          data.addnodeFn(res)
        }
      })
      obj.currrentObj.on("adjust", (res) => {
        if (data.adjustFn) {
          data.adjustFn(res)
        }
      })
      obj.currrentObj.on("removenode", (res) => {
        if (data.removenodetFn) {
          data.removenodeFn(res)
        }
      })
      obj.currrentObj.on("end", (res) => {
        if (data.endFn) {
          let tar = { currrentObj: res.target };
          tar.getPath = () => {
            return obj.currentObj.getPath();
          }
          tar.setPath = (arr) => {
            let pathData = [];
            for (var i = 0; i < arr.length; i++) {
              if (arr[i].currentObj) {
                pathData.push(arr[i].currentObj)
              } else {
                pathData.push(arr[i]);
              }
            }
            obj.currentObj.setPath(pathData)
          }
          this.formateOverlay(tar)
          let ret = {
            type: res.type,
            target: tar
          }
          data.endFn(ret)
        }
      })
      
    } else {
      console.error("shape is undefined");
    }
    return obj;
  },
  //矩形编辑器
  CreateRectangleEditor(data) {
    var map = this.getMapObj(data).currentObj;
    let rec = null;
    let obj = null;
    if (data.shape) {
      if (data.shape.currentObj) rec = data.shape.currentObj;
      else rec = data.shape;
      let recEdit = new AMap.RectangleEditor(map, rec)
      obj = { currrentObj: recEdit };
      obj.open = () => {
        obj.currrentObj.open();
      }
      obj.close = () => {
        obj.currrentObj.close();
      }
   
      obj.currrentObj.on("adjust", (res) => {
        if (data.adjustFn) {
          data.adjustFn()
        }
      })
      obj.currrentObj.on("end", (res) => {
        if (data.endFn) {
          let tar = { currrentObj: res.target };
          this.formateOverlay(tar)
          let ret = {
            type: res.type,
            target: tar
          }
          data.endFn(ret)
        }
      })
    } else {
      console.error("shape is undefined");
    }
    return obj;
  },
  //圆形编辑器
  CreateCircleEditor(data) {
    var map = this.getMapObj(data).currentObj;
    let circle = null;
    let obj = null;
    if (data.shape) {
      if (data.shape.currentObj) circle = data.shape.currentObj;
      else circle = data.shape;
      let cirEdit = new AMap.CircleEditor(map, circle);
      obj = { currrentObj: cirEdit };
      obj.open = () => {
        obj.currrentObj.open();
      }
      obj.close = () => {
        obj.currrentObj.close();
      }

      obj.currrentObj.on("adjust", (res) => {
        if (data.adjustFn) {
          data.adjustFn(res)
        }
      })
      obj.currrentObj.on("move", (res) => {
        if (data.moveFn) {
          data.moveFn(res)
        }
      })
      obj.currrentObj.on("end", (res) => {
        if (data.endFn) {
          let tar = { currrentObj: res.target };
          this.formateOverlay(tar)
          let ret = {
            type: res.type,
            target: tar
          }
          data.endFn(ret)
        }
      })
    } else {
      console.error("shape is undefined");
    }
    return obj;
  },
  convertFrom(gps, type, callback) {
    type = type ? type : "gps";
    AMap.convertFrom(gps, type, function (status, result) {
      if (result.info === 'ok') {
        let locs = [];
        let lnglats = result.locations; // Array.<LngLat>
        for (let i = 0; i < lnglats.length; i++) {
          locs.push([lnglats[i].lng, lnglats[i].lat]);
        }
        if (callback) {
          callback(locs)
        }
      } else {
        callback([])
      }
    });
  },
  getMapObj(data) {
    var instance = null;
    if (data && data.id !== undefined) {
      instance = {
        currentObj: this.amap[data.id]
      };
    } else {
      instance = {
        currentObj: this.amap["mapBox"]
      };
    }
    instance.getSize = function () {
      return instance.currentObj.getSize();
    }
    instance.getBounds = function () {
      var bounds = instance.currentObj.getBounds();
      var obj = {
        currentObj: bounds,
        northeast: bounds.northeast,
        southwest: bounds.southwest
      }
      return obj;
    }
    instance.setBounds = function (bounds) {
      var b = bounds.currentObj ? bounds.currentObj : bounds;
      instance.currentObj.setBounds(b);
    }
    instance.lngLatToContainer = function (lnglat) {
      var pos = null;
      if (lnglat.currentObj) {
        pos = instance.currentObj.lngLatToContainer(lnglat.currentObj);
      } else {
        pos = instance.currentObj.lngLatToContainer(lnglat);
      }
      return pos;
    }
    /**
   * 设置地图上显示的元素种类，支持bg（地图背景）、point（兴趣点）、
        road（道路）、building（建筑物）
   * @param {any} arr
   */
    instance.setFeatures = function (arr) {
      instance.currentObj.setFeatures(arr);
    }

    instance.getFeatures = function () {
      return instance.currentObj.getFeatures();
    }
    return instance;
  },
  //摧毁地图
  destroy(id) {
    var map = this.getMapObj({ id: id }).currentObj;
    if (map) {
      map.destroy();
    }
  },

  clearMap(id) {
    var map = this.getMapObj({ id: id }).currentObj;
    map.clearMap();
  },

  add(data, id) {
    if (data) {
      var map = this.getMapObj({ id: id }).currentObj;
      if (Array.isArray(data)) {
        var arr = [];
        for (var i = 0; i < data.length; i++) {
          arr.push(data[i].currentObj);
        }
        map.add(arr);
      } else {
        map.add(data.currentObj);
      }
      //map.setFitView();
    } else {
      throw new Error("data is undefined");
      return "";
    }
  },
  //移除覆盖物数组
  remove(arr, id) {
    var overlay = [];
    var data = { id: id };
    var map = this.getMapObj(data).currentObj;
    if (Array.isArray(arr)) {
      for (var i = 0; i < arr.length; i++) {
        if (arr[i].currentObj) {
          overlay.push(arr[i].currentObj);
        } else {
          overlay.push(arr[i]);
        }
      }
    } else {
      if (arr.currentObj) {
        overlay = arr.currentObj;
      } else {
        overlay = arr;
      }
    }
    map.remove(overlay);
  },

  //设置中心城市
  setCity(name, callback, id) {
    let data = { id: id };
    let map = this.getMapObj(data).currentObj;
    map.setCity(name, () => {
      if (callback) {
        callback();
      }
    });
  },
  panBy(num1, num2) {
    let map = this.getMapObj(data).currentObj;
    map.panBy(num1, num2);
  },
  panTo(lnglat) {
    let map = this.getMapObj(data).currentObj;
    if (lnglat.currentObj) {
      map.panTo(lnglat.currentObj);
    } else {
      map.panTo(lnglat);
    }
  },
  //设置中心点
  zoomAndCenter(zoom, center, id) {
    var data = { id: id };
    var map = this.getMapObj(data).currentObj;
    if (center.currentObj) {
      map.setZoomAndCenter(zoom, center.currentObj);
    } else {
      map.setZoomAndCenter(zoom, center);
    }
  },
  setZoom(num, id) {
    var data = { id: id };
    var map = this.getMapObj(data).currentObj;
    map.setZoom(num);
  },
  setCenter(center, id) {
    var data = { id: id };
    var map = this.getMapObj(data).currentObj;
    if (center.currentObj) {
      map.setCenter(center.currentObj);
    } else {
      map.setCenter(center);
    }
  },
  //正向地理编码
  getLocation(address, callback) {
    //address:String
    this.geocoder.getLocation(address, function (status, res) {
      if (callback) {
        var data = {};
        data["geocodes"] = [];
        if (status == "complete") {
          data["info"] = res.info;
          data["resultNum"] = res.resultNum;
          for (let i = 0; i < res.geocodes.length; i++) {
            if (!res.geocodes[i]) {
              data["geocodes"][i] = null;
            } else {
              data["geocodes"][i] = {};
              data["geocodes"][i]["formattedAddress"] = res.geocodes[i].formattedAddress;
              data["geocodes"][i]["adcode"] = res.geocodes[i].adcode;
              data["geocodes"][i]["level"] = res.geocodes[i].level;
              data["geocodes"][i]["location"] = res.geocodes[i].location;
            }
          }
        } else if (status == "error") {
          data["info"] = res;
          data["resultNum"] = 0;
        } else {
          data["info"] = "检索结果为0";
          data["resultNum"] = 0;
        }
        callback(status, data);
      }
    });
  },
  //获取地址 逆向地理编码
  getAddress(lnglat, callback) {
    var poi = null;
    if (Array.isArray(lnglat)) {
      poi = [];
      for (let i = 0; i < lnglat.length; i++) {
        let item = lnglat[i];
        let p = item.currentObj ? item.currentObj : item;
        poi.push(p);
      }
    } else {
      poi = lnglat.currentObj ? lnglat.currentObj : lnglat;
    }

    this.geocoder.getAddress(poi, function (status, res) {
      if (callback) {
        var data = {};
        if (status == "complete") {
          data["info"] = res.info;
          if (!res["regeocode"]) {
            let regs = res["regeocodes"];
            data["regeocodes"] = [];
            for (let i = 0; i < regs.length; i++) {
              let item = {
                formattedAddress: regs[i]['formattedAddress'],
                addressComponent: regs[i]['addressComponent'],
              };
              data["regeocodes"].push(item);
            }
          } else

          {
            let regs = res["regeocode"];
            data["regeocode"] = {};
            data["regeocode"]['formattedAddress'] = regs['formattedAddress'];
            data["regeocode"]["addressComponent"] = regs["addressComponent"];
          }
        } else if (status == "error") {
          data["info"] = res;
        } else {
          data["info"] = "检索结果为0";
        }
        callback(status, data);
      }
    });
  },

  //获取地图中心
  getCenter(id) {
    let data = { id: id };
    let map = this.getMapObj(data).currentObj;
    let currentObj = map.getCenter();
    let ret = { currentObj };
    ret.lng = currentObj.lng;
    ret.lat = currentObj.lat;
    return ret;
  },
  //获取地图zoom
  getZoom(id) {
    var data = { id: id };
    var map = this.getMapObj(data).currentObj;
    return map.getZoom();
  },
  //创建坐标点
  lngLat(lng, lat) {
    let pnt = {};
    var currentObj = new AMap.LngLat(lng, lat);
    pnt.currentObj = currentObj;
    pnt.lng = currentObj.lng;
    pnt.lat = currentObj.lat;
    pnt.offset = (w, s) => { return currentObj.offset(w, s) };
    pnt.distance = (gt) => { return currentObj.distance(gt); }
    return pnt;
  },
  newPixel(data) {
    var pixel = {}
    var currentObj = new AMap.Pixel(data.x, data.y);
    pixel.currentObj = currentObj;
    pixel.x = currentObj.x;
    pixel.y = currentObj.y;
    return pixel;
  },
  //构造地点查询类
  newPlaceSearch(data) {
    var map = this.getMapObj(data).currentObj;
    var currentObj = new AMap.PlaceSearch({
      map: map
    });
    var placeSearch = {};
    placeSearch.currentObj = currentObj;
    placeSearch.setCity = function (str) {
      placeSearch.currentObj.setCity(str);
    }
    placeSearch.search = function (str, callback) {
      placeSearch.currentObj.search(str, (status, result) => {
        if (callback) {
          callback(status, result);
        }
      });
    }
    return placeSearch;
  },
  //创建mark
  newMark(data) {
    var $this = this;
    var mark = {};
    var dconfig = {
      position: [data.lng, data.lat],
      icon: data.icon ? $this.getObjOrgion(data.icon) : "",
      content: data.content ? data.content : "",
      anchor: data.anchor ? data.anchor : 'top-left',
      offset: data.offset ? data.offset : $this.newPixel({ x: -10, y: -34 }).currentObj,
      label: data.label ? data.label : {},
      title: data.title ? data.title : "",
      autoRotation: data.autoRotation ? true : false,
      angle: data.angle ? data.angle : 0,
      draggable: data.draggable ?true : false
    }
    if (data.extData) dconfig.extData = data.extData;
    var currentObj = new AMap.Marker(dconfig);
    mark.currentObj = currentObj;
    mark.getOffset = function () {
      var offset = {}
      var offsetObj = currentObj.getOffset();
      offset.currentObj = offsetObj;
      offset.x = offsetObj.x;
      offset.y = offsetObj.y;
      return offset;
    }
    mark.setOffset = function (data) {
      let offset = data;
      if (offset.currrentObj) offset = offset.currrentObj;
      currentObj.setOffset(offset);
    }
    mark.getPosition = function () {
      var poi = {};
      var poiObj = currentObj.getPosition();
      poi.currentObj = poiObj;
      poi.lng = poiObj.lng;
      poi.lat = poiObj.lat;
      return poi;
    }
    let _this = this;
    mark.setPosition = function (data) {
      let lnglat = data;
      if (!Array.isArray(data)) {
        lnglat = data.currentObj ? data.currentObj : lnglat;
      }
      currentObj.setPosition(lnglat)
    }
    mark.setIcon = function (str) {
      currentObj.setIcon(_this.getObjOrgion(str));
    }
    mark.setLabel = function (str) {
      currentObj.setLabel(str);
    }
    mark.setAngle = function (num) {
      currentObj.setAngle(num);
    }
    mark.setzIndex = function (num) {
      mark.currentObj.setzIndex(num);
    }
    mark.getzIndex = function () {
      return mark.currentObj.getzIndex();
    }
    mark.hide = function () {
      mark.currentObj.hide();
    }
    mark.show = function () {
      mark.currentObj.show();
    }
    mark.setCursor = function (str) {
      mark.currentObj.setCursor(str);
    }
    mark.setTop = function (bool) {
      mark.currentObj.setTop(bool);
    }
    mark.getTop = function () {
      return mark.currentObj.getTop();
    }
    mark.getAngle = function () {
      return currentObj.getAngle();
    }
    mark.setDraggable = function (value) {
      return currentObj.setDraggable(value);
    }
    mark.getDraggable = function () {
      return currentObj.getDraggable();
    }
    //mark.setLabel
    //mark.getLabel
    //mark.setIcon
    //mark.getIcon
    //mark.getContent
    //mark.setContent
    //mark.moveAlong
    //mark.moveTo
    //mark.stopMove
    //mark.pauseMove
    //mark.resumeMove
    //mark.setMap
    //mark.getMap
    //mark.setTitle
    //mark.getTitle
    return mark;
  },
  getObjOrgion(obj) {
    if (!obj) return obj;
    return obj.currentObj ? obj.currentObj : obj;
  },
  MapIcon(data) {
    var _this = this;
    let icon = {};
    let opt = {
      // 图标尺寸
      size: data.size ? _this.getObjOrgion(data.size) : new AMap.Size(36, 36),
      // 图标的取图地址
      image: data.image ? data.image : '',
      // 图标所用图片大小
      imageSize: data.imageSize ? _this.getObjOrgion(data.imageSize) : new AMap.Size(36, 36),
    }
    if (data.imageOffset) { opt.imageOffset = _this.getObjOrgion(data.imageOffset) } // 图标取图偏移量
    icon.currentObj = new AMap.Icon(opt);
    return icon;
  },
  MapSize(x, y) {
    var size = {};
    size.currentObj = new AMap.Size(x, y);
    return size;
  },
  setMarkPosition(data) {
    var lnglat = data.lnglat.currentObj ? data.lnglat.currentObj : data.lnglat;
    data.obj.currentObj.setPosition(lnglat);
  },
  getMarkPosition(obj) {
    return { currentObj: obj.currentObj.getPosition() };
  },
  getMarkLabel(obj) {
    return obj.currentObj.getLabel();
  },
  setMarkLabel(data) {
    data.obj.currentObj.setLabel(data.label);
  },
  setMarkOffset(data) {
    data.obj.currentObj.setOffset(data.pixel);
  },
  //轨迹展示UI组件
  pathSimplifierIns(data, callback) {
    var map = this.getMapObj(data).currentObj;
    var orginData = [];
    var _this = this;
    if (data.data) {
      for (let i = 0; i < data.data.length; i++) {
        orginData.push({
          name: data.data[i].name,
          path: _this.getPoints(data.data[i].path)
        })
      }
    } else {
      orginData = [];
    }
    var renderOptions = {};
    if (data.renderOptions) {
      renderOptions = data.renderOptions;
      //处理巡航器的形状样式处理，如果为图片则 需要特殊处理

    } else {
      renderOptions = {
        //轨迹线的样式
        renderAllPointsIfNumberBelow: 100,
        pathLineStyle: {
          strokeStyle: 'red',
          lineWidth: 6,
          dirArrowStyle: true
        }
      }
    }
    AMapUI.load(['ui/misc/PathSimplifier'], function (PathSimplifier) {

      if (!PathSimplifier.supportCanvas) {
        alert('当前环境不支持 Canvas！');
        return;
      }
      //启动页面
      var pathSimplifierIns = new PathSimplifier({
        zIndex: 100,
        map: map,
        data: orginData,
        autoSetFitView: data.autoSetFitView === undefined ? true : data.autoSetFitView,
        getPath: function (pathData, pathIndex) {
          if (data.getPath) {
            data.getPath(pathData, pathIndex);
          } else {
            return pathData.path
          }
        },
        getHoverTitle: function (pathData, pathIndex, pointIndex) {
          if (data.getHoverTitle) {
            return data.getHoverTitle(pathData, pathIndex, pointIndex);
          }
          else {
            //返回鼠标悬停时显示的信息
            if (pointIndex >= 0) {
              //鼠标悬停在某个轨迹节点上
              return pathData.name + '，点:' + (pointIndex + 1) + '/' + pathData.path.length;
            }
            //鼠标悬停在节点之间的连线上
            return pathData.name + '，点数量' + pathData.path.length;
          }
        },
        renderOptions: renderOptions,
        clickToSelectPath: false,//点击轨迹节点或者轨迹节点间的连线时，选中该轨迹，默认true。
        onTopWhenSelected: false,//置顶选中的轨迹线；置顶的含义是，将该轨迹线的zIndex设置为现存最大值+1。默认true。
      });
      var pathObj = {
        currentObj: pathSimplifierIns,
        getPathData: function (index) {
          return pathObj.currentObj.getPathData(index);
        },
        createPathNavigator: function (pathIndex, config) {

          //巡航器样式设置时，图标的兼容
          if (config.pathNavigatorStyle && config.pathNavigatorStyle.img) {
            config.pathNavigatorStyle.content = PathSimplifier.Render.Canvas.getImageContent(config.pathNavigatorStyle.img, () => {
              //图片加载成功，重新绘制一次
              pathSimplifierIns.renderLater();
            }, function onerror(e) { alert('图片加载失败！'); });
          }
          var index = pathIndex ? pathIndex : 0;
          var nav = pathSimplifierIns.createPathNavigator(index, {
            loop: config.loop ? config.loop : false,
            speed: config.speed ? config.speed : 1000,
            autoRotate: true,
            pathNavigatorStyle: config.pathNavigatorStyle ? config.pathNavigatorStyle : {}
          })
          return {
            currentObj: nav,
            start: function (pointIndex) {
              nav.start(pointIndex);
            },
            pause: function () {
              nav.pause();
            },
            resume: function () {
              nav.resume();
            },
            destroy: function () {
              nav.destroy();
            },
            getPosition: function () {
              var p = nav.getPosition();
              var position = _this.lngLat(p.lng, p.lat);
              return position;
            },
            setSpeed: function (speed) {
              nav.setSpeed(speed);
            },
            on: function (eventName, handler) {
              nav.on(eventName, handler);
            },
            off: function (eventName, handler) {
              nav.off(eventName, handler);
            },
          }
        },
        setData: function (data) {
          var arr = [];
          for (let i = 0; i < data.length; i++) {
            arr.push({
              name: data[i].name,
              path: _this.getPoints(data[i].path)
            })
          }
          pathSimplifierIns.setData(data);
        },
        on: function (eventName, handel) {
          pathSimplifierIns.on(eventName, handel);
        },
        clearPathNavigators: function () {
          pathSimplifierIns.clearPathNavigators();
        },
        getSelectedPathIndex: function () {
          return pathSimplifierIns.getSelectedPathIndex();
        },
        getSelectedPathData: function () {
          pathSimplifierIns.getSelectedPathData();
        },
        hide: function () {
          pathSimplifierIns.hide();
        },
        show: function () {
          pathSimplifierIns.show();

        }
      };
      if (callback) {
        callback(pathObj);
      }
    });

  },
  concatJson(newObj, orgin) {
    for (let or in orgin) {
      newObj[or] = orgin[or];
    }
  },
  //海量点
  loadPointSimplifier(data, callback) {
    var map = this.getMapObj(data).currentObj;
    let _this = this;
    AMapUI.load(['ui/misc/PointSimplifier', 'lib/$'], function (PointSimplifier, $) {
      if (!PointSimplifier.supportCanvas) {
        alert('当前环境不支持 Canvas！');
        return;
      }
      let config = {
        map: map, //所属的地图实例
      };
      _this.concatJson(config, data);
      if (config.renderConstructor == 'GroupStyleRender') {
        console.log("采用分组绘制引擎");
        config.renderConstructor = PointSimplifier.Render.Canvas.GroupStyleRender;
      }
      let content = config.renderOptions.pointStyle.content;
      let ret = {};
      let pointSimplifierIns = {};
      if (typeof content == 'object') {
        config.renderOptions.pointStyle.content = PointSimplifier.Render.Canvas.getImageContent(content.url, () => {
          if (content.onload)
            content.onload(ret);
          pointSimplifierIns.renderLater();
        }, (e) => {
          if (content.onerror)
            content.onerror(ret);
          console.error(e);
        });

      }
      let groupStyle = {};
      //分组的content设置
      if (config.renderOptions.groupIdStyleMap) {
        groupStyle = config.renderOptions.groupIdStyleMap;
        for (let key in groupStyle) {
          let content = groupStyle[key].pointStyle.content;
          if (typeof content == 'object') {
            groupStyle[key].pointStyle.content = PointSimplifier.Render.Canvas.getImageContent(content.url, () => {
              if (content.onload)
                content.onload(ret);
              pointSimplifierIns.renderLater();
            }, (e) => {
              if (content.onerror)
                content.onerror(ret);
              console.error(e);
            });
          }
        }
        config.renderOptions.groupStyleOptions = function (gid) {
          return groupStyle[gid]
        }
      }
      pointSimplifierIns = new PointSimplifier(config);

      function onIconLoad() {
        pointSimplifierIns.renderLater();
      }

      function onIconError(e) {
        alert('图片加载失败！');
      }

      ret.currrentObj = pointSimplifierIns;
      //扩展方法
      //立即重新绘制
      ret.render = () => {
        pointSimplifierIns.render();
      }
      //延时设定的毫秒(默认10)后绘制
      ret.renderLater = (arr) => {
        if (arr) pointSimplifierIns.renderLater(arr);
        else pointSimplifierIns.renderLater();
      }
      ret.setData = (data) => {
        pointSimplifierIns.setData(data);
      }
      ret.on = (names, handler) => {
        pointSimplifierIns.on(names, (e, points) => {
          if (handler) handler({ type: e.type }, points);
        });
      }
      ret.hide = () => {
        pointSimplifierIns.hide();
      }
      ret.show = () => {
        pointSimplifierIns.show();
      }
      ret.isHidden = () => {
        return pointSimplifierIns.isHidden();
      }
      //ret.getRender = () => {
      //  return pointSimplifierIns.getRender();
      //}
      ret.getRenderOptions = () => {
        return pointSimplifierIns.getRenderOptions();
      }
      //执行业务代码
      if (callback) {
        callback(ret);
      }
    })
  },
  //轨迹回放 创建图片内容
  getImageContent() {

    PathSimplifier.Render.Canvas.getImageContent
  },
  //轨迹回放
  //移动
  moveAlong(data) {
    var circlable = data.circlable ? data.circlable : false;
    //data.obj.moveAlong(data.lineArr, 500);
    var points = this.getPoints(data.lineArr);
    data.obj.currentObj.moveAlong(points, data.speed, (k) => { if (data.fn) { return data.fn() } else { return k } }, circlable);
  },
  //停止移动
  stopMove(data) {
    data.obj.currentObj.stopMove();
  },
  //暂停移动
  pauseMove(data) {
    data.obj.currentObj.pauseMove();
  },
  //继续移动
  resumeMove(data) {
    data.obj.currentObj.resumeMove();
  },
  //注册地图事件
  on(data) {
    var $this = this;
    data.obj.currentObj.on(data.eventName, (e) => {
      var obj = {};
      if (data.eventName == "click") {
        obj.target = {};
        obj.target.currentObj = e.target;
        obj.type = e.type;
        obj.lnglat = $this.lngLat(e.lnglat.lng, e.lnglat.lat);
      } else if (data.eventName == "zoomend") {
        obj.target = {};
        obj.target.currentObj = e.target;
      } else {
        obj.target = {};
        obj.target.currentObj = e.target;
        obj.type = e.type;
        if (e.lnglat) {
          obj.lnglat = $this.lngLat(e.lnglat.lng, e.lnglat.lat);
        }

      }
      if (data.callback) {
        data.callback(obj);
      }
    });

  },
  //poi搜索
  search(id, callback) {
    //加载PoiPicker
    AMapUI.loadUI(['misc/PoiPicker'], function (PoiPicker) {
      var auto = new PoiPicker({
        input: id,//输入框id
      });
      auto.on("poiPicked", (res) => {
        var data = {};
        data.location = res.item.location;
        data.adcode = res.item.adcode;
        data.district = res.item.district;
        data.address = res.item.address;
        data.name = res.item.name;
        if (callback) {
          callback(data);
        }
      })

    });
  },

  //关闭测距
  closeRangTool(data) {
    let rangtool = this.rangtool[data.id]
    //兼容处理旧版系统默认创建的测距工具，业务在没创建就调用时的报错
    if (!rangtool) {
      console.warn("测距工具未创建，无需关闭");
      return;
    };
    rangtool.turnOff();
  },
  turnOnRangTool(data) {
    let rangtool = this.rangtool[data.id]
    if (!rangtool) {
      throw "测距工具未创建，无法开启，请先使用rangingTool创建"
    }
    rangtool.turnOn();
  },
  //测距,兼容旧版本，第一次创建对象的时候注册事件。新版应该将注册和开关分开
  creatRangTool(data) {
    if (!this.rangtool[data.id]) {
      this.rangtool[data.id] = new AMap.RangingTool(this.amap[data.id]);
      //第一次创建时注册事件，避免旧版本每次打开测距注册重复事件。新版应该调用开关
        this.rangtool[data.id].on("addnode", (res) => {
          if (data.addcallback) {
            data.addcallback(res)
          }
        });
        this.rangtool[data.id].on("removenode", (res) => {
          if (data.delcallback) {
            data.delcallback(res)
          }
        });
        this.rangtool[data.id].on("end", (res) => {
          if (data.endcallback) {
            data.endcallback(res)
          }
        })
    }
    this.turnOnRangTool(data);
    return this.rangtool[data.id];
  },

  //鼠标工具
  getMouseToole() {
    //let mousetool = new AMap.MouseTool(this.amap[data.id]);
    let mousetool = this.mousetool[data.id];
    var obj = { currentObj: mousetool };
    obj.marker = function () {
      mousetool.marker();
    };
    obj.polyline = function () {
      mousetool.polyline();
    };
    obj.polygon = function () {
      mousetool.polygon();
    };
    obj.rectangle = function () {
      mousetool.rectangle();
    };
    obj.circle = function () {
      mousetool.circle();
    };

    obj.measureArea = function () {
      mousetool.measureArea();
    };
    obj.rule = function () {
      mousetool.rule();
    };
    obj.rectZoomOut = function () {
      mousetool.rectZoomOut();
    };
    obj.rectZoomIn = function () {
      mousetool.rectZoomIn();
    };
    return obj;

  },
   //鼠标拉框缩小
  rectZoomOut(data) {
    var _this = this;
    //目前按地图组件id作为唯一键，一个地图组件默认创建一个鼠标工具。可扩展为可创建多个，但需要传入名称标志，便于内部维护。
    if (!this.mousetool[data.id]) throw "该地图没有创建对应的鼠标工具";
    let mousetool = this.mousetool[data.id];
    let config= {
      strokeColor: '#80d8ff',
      fillColor: '#80d8ff',
      fillOpacity: 0.3
      //同 Polygon 的 Option 设置
    }
    if (data.strokeColor) config.strokeColor = data.strokeColor;
    if (data.fillColor) config.fillColor = data.fillColor;
    if (data.fillOpacity) config.fillOpacity = data.fillOpacity;
    mousetool.rectZoomOut(config);
  },
  //鼠标拉框放大
  rectZoomIn(data) {
    var _this = this;
    //目前按地图组件id作为唯一键，一个地图组件默认创建一个鼠标工具。可扩展为可创建多个，但需要传入名称标志，便于内部维护。
    if (!this.mousetool[data.id]) throw "该地图没有创建对应的鼠标工具";
    let mousetool = this.mousetool[data.id];
    let config = {
      strokeColor: '#80d8ff',
      fillColor: '#80d8ff',
      fillOpacity: 0.3
      //同 Polygon 的 Option 设置
    }
    if (data.strokeColor) config.strokeColor = data.strokeColor;
    if (data.fillColor) config.fillColor = data.fillColor;
    if (data.fillOpacity) config.fillOpacity = data.fillOpacity;
    mousetool.rectZoomIn(config);
  },
  //创建鼠标工具 
  openMouseTool(data) {
    var _this = this;
    //目前按地图组件id作为唯一键，一个地图组件默认创建一个鼠标工具。可扩展为可创建多个，但需要传入名称标志，便于内部维护。
    //兼容处理旧版数据,旧版本的开启业务会调用这个接口，防止重复注册事件则第一次创建时才注册事件
    if (!data.key) {
      data.key = data.id;
    }
    if (!this.mousetool[data.key]) {
      let mousetool = new AMap.MouseTool(this.amap[data.id]);
      this.mousetool[data.key] = mousetool;
      if (data.callback) {
        //注册
        this.mousetool[data.key].on("draw", (e) => {
          let drawData = {
            radius: 0,
            points: null,
            type: data.type,
            target: e.obj
          }
          switch (e.obj.CLASS_NAME) {
            case "AMap.Circle":
              drawData.radius = e.obj.getRadius();
              let position = e.obj.getCenter();
              drawData.points = {
                currentObj: position,
                lng: position.lng,
                lat: position.lat,

              };
              break;
            case "AMap.Polygon":
              drawData.points = _this.setPoints(e.obj.getPath());
              drawData.area = e.obj.getArea();
              break;
            case "AMap.Polyline":
              drawData.points = _this.setPoints(e.obj.getPath());
              drawData.length = e.obj.getLength();
          }
          if (data.callback) {
            data.callback(drawData);
          }
        })
      }
    } else {
      //throw "请勿重复初始化，指定Key，或使用openDrawMouseTool";
      //return;
    }
   //创建完鼠标工具的默认状态，true表示默认工具关闭，不传则默认开启，为了兼容旧版直接使用该函数来开启绘制。新模式该函数只用于创建工具开启外部使用openDrawMouseTool
    if(data.defCloseTool) return this.mousetool[data.key];
    this.openDrawMouseTool(data);
    return this.mousetool[data.key];
  },
  //开启鼠标操作
  openDrawMouseTool({ type, id, key,style }) {
    if(!style)style={}
    if (!key) key = id;
    let mousetool = this.mousetool[key];
    if (!mousetool) throw "该地图没有创建对应的鼠标工具";
    switch (type) {
      case 'marker':
        mousetool.marker(style);
        break;
      case 'polyline':
        mousetool.polyline(style);
        break;
      case 'polygon':
        mousetool.polygon(style);
        break;
      case 'rectangle':
        mousetool.rectangle(style);
        break;
      case 'circle':
        mousetool.circle(style);
        break;
      case 'rule':
        mousetool.rule(style);
        break;
      case 'measureArea':
        mousetool.measureArea(style);
        break;
      case 'rectZoomIn':
        mousetool.rectZoomIn(style);
        break;
      case 'rectZoomOut':
        mousetool.rectZoomOut(style);
        break;
    }
  },

  //关闭鼠标工具
  closeMouseTool({ id, key }) {
    if (!key) key = id;
    let mousetool = this.mousetool[key];
    if (!mousetool) throw "地图没有创建对应的鼠标工具或者未传入key";
    mousetool.close();
  },
  // 计算两点间的角度
  calcAngle: function (data, start, end) {
    let map = this.getMapObj(data).currentObj;
    let p_start = map.lngLatToContainer(start),
      p_end = map.lngLatToContainer(end);
    let diff_x = p_end.x - p_start.x,
      diff_y = p_end.y - p_start.y;
    return 360 * Math.atan2(diff_y, diff_x) / (2 * Math.PI) + 90;
  },
  addText(data) {
    let map = this.getMapObj(data).currentObj;
    let TextOptions = {
      text: data.text,
      map: map,
      position: data.position
    }
    if (data.style) TextOptions.style = data.style;
    if (data.offset) TextOptions.offset = data.offset;
    let text = new AMap.Text(TextOptions);
    let ret = {
      currentObj: text,
    };
    ret.setStyle = (style) => {
      text.setStyle(style);
    }
    ret.getText = () => {
      return text.getText();
    }
    ret.setText = (str) => {
      text.setStyle(str);
    }
    ret.hide = () => {
      text.hide();
    }
    ret.show = () => {
      text.show();
    }
    ret.setPosition = (p) => {
      let data = p;
      if (p.currentObj) data = p.currentObj;
      text.setPosition(data);
    }
    ret.getPosition = () => {
      return text.getPosition();
    }
    return ret;
  },
  //覆盖物
  overLay: function (data, colorOpt) {
    if (!colorOpt) colorOpt = {};
    var map = this.getMapObj(data).currentObj;
    var shape = null;
    var $this = this;
    var obj = {};
    //绘制作战线路线图
    if (data.type == "geodesic") {
      let shapes = [];
      let shapesAry = [];
      data.points.forEach((item, index) => {
        let geoConfig = {
          map: map,
          path: item.value,
          ...data.config
        };
        shape = new AMap.Polyline(geoConfig);
        let shapeObj = {
          lineName: item.name,
          shape: shape,
        }
        shapes.push(shape);
        shapesAry.push(shapeObj);
      })
      // 缩放地图到合适的视野级别
      //map.setFitView(shapes);
      return shapesAry;
    };
    var lnglat = this.getPoints(data.points);
    if (data.type == "circle") {
      shape = new AMap.Circle({
        title: data.title ? data.title : "特殊",
        map: map,
        zIndex: data.zIndex,
        center: lnglat[0],  // 圆心位置
        radius: data.radius, // 圆半径
        strokeColor: colorOpt.strokeColor ? colorOpt.strokeColor : strokeColor, //线颜色
        strokeOpacity: colorOpt.strokeOpacity !== undefined ? colorOpt.strokeOpacity : strokeOpacity, //线透明度
        strokeWeight: colorOpt.strokeWeight ? colorOpt.strokeWeight : strokeWeight,    //线宽
        fillColor: colorOpt.fillColor ? colorOpt.fillColor : fillColor, //填充色
        fillOpacity: colorOpt.fillOpacity !== undefined ? colorOpt.fillOpacity : fillOpacity,//填充透明度
        isOutline: colorOpt.isOutline ? colorOpt.isOutline : false,
        outlineColor: colorOpt.outlineColor ? colorOpt.outlineColor : "#000000",
        borderWeight: colorOpt.borderWeight !== undefined ? colorOpt.borderWeight : 1,
        strokeStyle: colorOpt.strokeStyle ? colorOpt.strokeStyle : "solid",
        strokeDasharray: colorOpt.strokeDasharray ? colorOpt.strokeDasharray : [0, 0, 0],
        lineJoin: colorOpt.lineJoin ? colorOpt.lineJoin : 'miter',
        lineCap: colorOpt.lineCap ? colorOpt.lineCap : 'butt',
        draggable: colorOpt.draggable ? colorOpt.draggable : false,
        extData: colorOpt.extData ? colorOpt.extData : {},
        bubble: data.bubble === true ? true : false//是否将覆盖物的鼠标或touch等事件冒泡到地图上
      });
      //map.setZoomAndCenter(12, lnglat[0]);
    }
    else if (data.type == "ellipse") {
      shape = new AMap.Ellipse({
        map: map,
        center: lnglat[0],  // 圆心位置
        radius: data.radius,
        zIndex: data.zIndex,
        strokeColor: colorOpt.strokeColor ? colorOpt.strokeColor : strokeColor, //线颜色
        strokeOpacity: colorOpt.strokeOpacity !== undefined ? colorOpt.strokeOpacity : strokeOpacity, //线透明度
        strokeWeight: colorOpt.strokeWeight ? colorOpt.strokeWeight : strokeWeight,    //线宽
        fillColor: colorOpt.fillColor ? colorOpt.fillColor : fillColor, //填充色
        fillOpacity: colorOpt.fillOpacity !== undefined ? colorOpt.fillOpacity : fillOpacity,//填充透明度
        isOutline: colorOpt.isOutline ? colorOpt.isOutline : false,
        outlineColor: colorOpt.outlineColor ? colorOpt.outlineColor : "#000000",
        borderWeight: colorOpt.borderWeight !== undefined ? colorOpt.borderWeight : 1,
        strokeStyle: colorOpt.strokeStyle ? colorOpt.strokeStyle : "solid",
        strokeDasharray: colorOpt.strokeDasharray ? colorOpt.strokeDasharray : [0, 0, 0],
        lineJoin: colorOpt.lineJoin ? colorOpt.lineJoin : 'miter',
        lineCap: colorOpt.lineCap ? colorOpt.lineCap : 'butt',
        draggable: colorOpt.draggable ? colorOpt.draggable : false,
        extData: colorOpt.extData ? colorOpt.extData : {},
        bubble: data.bubble === true ? true : false//是否将覆盖物的鼠标或touch等事件冒泡到地图上
      }); 
      //map.setZoomAndCenter(12, lnglat[0]);
    }
    else {
      var config = {
        map: map,
        path: lnglat,
        zIndex: data.zIndex,
        strokeColor: colorOpt.strokeColor ? colorOpt.strokeColor : strokeColor, //线颜色
        strokeOpacity: colorOpt.strokeOpacity ? colorOpt.strokeOpacity : strokeOpacity, //线透明度
        strokeWeight: colorOpt.strokeWeight !== undefined ? colorOpt.strokeWeight : strokeWeight,    //线宽
        fillColor: colorOpt.fillColor ? colorOpt.fillColor : fillColor, //填充色
        fillOpacity: colorOpt.fillOpacity !== undefined ? colorOpt.fillOpacity : fillOpacity,//填充透明度
        isOutline: colorOpt.isOutline ? colorOpt.isOutline : false,
        outlineColor: colorOpt.outlineColor ? colorOpt.outlineColor : "#000000",
        borderWeight: colorOpt.borderWeight ? colorOpt.borderWeight : 1,
        strokeStyle: colorOpt.strokeStyle ? colorOpt.strokeStyle : "solid",
        strokeDasharray: colorOpt.strokeDasharray ? colorOpt.strokeDasharray : [0, 0, 0],
        lineJoin: colorOpt.lineJoin ? colorOpt.lineJoin : 'miter',
        lineCap: colorOpt.lineCap ? colorOpt.lineCap : 'butt',
        draggable: colorOpt.draggable ? colorOpt.draggable : false,
        extData: colorOpt.extData ? colorOpt.extData : {},
        bubble: data.bubble === true ? true : false//是否将覆盖物的鼠标或touch等事件冒泡到地图上
      }
      //map.setZoomAndCenter(12, lnglat[0]);
    }
    if (data.type == "polygon" || data.type == "rectangle") {
      shape = new AMap.Polygon(config);
      obj.currentObj = shape;
      obj.getPath = () => {
        return obj.currentObj.getPath();
      }
      obj.setPath = (arr) => {
        let pathData = [];
        for (var i = 0; i < arr.length; i++) {
          if (arr[i].currentObj) {
            pathData.push(arr[i].currentObj)
          } else {
            pathData.push(arr[i]);
          }
        }
        obj.currentObj.setPath(pathData)
      }
    }
    else if (data.type == "polyline") {
      shape = new AMap.Polyline(config);
      obj.currentObj = shape;
      obj.getPath = () => {
        return obj.currentObj.getPath();
      }
      obj.setPath = (arr) => {
        let pathData = [];
        for (var i = 0; i < arr.length; i++) {
          if (arr[i].currentObj) {
            pathData.push(arr[i].currentObj)
          } else {
            pathData.push(arr[i]);
          }
        }
        obj.currentObj.setPath(pathData)
      }
    }
    obj.currentObj = shape;
    this.formateOverlay(obj)
    return obj;
  },
  formateOverlay(obj) {
    obj.setColorOptions = (type, colorOpt) => {
      obj.currentObj.setOptions(colorOpt);
    };
    obj.setDataOptions = (type, data) => {
      let opt = {};
      if (!type) throw new Error("请传入type");
      if (data.points) {
        let lnglat = this.getPoints(data.points);
        if (type == "polygon" || type == "rectangle") {
          opt.path = lnglat;
        } else {
          opt.center = lnglat[0];
        }
      }
      if (data.radius) {
        opt.radius = data.radius;
      }
      obj.currentObj.setOptions(opt);
    };

    obj.setCenter = (lnglat) => {
      if (lnglat.currentObj) lnglat = lnglat.currentObj;
      obj.currentObj.setCenter(lnglat);
    }
    obj.getCenter = () => {
      return obj.currentObj.getCenter();
    }

    obj.setRadius = (num) => {
       obj.currentObj.setRadius(num);
    }
    obj.getRadius = (num) => {
      return obj.currentObj.getRadius(num);
    }
    obj.hide = () => {
      obj.currentObj.hide();
    }
    obj.show = () => {
      obj.currentObj.show();
    }
    obj.getBounds = () => {
      return obj.currentObj.getBounds();
    }
  },
  //绘制线路
  drawRoute(data) {
    var amap = this.getMapObj(data).currentObj;
    if (data.dragroute === undefined) {
      //路线规划没有传递对象时，默认只有一条数据，操作的都是一条数据
      var $this = this;
      var mapData = {};
      if (this.dragroute) {
        this.dragroute.destroy();
      }
      if (data.path) {
        if (data.path.length == 0) {
          path = [];
          mapData.points = [];
          mapData.routeObj = null;
          mapData.start = null;
          mapData.end = null;
          mapData.waypoints = [];
          if (data.completecallback) {
            data.completecallback(mapData);
          }
        }
      }
      let copyPath = core.clone(data.path);
      this.dragroute = new AMap.DragRoute(amap, copyPath, data.type);
      this.dragroute.search();

      this.dragroute.on("complete", obj => {
        mapData.points = mapFunc.getPath(obj.data.start.location, obj.data.waypoints, obj.data.end.location);
        mapData.routeObj = obj.target;
        mapData.start = obj.data.start.location;
        mapData.end = obj.data.end.location;
        var way = obj.data.waypoints;
        mapData.waypoints = [];
        for (var i = 0; i < way.length; i++) {
          mapData.waypoints.push(way[i].location);
        }
        mapData.routesObj = obj.data;
        mapData.routes = this.dragroute.getRoute();
        if (data.completecallback) {
          data.completecallback(mapData);
        }
      })
      this.dragroute.on("addway", function (obj) {
        if (data.addwaycallback) {
          data.addwaycallback(obj);
        }
      })
    }
    else {
      //参数包括对象时，处理的是参数的路线对象
    }
  },
  //驾车线路规划
  driving() {
    let driving = new AMap.Driving({
      // 驾车路线规划策略，AMap.DrivingPolicy.LEAST_TIME是最快捷模式
      policy: AMap.DrivingPolicy.LEAST_TIME
    });
    let obj = {};
    obj.currentObj = driving;
    //使用经纬度获取驾车规划数据
    obj.search = function (config, callback) {
      if (!config.opts) {
        //无途经点
        config.opts = {
          waypoints: []
        };
      } else if (!config.opts.waypoints) {
        //无途经点
        config.opts.waypoints = [];
      }
      if (config.startLngLat && config.endLngLat) {
        //经纬度
        let startLngLat = config.startLngLat.currentObj ? config.startLngLat.currentObj : config.startLngLat;
        let endLngLat = config.endLngLat.currentObj ? config.endLngLat.currentObj : config.endLngLat;
        obj.currentObj.search(startLngLat, endLngLat, config.opts, (status, result) => {
          //status =>complete error no_data
          callback(status, result)
        })
      } else if (config.points) {
        //关键字获取驾车规划数据
        //var points = [
        //    { keyword: '北京市地震局（公交站）', city: '北京' },
        //    { keyword: '亦庄文化园（地铁站）', city: '北京' }
        //]
        console.log(config.points);
        obj.currentObj.search(config.points, (status, result) => {
          //status =>complete error no_data
          callback(status, result)
        })
      }
    }
    return obj;
  },
  //摧毁路线
  destroyRoute(data) {
    if (data.obj) {
      data.obj.destroy();
    } else {
      throw new Error("dstory 的参数不正确");
    }
  },
  //绘制行政区划
  drawDistrict(data) {
    var $this = this;
    var amap = this.getMapObj(data).currentObj;
    this.district.setLevel = data.level;
    this.district.search(data.name + "", (status, result) => {
      if (status == "complete") {
        var bounds = result.districtList[0].boundaries
        var polygons = []
        if (bounds) {
          for (let i = 0, l = bounds.length; i < l; i++) {
            //生成行政区划polygon
            var polygon = new AMap.Polygon({
              map: amap,
              path: bounds[i],
              strokeColor: data.strokeColor ? data.strokeColor : strokeColor, //线颜色
              strokeOpacity: data.strokeOpacity ? data.strokeOpacity : strokeOpacity, //线颜色
              strokeWeight: data.strokeWeight ? data.strokeWeight : strokeWeight, //线颜色
              fillColor: data.fillColor ? data.fillColor : fillColor, //线颜色
              fillOpacity: data.fillOpacity ? data.fillOpacity : fillOpacity, //线颜色
              bubble: data.bubble === true ? true : false, //是否将覆盖物的鼠标或touch等事件冒泡到地图上
            })
            polygons.push(polygon)
          }
          // 地图自适应
          amap.setFitView();
        }
      } else {
        console.error("数据错误", data.name, result);
      }
      if (data.callback) {
        var shape = {};
        shape[data.name] = polygons;
        data.callback(status, result, shape);
      }
    });

  },
  drawDistrictAll(param) {
    var $this = this;
    var amap = this.getMapObj(param).currentObj;
    let total = param.config.data.length
    let num = 0
    let allBouns = []
    var polygons = []
    for (let i in param.config.data) {
      let data = param.config.data[i]
      this.getBounds(data.adcode, data.level).then(res => {
        if (res == '数据错误') {
          param.callback([], res);
          return
        }
        if (res) {
          num++
          allBouns = [...allBouns, ...res];
        }
        if (total == num) {
          for (let k = 0, l = allBouns.length; k < l; k++) {
            //生成行政区划polygon
            var polygon = new AMap.Polygon({
              map: amap,
              path: allBouns[k],
              strokeColor: param.strokeColor ? param.strokeColor : strokeColor, //线颜色
              strokeOpacity: param.strokeOpacity ? param.strokeOpacity : strokeOpacity, //线颜色
              strokeWeight: param.strokeWeight ? param.strokeWeight : strokeWeight, //线颜色
              fillColor: param.fillColor ? param.fillColor : fillColor, //线颜色
              fillOpacity: param.fillOpacity ? param.fillOpacity : fillOpacity, //线颜色
            })
            polygons.push(polygon)
          }
          // 地图自适应
          amap.setFitView();
          if (param.callback) {
            param.callback(polygons);
          }
        }
      });
    }

  },
  //根据行政code，返回经纬度
  getBounds(code, level) {
    this.district.setLevel = level;
    return new Promise((resolve, reject) => {
      this.district.search(code + "", (status, result) => {
        if (status == "complete") {
          var bounds = result.districtList[0].boundaries
          if (bounds) {
            resolve(bounds)
          }
        } else {
          resolve("数据错误", code, result);
        }
      });
    });
  },
  setClusterMarkers(data) {
    var amap = this.getMapObj(data).currentObj;
    data.obj.currentObj.setMarkers(data.datas);
    //amap.setFitView();
  },
  //点聚合
  markerClusterer(data) {
    var amap = this.getMapObj(data).currentObj;
    var $this = this;
    var points = this.getPoints(data.markers);
    var config = {
      gridSize: data.gridSize !== undefined ? data.gridSize : 60,
      maxZoom: data.maxZoom !== undefined ? data.maxZoom : 18,
      zoomOnClick: data.zoomOnClick === false ? false : true,
      renderClusterMarker: data.renderClusterMarker !== undefined ? data.renderClusterMarker : null
    }
    if (data.styles) {
      config.styles = data.styles;
      for (let i = 0; i < config.styles.length; i++) {
        let item = config.styles[i];
        if (item.size)
          item.size = item.size.currentObj ? item.size.currentObj : item.size;
        if (item.offset)
          item.offset = item.offset.currentObj ? item.offset.currentObj : item.offset;
      }
    }

    var cluster = new AMap.MarkerClusterer(amap, points, config);
    //重叠时，默认展开所有
    if (data.gridSize === undefined && data.overlap === false) {
      amap.on("zoomend", (data) => {
        var zoom = Math.floor(amap.getZoom());
        if (zoom >= 18) {
          cluster.setGridSize(0);
        } else {
          if (cluster.getGridSize() == 0) cluster.setGridSize(60);
        }
      })
    }
    cluster.on("click", (res) => {
      if (data.clickcallback) {
        data.clickcallback(res);
      }
    });
    amap.setFitView();
    obj = {};
    obj.currentObj = cluster;
    obj.setMarkers = function (arr) {

    }

    return obj;
  },
  //清除地图上的所有信息窗体
  clearInfoWindow(data) {
    var amap = this.getMapObj(data).currentObj;
    amap.clearInfoWindow();
  },
  //点聚合添加点（单个）
  addClusterMarker(data) {

    let mark = data.marker.currentObj
    if (!mark && Array.isArray(data.marker)) {
      throw Error("1.0版本仅支持传入Mark对象。");
      return;
    }
    data.obj.currentObj.addMarker(mark);

  },
  //点聚合添加点(批量)
  addClusterMarkers(data) {
    data.obj.currentObj.addMarkers(data.markers);
  },

  //清除所有点聚合
  clearClusterMarkers(config) {
    config.obj.currentObj.clearMarkers();
  },
  //移除单个聚合点
  removeClusterMarker(data) {
    if (data.obj) {
      data.obj.currentObj.removeMarker(data.marker.currentObj);
    } else {
      throw new Error("removeClusterMarker参数不正确");
    }
  },
  setClusterData() {
    throw new Error("高德地图1.0版本不支持方法 setClusterData");
  },
  //移除聚合点(批量)
  removeClusterMarkers(data) {
    if (data.obj) {
      data.obj.currentObj.removeMarkers(data.markers);
    } else {
      throw new Error("removeClusterMarker参数不正确");
    }
  },
  //获取聚合数量
  getClustersCount(data) {
    if (data.obj) {
      return data.obj.currentObj.getClustersCount();
    } else {
      throw new Error("getClustersCount参数不正确");
    }
  },
  //信息窗体
  infoWindow(data) {
    let infoWindow = new AMap.InfoWindow({
      content: data.content ? data.content : "内容为空",
      offset: data.offset ? data.offset : new AMap.Pixel(20, -20),
      autoMove: data.autoMove === undefined ? false : data.autoMove,
      isCustom: data.isCustom === undefined ? false : data.isCustom
    })
    infoWindow.on("close", () => {
      if (data.closecallback) {
        data.closecallback();
      }
    })
    infoWindow.on("open", () => {
      if (data.opencallback) {
        data.opencallback();
      }
    })
    infoWindow.on("change", () => {
      if (data.changecallback) {
        data.changecallback();
      }
    })
    let obj = {};
    obj.currentObj = infoWindow;
    return obj;
  },
  simpleInfoWindow(callback) {
    //加载SimpleInfoWindow，loadUI的路径参数为模块名中 'ui/' 之后的部分
    AMapUI.loadUI(['overlay/SimpleInfoWindow'], function (SimpleInfoWindow) {
      var infoWindow = new SimpleInfoWindow({
        infoTitle: '<strong>这里是标题</strong>',
        infoBody: '<p>这里是内容。</p>'
      });
      //显示在map上
      infoWindow.open(map, map.getCenter());
      var obj = {};
      obj.currentObj = infoWindow;
      if (callback) {
        callback(obj);
      }
    });
  },
  //打开信息窗体
  openInfoWindow(data) {
    let amap = this.getMapObj(data).currentObj;
    if (data.position.currentObj) {
      data.obj.currentObj.open(amap, data.position.currentObj);
    } else if (Array.isArray(data.position)) {
      let position = new AMap.LngLat(data.position[0], data.position[1], true);
      data.obj.currentObj.open(amap, position);
    }
  },
  closeInfoWindow(obj) {
    obj.currentObj.close();
  },
  setInfoContent(data) {
    data.obj.currentObj.setContent(data.content);
  },
  getInfoContent(obj) {
    return obj.currentObj.getContent()
  },
  setInfoPosition(data) {
    var lnglat = data.lnglat.currentObj ? data.lnglat.currentObj : data.lnglat;
    data.obj.currentObj.setPosition(lnglat);
  },
  getInfoPosition(obj) {
    return { currentObj: obj.currentObj.getPosition() }
  },
  //自定义图层对象
  customLayer(data) {
    //创建图层对象
    let amap = this.getMapObj(data).currentObj;
    var options = data.options ? data.options : {};
    var customLayer = new AMap.CustomLayer(data.drawObj, {
      map: options.map ? options.map : amap,
      zooms: options.zooms ? options.zooms : [3, 20],
      alwaysRender: options.alwaysRender !== undefined ? options.alwaysRender : true,//缩放过程中是否重绘，复杂绘制建议设为false
      zIndex: options.zIndex ? options.zIndex : 120
    });
    var instance = { currentObj: customLayer };
    instance.setMap = function (map) {
      //map对象
      instance.currentObj.setMap(map);

    }
    instance.hide = function () {
      instance.currentObj.hide();
    }
    instance.show = function () {
      instance.currentObj.show();
    }
    instance.setzIndex = function (number) {
      instance.currentObj.setzIndex(number);
    }
    instance.getzIndex = function () {
      return instance.currentObj.getzIndex();
    }
    instance.setOpacity = function (number) {
      instance.currentObj.setOpacity(number);
    }
    instance.setRender = function (fn) {
      if (typeof fn == "function") {
        instance.currentObj.render = fn;
      } else {
        console.error("参数类型应为 Function");
      }
    }
    return instance;
  },
  //轨迹纠偏服务 绑路
  graspRoad() {
    let grasp = new AMap.GraspRoad()
    let obj = {
      currentObj: grasp,
    }
    obj.driving = function (originPath, callback) {
      if (!error) {
        var newPath = result.data.points;//纠偏后的轨迹
        var distance = result.data.distance;//里程
        callback(newPath, distance);
      } else {
        callback(null, null);
      }
    }
    return obj;
  },
  //定位对象
  getGeoLocation(data) {
    let config = {
      enableHighAccuracy: data.enableHighAccuracy ? data.enableHighAccuracy : true,//是否使用高精度定位，默认:true
      timeout: data.timeout ? data.timeout : 4000,          //超过10秒后停止定位，默认：5s
      buttonPosition: 'RB',    //定位按钮的停靠位置
      noIpLocate: data.noIpLocate ? data.noIpLocate : 0,
      noGeoLocation: data.noGeoLocation ? data.noGeoLocation : 0,
      GeoLocationFirst: data.GeoLocationFirst ? true : false,
    
      useNative: data.useNative ? true : false,
      buttonOffset: new AMap.Pixel(10, 20),//定位按钮与设置的停靠位置的偏移量，默认：Pixel(10, 20)
      zoomToAccuracy: data.zoomToAccuracy ? data.zoomToAccuracy : true   //定位成功后是否自动调整地图视野到定位点
    }
    //默认高德坐标。设置false则为浏览器坐标
    if (data.convert === false) config.convert = false;
    var geolocation = new AMap.Geolocation(config);
    var obj = {
      currentObj: geolocation,
      getCurrentPosition: (callback) => {
        geolocation.getCurrentPosition(function (status, result) {
          if (status == 'complete') {
            var param = {
              info: "成功",
              message: "",
              position: [result.position.lng, result.position.lat],
            }
          } else {
            var param = {
              info: "失败",
              message: result.message,
              position: null
            }
          }
          if (callback) {
            callback(param)
          }
        });
      }
    }
    return obj;
  },
  //调整地图视野
  setFitView(id, data) {
    var map = this.getMapObj({ id: id }).currentObj;
    if (data) {
      let param = null;
      if (Array.isArray(data)) {
        param = this.serArrParam(data);
      } else {
        param = data.currentObj ? data.currentObj : data
      }
      map.setFitView(param);
    } else {
      map.setFitView();
    }

  },
  getHeatmap(data) {
    var amap = this.getMapObj(data).currentObj;
    var obj = {};
    var heatmap = new AMap.Heatmap(amap, {});
    obj.currentObj = heatmap;
    //radius gradient opacity zooms
    //设置热力图属性
    obj.setOptions = (option) => {
      obj.currentObj.setOptions(option);
    }
    //设置热力图展现的数据集
    obj.setDataSet = (dataset) => {

      obj.currentObj.setDataSet(dataset);
    }
    //设置热力图展现的数据集
    obj.setMap = (data) => {
      if (data) {
        data = data.currentObj ? data.currentObj : data;
      }
      obj.currentObj.setMap(data);
    }
    //
    obj.show = () => {
      obj.currentObj.show();
    }
    //
    obj.hide = (dataset) => {
      obj.currentObj.hide();
    }
    return obj;
  },
  setObject3DLayer(data) {
    var amap = this.getMapObj(data).currentObj;
    var obj = {};
    var object3Dlayer = new AMap.Object3DLayer();
    obj.currentObj = object3Dlayer;
    obj.add = (wall) => {
      obj.currentObj.add(wall);
    }
    amap.add(object3Dlayer);
    return obj;
  },
  setObject3DWall(data) {
    //data :object3Dlayer
    var obj = {};
    var wall = new AMap.Object3D.Wall({
      path: data.options.bounds,
      height: data.options.height,
      color: data.options.color
    });
    obj.currentObj = wall;
    data.object3Dlayer.currentObj.add(obj.currentObj);
    return obj;
  },
  //设置省级图层
  getDistrictLayerProvince(data) {
    var obj = {};
    var disProvince = new AMap.DistrictLayer.Province(data);
    obj.currentObj = disProvince;
    obj.setMap = (map) => {
      if (map.currentObj) {
        obj.currentObj.setMap(map.currentObj);
      } else {
        obj.currentObj.setMap(map);
      }
    };
    return obj;
  },
  getDistrictLayerCountry(data) {
    var obj = {};
    var disCountry = new AMap.DistrictLayer.Country(data);
    obj.currentObj = disCountry;
    obj.setMap = (map) => {
      if (map.currentObj) {
        obj.currentObj.setMap(map.currentObj);
      } else {
        obj.currentObj.setMap(map);
      }
    };
    return obj;
  },
  getAMapBounds(southWest, northEast) {
    southWest = southWest.currentObj ? southWest.currentObj : southWest;
    northEast = northEast.currentObj ? northEast.currentObj : northEast;
    var myBounds = new AMap.Bounds(southWest, northEast);
    var obj = {
      currentObj: myBounds
    };
    obj.contains = function (lnglat) {
      lnglat = lnglat.currentObj ? lnglat.currentObj : lnglat
      return obj.currentObj.contains(lnglat);
    }
    obj.getCenter = function () {
      return obj.currentObj.getCenter();
    }
    obj.getNorthEast = function () {
      return obj.currentObj.getNorthEast();
    }
    obj.getSouthWest = function () {
      return obj.currentObj.getSouthWest();
    }
    obj.toString = function () {
      return obj.currentObj.toString();
    }
    return obj;
  },
  //创建第三方标准图层之切片图层类
  createTileLayer({ map, tileUrl, errorUrl, getTileUrl, complete, zIndex }) {
    let param = {};

    if (map) param.map = map;
    if (getTileUrl) param.tileUrl = getTileUrl;
    if (tileUrl) param.tileUrl = tileUrl;
    if (errorUrl) param.errorUrl = errorUrl;
    let lay = new AMap.TileLayer(param);
    param.zIndex = zIndex ? zIndex : 0;
    let ret = {
      currentObj: lay
    };
    ret.setMap = (fMap) => {
      if (fMap.currentObj)
        ret.currentObj.setMap(fMap.currentObj);
      else ret.currentObj.setMap(fMap);
    };
    ret.hide = ret.currentObj.hide;
    ret.show = ret.currentObj.show;
    ret.setTileUrl = ret.currentObj.setTileUrl;
    ret.reload = ret.currentObj.reload;
    ret.setOpacity = ret.currentObj.setOpacity;
    ret.currentObj.on("complete", () => {
      if (complete) complete();
    });
    return ret;
  },
  //创建自定义聚合图层
  createMarkLayer(config){
    let map=this.amap[config.mapId];
    if(!config.layId)config.layId=config.mapId;
    if( this.mLayerDic[config.layId]) {
      // console.warn("请不要重复创建对象,请修改layId") ;
      return this.mLayerDic[config.layId];
    }
    let context =document.getElementById(config.mapId);
    if(!context) throw config.mapId+ " 获取不到元素";
    let layer= this.mLayerDic[config.layId]=new markLayer(map,context.clientWidth,context.clientHeight,config.styles,config);
    if(!config.markitems)throw "Point不能为空";

    layer.marks=config.markitems;
    layer.refresh();
    return layer;
  },
  createMarkItem(config){
    return new markItem(config);
  },

  /**
  *  传入的参数为数组时，把自定义格式处理成地图适用的格式
  * @param {any} arr
  */
  serArrParam(arr) {
    let retArr = [];
    for (let i = 0; i < arr.length; i++) {
      if (arr[i].currentObj) retArr[i] = arr[i].currentObj;
      else retArr[i] = arr[i];
    }
    return retArr;
  }
}
export default obj;
