

import core from '../core';
import mapFunc from './map';
import amapWeb from './amapWeb';

const fillColor = "#1791fc"; //填充色 默认值"#1791fc"
const fillOpacity = 0.35;//填充透明度 默认值0.35
const strokeWeight = 2;//线宽 默认值2
const strokeColor = "#1791fc";//线颜色 默认值"#1791fc"
const strokeOpacity = 0.9;//线透明度 默认值0.9
//地图实例配置对象
const instanceConfig = {
  protocol: 'https',
  hostAndPath: 'webapi.amap.com/maps',
  callback: 'amapOnload',
  key: 'ecd528345198c8dc6f5e399beabe9b42',
  plugin: [
    'AMap.AutoComplete',
    'AMap.PlaceSearch',
    'AMap.Scale',
    'AMap.HawkEye',
    'AMap.ToolBar',
    'AMap.MapType',
    'AMap.ControlBar',
    'AMap.PolygonEditor',
    'AMap.PolylineEditor',
    'AMap.CircleEditor',
    //定位
    'AMap.Geolocation',
    //地理编码
    'AMap.Geocoder',
    //行政区查询
    'AMap.DistrictSearch',
    //自动驾车
    'AMap.Driving',
    //可拖拽
    'AMap.DragRoute',
    //鼠标工具插件
    'AMap.MouseTool',
    //测距
    'AMap.RangingTool',
    //点聚合
    'MarkerClusterer',
    //自定义图层
    "AMap.CustomLayer",
    //轨迹纠偏服务
    "AMap.GraspRoad",
    //热力图
    "AMap.HeatMap",
    "AMap.PlaceSearch",
    //简易行政区图
    "DistrictLayer",
    "AMap.MoveAnimation"
  ],
  // 默认高德 sdk 版本为 1.4.4
  v: '2.0',
  uiVersion: '1.1'
};


let obj = {
  amap: {},
  geocoder: null,
  mousetool: {},
  rangtool: {},
  dragroute: null,
  satellite: {},
  roadNet: {},
  traffic: {},
  patchIOS11Geo() {
    // ios环境切换到使用远程https定位
    if (AMap.UA.ios && document.location.protocol !== 'https:') {
      // 使用远程定位，见 remogeo.js
      var remoGeo = new RemoGeoLocation();
      // 替换方法
      navigator.geolocation.getCurrentPosition = function () {
        return remoGeo.getCurrentPosition.apply(remoGeo, arguments);
      };
      // 替换方法
      navigator.geolocation.watchPosition = function () {
        return remoGeo.watchPosition.apply(remoGeo, arguments);
      };
    }
  },
  create(data) {
    this.loadAMapScript((resData) => {
      if (resData.Code !== 0) {
        console.error("加载Js Api 失败");
        return;
      }
      var $this = this;
      this.district = new AMap.DistrictSearch({
        // 返回行政区边界坐标等具体信息
        extensions: 'all',
        // 设置查询行政区级别为 区，
      });
      this.amap[data.id] = null;
      let mapData = {
        animateEnable: false
      };
      if (data.mapStyle) mapData.mapStyle = data.mapStyle;
      if (data.viewMode) mapData.viewMode = '3D';
      if (data.pitch) mapData.pitch = data.pitch;
      if (!data.pitchEnable) mapData.pitchEnable = false;
      if (!data.rotateEnable) mapData.rotateEnable = false;
      if (data.zooms) mapData.zooms = data.zooms;
      if (data.layers) mapData.layers = data.layers;
      if (data.showIndoorMap) mapData.showIndoorMap = data.showIndoorMap;
      if (data.mask && data.mask.keywords) {
        let mapMask = [];
        this.keywords = data.mask.keywords;
        this.index = 0;
        let keyMask = {};
        this.drawMaskMap(mapMask, keyMask, this.keywords[0], (mask, keyMask) => {
          mapData.mask = mask;
          var retData = {
            keyMask: keyMask,
            mask: mask
          };
          this.initMapObj(data, mapData, retData);
        });
      } else {
        this.initMapObj(data, mapData);
      }
    });
  },
  //添加地图引擎script标签
  loadAMapScript(callback) {
    this.loadUIAMap((res) => {
      if (res.Code === 0) {
        if (window.AMap && window.AMap.Map) {
          if (callback) callback({ Code: 0 });
          return;
        };
        const script = document.createElement('script');
        script.type = 'text/javascript';
        script.async = true;
        script.defer = true;
        script.src = this._getScriptSrc();
        //异步高德地图JS API 加载回调
        if (!window['amapOnload']) {
          window['amapOnload'] = () => {
            initAMapUI();
            if (callback) callback({ Code: 0 });
          };
        }
        document.head.appendChild(script);
      } else {
        callback({ Code: -1 });
        console.error("加载AMapUI出了问题！");
      }
    });

  },
  loadUIAMap(callback) {
    if (!instanceConfig.uiVersion || window.AMapUI) { callback({ Code: 0 }); return; };
    const UIScript = document.createElement('script');
    const [versionMain, versionSub, versionDetail] = instanceConfig.uiVersion.split('.');
    if (versionMain === undefined || versionSub === undefined) {
      console.error('amap ui version is not correct, please check! version: ', instanceConfig.uiVersion);
      return;
    }
    let src = `${instanceConfig.protocol}://webapi.amap.com/ui/${versionMain}.${versionSub}/main-async.js`;
    if (versionDetail) src += `?v=${versionMain}.${versionSub}.${versionDetail}`;
    UIScript.src = src;
    UIScript.type = 'text/javascript';
    UIScript.async = true;
    document.head.appendChild(UIScript);
    UIScript.onload = () => {
      callback({ Code: 0 });
    };
    UIScript.onerror = () => {
      callback({ Code: -1 });
    };
  },
  _getScriptSrc() {
    // amap plugin prefix reg
    const amap_prefix_reg = /^AMap./;

    const config = instanceConfig;
    const paramKeys = ['v', 'key', 'plugin', 'callback'];

    // check 'AMap.' prefix
    if (config.plugin && config.plugin.length > 0) {
      // push default types
      config.plugin.push('Autocomplete', 'PlaceSearch', 'PolylineEditor', 'CircleEditor', 'PolygonEditor');

      config.plugin = config.plugin.map(item => {
        return (amap_prefix_reg.test(item)) ? item : 'AMap.' + item;
      });
    }

    const params = Object.keys(config)
      .filter(k => ~paramKeys.indexOf(k))
      .filter(k => config[k] != null)
      .filter(k => {
        return !Array.isArray(config[k]) ||
          (Array.isArray(config[k]) && config[k].length > 0);
      })
      .map(k => {
        let v = config[k];
        if (Array.isArray(v)) return { key: k, value: v.join(',') };
        return { key: k, value: v };
      })
      .map(entry => `${entry.key}=${entry.value}`)
      .join('&');
    return `${instanceConfig.protocol}://${instanceConfig.hostAndPath}?${params}`;
  },
  //地图初始化时需要执行的
  initMapObj(data, mapData, area) {
    this.amap[data.id] = new AMap.Map(data.id, mapData);
    this.satellite[data.id] = new AMap.TileLayer.Satellite();
    this.roadNet[data.id] = new AMap.TileLayer.RoadNet();
    this.traffic[data.id] = new AMap.TileLayer.Traffic();
    this.amap[data.id].add(this.traffic[data.id]);
    this.amap[data.id].add(this.roadNet[data.id]);
    this.amap[data.id].add(this.satellite[data.id]);
    this.roadNet[data.id].hide();
    this.traffic[data.id].hide();
    this.satellite[data.id].hide();
    //地图缩放监听
    // 在图面添加比例尺控件，展示地图在当前层级和纬度下的比例尺
    if (data.scale) {
      this.amap[data.id].addControl(new AMap.Scale());
    }
    // 在图面添加鹰眼控件，在地图右下角显示地图的缩略图
    if (data.overview) {
      this.amap[data.id].addControl(new AMap.HawkEye());
    }
    // 在图面添加类别切换控件，实现默认图层与卫星图、实施交通图层之间切换的控制
    if (data.toolbar) {
      // 在图面添加工具条控件，工具条控件集成了缩放、平移、定位等功能按钮在内的组合控件
      this.amap[data.id].addControl(new AMap.ToolBar({
        offset: new AMap.Pixel(10, 60),
      }));
    }
    if (data.maptype) {
      this.amap[data.id].addControl(new AMap.MapType({ defaultType: data.defaultType }));

    }
    if (data.controlBar) {
      this.amap[data.id].addControl(new AMap.ControlBar());
    }
    //地理编码
    this.geocoder = new AMap.Geocoder();
    this.rangtool[data.id] = new AMap.RangingTool(this.amap[data.id]);

    this.mousetool[data.id] = new AMap.MouseTool(this.amap[data.id]);

    if (data.callback) {
      data.callback(area)
    }
  },
  drawMaskMap(mask, keyMask, key, callback) {
    this.district.search(key, (status, info) => {
      if (status == "complete") {
        let bounds = info.districtList[0].boundaries;
        keyMask[key] = bounds;
        for (let j = 0; j < bounds.length; j++) {
          mask.push([bounds[j]]);
        }
      }
      let next = this.keywords[this.index++];
      if (next) {
        this.drawMaskMap(mask, keyMask, next, callback);
      } else {
        if (callback) callback(mask, keyMask);
      }
    });
  },
  //设置地图风格
  setMapStyle(style, id) {
    var map = this.getMapObj({ id: id }).currentObj;
    let styleStr = 'amap://styles/' + style;
    map.setMapStyle(styleStr);
  },
  setFeatures(arr, id) {
    var map = this.getMapObj({ id: id }).currentObj;
    map.setFeatures(arr);
  },
  getFeatures(id) {
    var map = this.getMapObj({ id: id }).currentObj;
    return map.getFeatures();
  },
  //设置俯仰角,3D视图有效
  setPitch(Number, id) {
    var map = this.getMapObj({ id: id }).currentObj;
    map.setPitch(Number);
  },
  zoomIn(id) {
    var map = this.getMapObj({ id: id }).currentObj;
    map.zoomIn();
  },
  zoomOut(id) {
    var map = this.getMapObj({ id: id }).currentObj;
    map.zoomOut();
  },
  //卫星图层
  satelliteShow(data) {
    this.satellite[data.id].show();
  },
  //路网图层
  roadNetShow(data) {
    this.roadNet[data.id].show();
  },
  //路况图层
  trafficShow(data) {
    this.traffic[data.id].show();
  },
  //卫星图层
  satelliteHide(data) {
    this.satellite[data.id].hide();
  },
  //路网图层
  roadNetHide(data) {
    this.roadNet[data.id].hide();
  },
  //路况图层
  trafficHide(data) {
    this.traffic[data.id].hide();
  },
  //获取两点间距离
  distance(data) {
    var p = [];
    for (var i = 0; i < data.length; i++) {
      if (!data[i].lng || !data[i].lat) {
        p.push(p);
      } else {
        p.push([data[i].lng, data[i].lat]);
      }
    }
    var dis = AMap.GeometryUtil.distance(p[0], p[1]);
    return dis;
  },
  //行政区查询实例
  districtSearch(data) {
    var inst = {};
    var district = new AMap.DistrictSearch({
      // 返回行政区边界坐标等具体信息
      extensions: 'all',
      // 设置查询行政区级别为 区 
      level: data.level ? data.level : 'district',
      subdistrict: data.subdistrict ? data.subdistrict : '1',
      showbiz: data.showbiz !== undefined ? data.showbiz : true,
    });
    inst.obj = district;
    inst.search = function (keywords, callback) {
      inst.obj.search(keywords, (status, res) => {
        var resInfo = {};
        if (status == "complete") {
          resInfo.Info = res.Info;
          resInfo.districtList = res.districtList;
        } else {
          resInfo.Info = "";
          resInfo.districtList = null;
        }
        callback(status, resInfo);
      })
    };
    inst.setLevel = function (level) {
      //设置关键字对应的行政区级别或商圈，可选值：
      // country：国家
      //province：省 / 直辖市
      // city：市
      //district：区 / 县
      //biz_area：商圈
      inst.obj.setLevel(level);
    }
    inst.setSubdistrict = function (district) {
      //设置下级行政区级数（行政区级别包括：国家、省/直辖市、市、区/县4个级别），商圈为区/县下一级，默认值：1
      //可选值：0、1、2、3
      //0：不返回下级行政区；
      //1：返回下一级行政区；
      //2：返回下两级行政区；
      //3：返回下三级行政区
      inst.obj.setSubdistrict(district);
    }
    return inst;
  },
  //数组path处理
  getPoints(data) {
    var lnglat = [];
    if (core.isNull(data.length) && data.currentObj) {
      return [data.lng, data.lat];
    };
    for (let i = 0; i < data.length; i++) {
      if (Array.isArray(data[i])) {
        lnglat.push(data[i]);
      } else if (typeof (data[i]) == "object") {
        lnglat.push([data[i].lng, data[i].lat]);
      }
    }
    return lnglat;
  },
  setPoints(arr) {
    var poi = []
    for (var i = 0; i < arr.length; i++) {
      poi.push({
        currentObj: arr[i],
        lng: arr[i].lng,
        lat: arr[i].lat,
      })
    }
    return poi;
  },
  convertFrom(gps, type, callback) {
    type = type ? type : "gps";
    AMap.convertFrom(gps, type, function (status, result) {
      if (result.info === 'ok') {
        let locs = [];
        let lnglats = result.locations; // Array.<LngLat>
        for (let i = 0; i < lnglats.length; i++) {
          locs.push([lnglats[i].lng, lnglats[i].lat]);
        }
        if (callback) {
          callback(locs)
        }
      } else {
        callback([])
      }
    });
  },
  getMapObj(data) {
    var instance = null;
    if (data && data.id !== undefined) {
      instance = {
        currentObj: this.amap[data.id]
      };
    } else {
      instance = {
        currentObj: this.amap["mapBox"]
      };
    }
    instance.getSize = function () {
      return instance.currentObj.getSize();
    }
    instance.getBounds = function () {
      var bounds = instance.currentObj.getBounds();
      var obj = {
        currentObj: bounds,
        northeast: bounds.northEast,
        southwest: bounds.southWest
      }
      return obj;
    }
    instance.setBounds = function (bounds) {
      var b = bounds.currentObj ? bounds.currentObj : bounds;
      instance.currentObj.setBounds(b);
    }
    instance.lngLatToContainer = function (lnglat) {
      var pos = null;
      if (lnglat.currentObj) {
        pos = instance.currentObj.lngLatToContainer(lnglat.currentObj);
      } else {
        pos = instance.currentObj.lngLatToContainer(lnglat);
      }
      return pos;
    }
    /**
   * 设置地图上显示的元素种类，支持bg（地图背景）、point（兴趣点）、
        road（道路）、building（建筑物）
   * @param {any} arr
   */
    instance.setFeatures = function (arr) {
      instance.currentObj.setFeatures(arr);
    }

    instance.getFeatures = function () {
      return instance.currentObj.getFeatures();
    }
    return instance;
  },
  //摧毁地图
  destroy(id) {
    var map = this.getMapObj({ id: id }).currentObj;
    if (map) {
      map.destroy();
    }
  },

  clearMap(id) {
    var map = this.getMapObj({ id: id }).currentObj;
    map.clearMap();
  },

  add(data, id) {
    if (data) {
      var map = this.getMapObj({ id: id }).currentObj;
      if (Array.isArray(data)) {
        var arr = [];
        for (var i = 0; i < data.length; i++) {
          arr.push(data[i].currentObj);
        }
        map.add(arr);
      } else {
        map.add(data.currentObj);
      }
      //map.setFitView();
    } else {
      throw new Error("data is undefined");
      return "";
    }
  },
  //移除覆盖物数组
  remove(arr, id) {
    var overlay = [];
    var data = { id: id };
    var map = this.getMapObj(data).currentObj;
    if (Array.isArray(arr)) {
      for (var i = 0; i < arr.length; i++) {
        if (arr[i].currentObj) {
          overlay.push(arr[i].currentObj);
        } else {
          overlay.push(arr[i]);
        }
      }
    } else {
      if (arr.currentObj) {
        overlay = arr.currentObj;
      } else {
        overlay = arr;
      }
    }
    map.remove(overlay);
  },

  //设置中心城市
  setCity(name, callback, id) {
    var data = { id: id };
    var map = this.getMapObj(data).currentObj;
    map.setCity(name, () => {
      if (callback) {
        callback();
      }
    });
  },

  //设置中心点
  zoomAndCenter(zoom, center, id) {
    var data = { id: id };
    var map = this.getMapObj(data).currentObj;
    if (center.currentObj) {
      map.setZoomAndCenter(zoom, center.currentObj);
    } else {
      map.setZoomAndCenter(zoom, center);
    }
  },
  setZoom(num, id, immediately, duration) {
    var data = { id: id };
    var map = this.getMapObj(data).currentObj;
    if (!immediately) immediately = false;
    if (duration)
      map.setZoom(num, immediately, duration);
    else
      map.setZoom(num, immediately);
  },
  setCenter(center, id) {
    var data = { id: id };
    var map = this.getMapObj(data).currentObj;
    if (center.currentObj) {
      map.setCenter(center.currentObj);
    } else {
      map.setCenter(center);
    }
  },
  //正向地理编码
  getLocation(address, callback) {
    //address:String
    this.geocoder.getLocation(address, function (status, res) {
      if (callback) {
        var data = {};
        data["geocodes"] = [];
        if (status == "complete") {
          data["info"] = res.info;
          data["resultNum"] = res.resultNum;
          for (let i = 0; i < res.geocodes.length; i++) {
            if (!res.geocodes[i]) {
              data["geocodes"][i] = null;
            } else {
              data["geocodes"][i] = {};
              data["geocodes"][i]["formattedAddress"] = res.geocodes[i].formattedAddress;
              data["geocodes"][i]["adcode"] = res.geocodes[i].adcode;
              data["geocodes"][i]["level"] = res.geocodes[i].level;
              data["geocodes"][i]["location"] = res.geocodes[i].location;
            }
          }
        } else if (status == "error") {
          data["info"] = res;
          data["resultNum"] = 0;
        } else {
          data["info"] = "检索结果为0";
          data["resultNum"] = 0;
        }
        callback(status, data);
      }
    });
  },
  //获取地址 逆向地理编码
  getAddress(lnglat, callback) {
    var poi = lnglat.currentObj ? lnglat.currentObj : this.lngLat(lnglat[0], lnglat[1]).currentObj;
    this.geocoder.getAddress(poi, function (status, res) {
      if (callback) {
        var data = {};
        data["regeocode"] = {};
        if (status == "complete") {
          data["info"] = res.info;
          data["regeocode"]['formattedAddress'] = res["regeocode"]['formattedAddress'];
          data["regeocode"]["addressComponent"] = res["regeocode"]["addressComponent"];
        } else if (status == "error") {
          data["info"] = res;
        } else {
          data["info"] = "检索结果为0";
        }
        callback(status, data);
      }
    });
  },

  //获取地图中心
  getCenter(id) {
    let data = { id: id };
    let map = this.getMapObj(data).currentObj;
    let currentObj = map.getCenter();
    let ret = { currentObj };
    ret.lng = currentObj.lng;
    ret.lat = currentObj.lat;
    return ret;
  },
  //获取地图zoom
  getZoom(id) {
    var data = { id: id };
    var map = this.getMapObj(data).currentObj;
    return map.getZoom();
  },
  //创建坐标点
  lngLat(lng, lat) {
    let pnt = {};
    var currentObj = new AMap.LngLat(lng, lat);
    pnt.currentObj = currentObj;
    pnt.lng = currentObj.lng;
    pnt.lat = currentObj.lat;
    return pnt;
  },
  newPixel(data) {
    var pixel = {}
    var currentObj = new AMap.Pixel(data.x, data.y);
    pixel.currentObj = currentObj;
    pixel.x = currentObj.x;
    pixel.y = currentObj.y;
    return pixel;
  },
  //构造地点查询类
  newPlaceSearch(data) {
    var map = this.getMapObj(data).currentObj;
    var currentObj = new AMap.PlaceSearch({
      map: map
    });
    var placeSearch = {};
    placeSearch.currentObj = currentObj;
    placeSearch.setCity = function (str) {
      placeSearch.currentObj.setCity(str);
    }
    placeSearch.search = function (str, callback) {
      placeSearch.currentObj.search(str, (status, result) => {
        if (callback) {
          callback(status, result);
        }
      });
    }
    return placeSearch;
  },
  //创建mark
  newMark(data) {
    return  amapWeb.newMark(data);
  },
  getObjOrgion(obj) {
    if (!obj) return obj;
    return obj.currentObj ? obj.currentObj : obj;
  },
  MapIcon(data) {
    var _this = this;
    let icon = {};
    let opt = {
      // 图标尺寸
      size: data.size ? _this.getObjOrgion(data.size) : new AMap.Size(36, 36),
      // 图标的取图地址
      image: data.image ? data.image : '',
      // 图标所用图片大小
      imageSize: data.imageSize ? _this.getObjOrgion(data.imageSize) : new AMap.Size(36, 36),
    }
    if (data.imageOffset) { opt.imageOffset = _this.getObjOrgion(data.imageOffset) } // 图标取图偏移量
    icon.currentObj = new AMap.Icon(opt);
    return icon;
  },
  MapSize(x, y) {
    var size = {};
    size.currentObj = new AMap.Size(x, y);
    return size;
  },
  setMarkPosition(data) {
    var lnglat = data.lnglat.currentObj ? data.lnglat.currentObj : this.lngLat(data.lnglat).currentObj;
    data.obj.currentObj.setPosition(lnglat);
  },
  getMarkPosition(obj) {
    return { currentObj: obj.currentObj.getPosition() };
  },
  getMarkLabel(obj) {
    return obj.currentObj.getLabel();
  },
  setMarkLabel(data) {
    data.obj.currentObj.setLabel(data.label);
  },
  setMarkOffset(data) {
    data.obj.currentObj.setOffset(data.pixel);
  },
  //轨迹展示UI组件
  pathSimplifierIns(data, callback) {
    var map = this.getMapObj(data).currentObj;
    var orginData = [];
    var _this = this;
    if (data.data) {
      for (let i = 0; i < data.data.length; i++) {
        orginData.push({
          name: data.data[i].name,
          path: _this.getPoints(data.data[i].path)
        })
      }
    } else {
      orginData = [];
    }
    var renderOptions = {};
    if (data.renderOptions) {
      renderOptions = data.renderOptions;
      //处理巡航器的形状样式处理，如果为图片则 需要特殊处理

    } else {
      renderOptions = {
        //轨迹线的样式
        renderAllPointsIfNumberBelow: 100,
        pathLineStyle: {
          strokeStyle: 'red',
          lineWidth: 6,
          dirArrowStyle: true
        }
      }
    }
    AMapUI.load(['ui/misc/PathSimplifier'], function (PathSimplifier) {

      if (!PathSimplifier.supportCanvas) {
        alert('当前环境不支持 Canvas！');
        return;
      }
      //启动页面
      var pathSimplifierIns = new PathSimplifier({
        zIndex: 100,
        map: map,
        data: orginData,
        autoSetFitView: data.autoSetFitView === undefined ? true : data.autoSetFitView,
        getPath: function (pathData, pathIndex) {
          if (data.getPath) {
            data.getPath(pathData, pathIndex);
          } else {
            return pathData.path
          }
        },
        getHoverTitle: function (pathData, pathIndex, pointIndex) {
          if (data.getHoverTitle) {
            return data.getHoverTitle(pathData, pathIndex, pointIndex);
          }
          else {
            //返回鼠标悬停时显示的信息
            if (pointIndex >= 0) {
              //鼠标悬停在某个轨迹节点上
              return pathData.name + '，点:' + (pointIndex + 1) + '/' + pathData.path.length;
            }
            //鼠标悬停在节点之间的连线上
            return pathData.name + '，点数量' + pathData.path.length;
          }
        },
        renderOptions: renderOptions,
        clickToSelectPath: false,//点击轨迹节点或者轨迹节点间的连线时，选中该轨迹，默认true。
        onTopWhenSelected: false,//置顶选中的轨迹线；置顶的含义是，将该轨迹线的zIndex设置为现存最大值+1。默认true。
      });
      var pathObj = {
        currentObj: pathSimplifierIns,
        getPathData: function (index) {
          return pathObj.currentObj.getPathData(index);
        },
        createPathNavigator: function (pathIndex, config) {

          //巡航器样式设置时，图标的兼容
          if (config.pathNavigatorStyle && config.pathNavigatorStyle.img) {
            config.pathNavigatorStyle.content = PathSimplifier.Render.Canvas.getImageContent(config.pathNavigatorStyle.img, () => {
              //图片加载成功，重新绘制一次
              pathSimplifierIns.renderLater();
            }, function onerror(e) { alert('图片加载失败！'); });
          }
          var index = pathIndex ? pathIndex : 0;
          var nav = pathSimplifierIns.createPathNavigator(index, {
            loop: config.loop ? config.loop : false,
            speed: config.speed ? config.speed : 1000,
            autoRotate: true,
            pathNavigatorStyle: config.pathNavigatorStyle ? config.pathNavigatorStyle : {}
          })
          return {
            currentObj: nav,
            start: function (pointIndex) {
              nav.start(pointIndex);
            },
            pause: function () {
              nav.pause();
            },
            resume: function () {
              nav.resume();
            },
            destroy: function () {
              nav.destroy();
            },
            getPosition: function () {
              var p = nav.getPosition();
              var position = _this.lngLat(p.lng, p.lat);
              return position;
            },
            setSpeed: function (speed) {
              nav.setSpeed(speed);
            },
            on: function (eventName, handler) {
              nav.on(eventName, handler);
            },
            off: function (eventName, handler) {
              nav.off(eventName, handler);
            },
          }
        },
        setData: function (data) {
          var arr = [];
          for (let i = 0; i < data.length; i++) {
            arr.push({
              name: data[i].name,
              path: _this.getPoints(data[i].path)
            })
          }
          pathSimplifierIns.setData(data);
        },
        on: function (eventName, handel) {
          pathSimplifierIns.on(eventName, handel);
        },
        clearPathNavigators: function () {
          pathSimplifierIns.clearPathNavigators();
        },
        getSelectedPathIndex: function () {
          return pathSimplifierIns.getSelectedPathIndex();
        },
        getSelectedPathData: function () {
          pathSimplifierIns.getSelectedPathData();
        },
        hide: function () {
          pathSimplifierIns.hide();
        },
        show: function () {
          pathSimplifierIns.show();

        }
      };
      if (callback) {
        callback(pathObj);
      }
    });

  },

  //轨迹回放 创建图片内容
  getImageContent() {

    PathSimplifier.Render.Canvas.getImageContent
  },
  //轨迹回放
  //移动
  moveAlong(data) {
    var circlable = data.circlable ? data.circlable : false;
    //data.obj.moveAlong(data.lineArr, 500);
    var points = this.getPoints(data.lineArr);
    data.obj.currentObj.moveAlong(points, data.speed, circlable);
  },
  //停止移动
  stopMove(data) {
    data.obj.currentObj.stopMove();
  },
  //暂停移动
  pauseMove(data) {
    data.obj.currentObj.pauseMove();
  },
  //继续移动
  resumeMove(data) {
    data.obj.currentObj.resumeMove();
  },
  //注册地图事件
  on(data) {
    var $this = this;
    data.obj.currentObj.on(data.eventName, (e) => {
      var obj = {};
      if (data.eventName == "click") {
        obj.target = {};
        obj.target.currentObj = e.target;
        obj.type = e.type;
        obj.lnglat = $this.lngLat(e.lnglat.lng, e.lnglat.lat);
      } else if (data.eventName == "zoomend") {
        obj.target = {};
        obj.target.currentObj = e.target;
      } else {
        obj.target = {};
        obj.target.currentObj = e.target;
        obj.type = e.type;
        if (e.lnglat) {
          obj.lnglat = $this.lngLat(e.lnglat.lng, e.lnglat.lat);
        }

      }
      if (data.callback) {
        data.callback(obj);
      }
    });

  },
  //poi搜索
  search(id, callback) {
    //加载PoiPicker
    AMapUI.loadUI(['misc/PoiPicker'], function (PoiPicker) {
      var auto = new PoiPicker({
        input: id,//输入框id
      });
      auto.on("poiPicked", (res) => {
        var data = {};
        data.location = res.item.location;
        data.adcode = res.item.adcode;
        data.district = res.item.district;
        data.address = res.item.address;
        data.name = res.item.name;
        if (callback) {
          callback(data);
        }
      })

    });
  },

  //关闭测距
  closeRangTool(data) {
    this.rangtool[data.id].turnOff();
  },

  //测距
  openRangTool(data) {
    this.rangtool[data.id].turnOn();
    this.rangtool[data.id].on("addnode", (res) => {
      if (data.addcallback) {
        data.addcallback(res)
      }
    });
    this.rangtool[data.id].on("removenode", (res) => {
      if (data.delcallback) {
        data.delcallback(res)
      }
    });
    this.rangtool[data.id].on("end", (res) => {
      if (data.endcallback) {
        data.endcallback(res)
      }
    })

  },

  //鼠标工具
  getMouseToole() {
    let mousetool = new AMap.MouseTool(this.amap[data.id]);
    var obj = { currentObj: mousetool };
    obj.marker = function () {
      mousetool.marker();
    };
    obj.polyline = function () {
      mousetool.polyline();
    };
    obj.polygon = function () {
      mousetool.polygon();
    };
    obj.rectangle = function () {
      mousetool.rectangle();
    };
    obj.circle = function () {
      mousetool.circle();
    };

    obj.measureArea = function () {
      mousetool.measureArea();
    };
    obj.rule = function () {
      mousetool.rule();
    };
    obj.rectZoomOut = function () {
      mousetool.rectZoomOut();
    };
    obj.rectZoomIn = function () {
      mousetool.rectZoomIn();
    };
    return obj;

  },
  //开启鼠标工具
  openMouseTool(data) {
    var _this = this;
    if (!this.mousetool[data.id]) throw "该地图没有创建对应的鼠标工具";
    let tool = this.mousetool[data.id].on("draw", (e) => {
      let drawData = {
        radius: 0,
        points: null,
        type: data.type,
        target: e.obj
      }
      switch (e.obj.CLASS_NAME) {
        case "AMap.Circle":
          drawData.radius = e.obj.getRadius();
          let position = e.obj.getCenter();
          drawData.points = {
            currentObj: position,
            lng: position.lng,
            lat: position.lat,

          };
          break;
        case "AMap.Polygon":
          drawData.points = _this.setPoints(e.obj.getPath());
          drawData.area = e.obj.getArea();
          break;
        case "AMap.Polyline":
          drawData.points = _this.setPoints(e.obj.getPath());
          drawData.length = e.obj.getLength();
      }
      if (data.callback) {
        data.callback(drawData);
      }
    })

    //开启鼠标操作
    this.openDrawMouseTool(data);
    return tool;
  },
  //开启鼠标操作
  openDrawMouseTool({ type, id }) {
    let mousetool = this.mousetool[id];
    if (!mousetool) throw "该地图没有创建对应的鼠标工具";
    switch (type) {
      case 'marker':
        mousetool.marker();
        break;
      case 'polyline':
        mousetool.polyline();
        break;
      case 'polygon':
        mousetool.polygon();
        break;
      case 'rectangle':
        mousetool.rectangle();
        break;
      case 'circle':
        mousetool.circle();
        break;
      case 'rule':
        mousetool.rule();
        break;
      case 'measureArea':
        mousetool.measureArea();
        break;
      case 'rectZoomIn':
        mousetool.rectZoomIn();
        break;
      case 'rectZoomOut':
        mousetool.rectZoomOut();
        break;
    }
  },

  //关闭鼠标工具
  closeMouseTool({ id }) {
    let mousetool = this.mousetool[id];
    if (!mousetool) throw "该地图没有创建对应的鼠标工具";
    mousetool.close();
  },
  // 计算两点间的角度
  calcAngle: function (data, start, end) {
    let map = this.getMapObj(data).currentObj;
    let p_start = map.lngLatToContainer(start),
      p_end = map.lngLatToContainer(end);
    let diff_x = p_end.x - p_start.x,
      diff_y = p_end.y - p_start.y;
    return 360 * Math.atan2(diff_y, diff_x) / (2 * Math.PI) + 90;
  },
  //覆盖物
  overLay: function (data, colorOpt) {
    if (!colorOpt) colorOpt = {};
    var map = this.getMapObj(data).currentObj;
    var shape = null;
    var $this = this;
    var obj = {};
    //绘制作战线路线图
    if (data.type == "geodesic") {
      let shapes = [];
      let shapesAry = [];
      data.points.forEach((item, index) => {
        let geoConfig = {
          map: map,
          path: item.value,
          ...data.config
        };
        shape = new AMap.Polyline(geoConfig);
        let shapeObj = {
          lineName: item.name,
          shape: shape,
        }
        shapes.push(shape);
        shapesAry.push(shapeObj);
      })
      // 缩放地图到合适的视野级别
      //map.setFitView(shapes);
      return shapesAry;
    };
    var lnglat = this.getPoints(data.points);
    if (data.type == "circle") {
      shape = new AMap.Circle({
        title: data.title ? data.title : "特殊",
        map: map,
        center: lnglat[0],  // 圆心位置
        radius: data.radius, // 圆半径
        strokeColor: colorOpt.strokeColor ? colorOpt.strokeColor : strokeColor, //线颜色
        strokeOpacity: colorOpt.strokeOpacity !== undefined ? colorOpt.strokeOpacity : strokeOpacity, //线透明度
        strokeWeight: colorOpt.strokeWeight ? colorOpt.strokeWeight : strokeWeight,    //线宽
        fillColor: colorOpt.fillColor ? colorOpt.fillColor : fillColor, //填充色
        fillOpacity: colorOpt.fillOpacit !== undefined ? colorOpt.fillOpacity : fillOpacity,//填充透明度
        isOutline: colorOpt.isOutline ? colorOpt.isOutline : false,
        outlineColor: colorOpt.outlineColor ? colorOpt.outlineColor : "#000000",
        borderWeight: colorOpt.borderWeight !== undefined ? colorOpt.borderWeight : 1,
        strokeStyle: colorOpt.strokeStyle ? colorOpt.strokeStyle : "solid",
        strokeDasharray: colorOpt.strokeDasharray ? colorOpt.strokeDasharray : [0, 0, 0],
        lineJoin: colorOpt.lineJoin ? colorOpt.lineJoin : 'miter',
        lineCap: colorOpt.lineCap ? colorOpt.lineCap : 'butt',
        draggable: colorOpt.draggable ? colorOpt.draggable : false,
        extData: colorOpt.extData ? colorOpt.extData : {}
      });
      //map.setZoomAndCenter(12, lnglat[0]);
    }
    else if (data.type == "ellipse") {
      shape = new AMap.Ellipse({
        map: map,
        center: lnglat[0],  // 圆心位置
        radius: data.radius,
        strokeColor: colorOpt.strokeColor ? colorOpt.strokeColor : strokeColor, //线颜色
        strokeOpacity: colorOpt.strokeOpacity !== undefined ? colorOpt.strokeOpacity : strokeOpacity, //线透明度
        strokeWeight: colorOpt.strokeWeight ? colorOpt.strokeWeight : strokeWeight,    //线宽
        fillColor: colorOpt.fillColor ? colorOpt.fillColor : fillColor, //填充色
        fillOpacity: colorOpt.fillOpacity !== undefined ? colorOpt.fillOpacity : fillOpacity,//填充透明度
        isOutline: colorOpt.isOutline ? colorOpt.isOutline : false,
        outlineColor: colorOpt.outlineColor ? colorOpt.outlineColor : "#000000",
        borderWeight: colorOpt.borderWeight !== undefined ? colorOpt.borderWeight : 1,
        strokeStyle: colorOpt.strokeStyle ? colorOpt.strokeStyle : "solid",
        strokeDasharray: colorOpt.strokeDasharray ? colorOpt.strokeDasharray : [0, 0, 0],
        lineJoin: colorOpt.lineJoin ? colorOpt.lineJoin : 'miter',
        lineCap: colorOpt.lineCap ? colorOpt.lineCap : 'butt',
        draggable: colorOpt.draggable ? colorOpt.draggable : false,
        extData: colorOpt.extData ? colorOpt.extData : {}
      });
      //map.setZoomAndCenter(12, lnglat[0]);
    }
    else {
      var config = {
        map: map,
        path: lnglat,
        strokeColor: colorOpt.strokeColor ? colorOpt.strokeColor : strokeColor, //线颜色
        strokeOpacity: colorOpt.strokeOpacity ? colorOpt.strokeOpacity : strokeOpacity, //线透明度
        strokeWeight: colorOpt.strokeWeight ? colorOpt.strokeWeight : strokeWeight,    //线宽
        fillColor: colorOpt.fillColor ? colorOpt.fillColor : fillColor, //填充色
        fillOpacity: colorOpt.fillOpacity ? colorOpt.fillOpacity : fillOpacity,//填充透明度
        isOutline: colorOpt.isOutline ? colorOpt.isOutline : false,
        outlineColor: colorOpt.outlineColor ? colorOpt.outlineColor : "#000000",
        borderWeight: colorOpt.borderWeight ? colorOpt.borderWeight : 1,
        strokeStyle: colorOpt.strokeStyle ? colorOpt.strokeStyle : "solid",
        strokeDasharray: colorOpt.strokeDasharray ? colorOpt.strokeDasharray : [0, 0, 0],
        lineJoin: colorOpt.lineJoin ? colorOpt.lineJoin : 'miter',
        lineCap: colorOpt.lineCap ? colorOpt.lineCap : 'butt',
        draggable: colorOpt.draggable ? colorOpt.draggable : false,
        extData: colorOpt.extData ? colorOpt.extData : {}
      }
      //map.setZoomAndCenter(12, lnglat[0]);
    }

    if (data.type == "polygon" || data.type == "rectangle") {
      shape = new AMap.Polygon(config);
    }
    else if (data.type == "polyline") {
      shape = new AMap.Polyline(config);
      obj.currentObj = shape;
      obj.getPath = () => {
        return obj.currentObj.getPath();
      }
      obj.setPath = (arr) => {
        let pathData = [];
        for (var i = 0; i < arr.length; i++) {
          if (arr[i].currentObj) {
            pathData.push(arr[i].currentObj)
          } else {
            pathData.push(arr[i]);
          }
        }
        obj.currentObj.setPath(pathData)
      }
    }
    obj.currentObj = shape;
    obj.setColorOptions = (type, colorOpt) => {
      obj.currentObj.setOptions(colorOpt);
    };
    obj.setDataOptions = (type, data) => {
      let opt = {};
      if (!type) throw new Error("请传入type");
      if (data.points) {
        let lnglat = this.getPoints(data.points);
        if (type == "polygon" || type == "rectangle") {
          opt.path = lnglat;
        } else {
          opt.center = lnglat[0];
        }
      }
      if (data.radius) {
        opt.radius = data.radius;
      }
      obj.currentObj.setOptions(opt);
    };

    obj.setCenter = (lnglat) => {
      if (lnglat.currentObj) lnglat = lnglat.currentObj;
      obj.currentObj.setCenter(lnglat);
    }
    obj.getCenter = () => {
      return obj.currentObj.getCenter();
    }
    obj.hide = () => {
      obj.currentObj.hide();
    }
    obj.show = () => {
      obj.currentObj.show();
    }
    obj.setPath = () => {
      obj.currentObj.setPath();
    }
    obj.getBounds = () => {
      return obj.currentObj.getBounds();
    }
    return obj;
    return obj;
  },
  addText(data) {
    return  amapWeb.addText(data);
  },
  //绘制线路
  drawRoute(data) {
    var amap = this.getMapObj(data).currentObj;
    if (data.dragroute === undefined) {
      //路线规划没有传递对象时，默认只有一条数据，操作的都是一条数据
      var $this = this;
      var mapData = {};
      if (this.dragroute) {
        this.dragroute.destroy();
      }
      if (data.path) {
        if (data.path.length == 0) {
          path = [];
          mapData.points = [];
          mapData.routeObj = null;
          mapData.start = null;
          mapData.end = null;
          mapData.waypoints = [];
          if (data.completecallback) {
            data.completecallback(mapData);
          }
        }
      }
      let copyPath = core.clone(data.path);
      this.dragroute = new AMap.DragRoute(amap, copyPath, data.type);
      this.dragroute.search();

      this.dragroute.on("complete", obj => {
        mapData.points = mapFunc.getPath(obj.data.start.location, obj.data.waypoints, obj.data.end.location);
        mapData.routeObj = obj.target;
        mapData.start = obj.data.start.location;
        mapData.end = obj.data.end.location;
        var way = obj.data.waypoints;
        mapData.waypoints = [];
        for (var i = 0; i < way.length; i++) {
          mapData.waypoints.push(way[i].location);
        }
        mapData.routesObj = obj.data;
        mapData.routes = this.dragroute.getRoute();
        if (data.completecallback) {
          data.completecallback(mapData);
        }
      })
      this.dragroute.on("addway", function (obj) {
        if (data.addwaycallback) {
          data.addwaycallback(obj);
        }
      })
    }
    else {
      //参数包括对象时，处理的是参数的路线对象
    }
  },
  //驾车线路规划
  driving() {
    let driving = new AMap.Driving({
      // 驾车路线规划策略，AMap.DrivingPolicy.LEAST_TIME是最快捷模式
      policy: AMap.DrivingPolicy.LEAST_TIME
    });
    let obj = {};
    obj.currentObj = driving;
    //使用经纬度获取驾车规划数据
    obj.search = function (config, callback) {
      if (!config.opts) {
        //无途经点
        config.opts = {
          waypoints: []
        };
      } else if (!config.opts.waypoints) {
        //无途经点
        config.opts.waypoints = [];
      }
      if (config.startLngLat && config.endLngLat) {
        //经纬度
        let startLngLat = config.startLngLat.currentObj ? config.startLngLat.currentObj : config.startLngLat;
        let endLngLat = config.endLngLat.currentObj ? config.endLngLat.currentObj : config.endLngLat;
        obj.currentObj.search(startLngLat, endLngLat, config.opts, (status, result) => {
          //status =>complete error no_data
          callback(status, result)
        })
      } else if (config.points) {
        //关键字获取驾车规划数据
        //var points = [
        //    { keyword: '北京市地震局（公交站）', city: '北京' },
        //    { keyword: '亦庄文化园（地铁站）', city: '北京' }
        //]
        console.log(config.points);
        obj.currentObj.search(config.points, (status, result) => {
          //status =>complete error no_data
          callback(status, result)
        })
      }
    }
    return obj;
  },
  //摧毁路线
  destroyRoute(data) {
    if (data.obj) {
      data.obj.destroy();
    } else {
      throw new Error("dstory 的参数不正确");
    }
  },
  //绘制行政区划
  drawDistrict(data) {
    var $this = this;
    var amap = this.getMapObj(data).currentObj;
    this.district.setLevel = data.level;
    this.district.search(data.name + "", (status, result) => {
      if (status == "complete") {
        var bounds = result.districtList[0].boundaries
        var polygons = []
        if (bounds) {
          for (var i = 0, l = bounds.length; i < l; i++) {
            //生成行政区划polygon
            var polygon = new AMap.Polygon({
              map: amap,
              path: bounds[i],
              strokeColor: data.strokeColor ? data.strokeColor : strokeColor, //线颜色
              strokeOpacity: data.strokeOpacity ? data.strokeOpacity : strokeOpacity, //线颜色
              strokeWeight: data.strokeWeight ? data.strokeWeight : strokeWeight, //线颜色
              fillColor: data.fillColor ? data.fillColor : fillColor, //线颜色
              fillOpacity: data.fillOpacity ? data.fillOpacity : fillOpacity, //线颜色
            })
            polygons.push(polygon)
          }
          // 地图自适应
          amap.setFitView();
        }
      } else {
        throw new Error("数据错误");
      }
      if (data.callback) {
        var shape = {};
        shape[data.name] = polygons;
        data.callback(status, result, shape);
      }
    });

  },

  //点聚合
  markerClusterer(data) {
    var amap = this.getMapObj(data).currentObj;
    var $this = this;
    var points = this.formateClusterData(data.markers);
    let param = {
      gridSize: data.gridSize ? data.gridSize : 60,
      maxZoom: data.maxZoom ? data.maxZoom : 18,
      clusterByZoomChange: data.clusterByZoomChange === undefined ? false : data.clusterByZoomChange,

      renderMarker: (m) => {
        if (m.data && m.data[0]) {
          let bMarker = m.data[0].bMarker;
          if (bMarker) {
            //补充支持1.0功能，标记点图标做映射
            let icon = bMarker.getIcon();
            let icon2 = m.marker.getIcon();
            let lable = bMarker.getLabel();
            m.marker.setIcon(icon);
            m.marker.setLabel(lable);
            //非标准经纬度数组传入时，兼容1.0版本  映射点击事件
            let clickfn = function (e) {
              AMap.Event.trigger(bMarker, "click", e);
            }
            m.marker.on("click", clickfn);
          }
        }

        if (data.renderMarker) data.renderMarker();
      }
    };
    if (data.renderClusterMarker) param.renderClusterMarker = data.renderClusterMarker;

    var cluster = new AMap.MarkerCluster(amap, points, param);
    amap.setFitView();
    obj = {};
    obj.currentObj = cluster;
    obj.setMarkers = function (arr) {

    }
    obj.clusterMarkers = data.markers;
    cluster.on("click", (res) => {
      if (res.clusterData.length > 1 || (res.clusterData[0] && res.clusterData[0]._amapMarker.count > 1)) {
        //补充1.0支持的功能，点击聚合点放大
        amap.zoomIn();
        amap.setCenter(res.lnglat);
      } else {

      }
      //标准2.0版本在这里 注册点击事件
      if (data.clickcallback) {
        let bRes = {};
        bRes.currentObj = res;
        bRes.target = res.target;
        let bLng = { currentObj: res.lnglat, lng: res.lnglat.lng, lat: res.lnglat.lat };
        bRes.lnglat = bLng;
        data.clickcallback(bRes);
      }
    });

    return obj;
  },
  //清除地图上的所有信息窗体
  clearInfoWindow(data) {
    var amap = this.getMapObj(data).currentObj;
    amap.clearInfoWindow();
  },
  //点聚合添加点（单个）
  addClusterMarker(data) {
    this.addClusterMarkers({ obj: data.obj, markers: [data.marker] });
  },
  //点聚合添加点(批量)
  addClusterMarkers(option) {
    let data = option.markers
    let dataOption = this.formateClusterData(data);
    option.obj.currentObj.addData(dataOption);
  },

  setClusterMarkers(data) {
    //amap.setFitView();
    this.setClusterData({ obj: data.obj, markers: data.datas });

  },
  formateClusterData(data) {
    let dataOption = [];
    for (let i = 0; i < data.length; i++) {
      //按2.0标准传入数组
      if (Array.isArray(data[i])) {
        dataOption.push({ weight: 1, lnglat: data[i] });
      }
      else if (typeof data[i] == "object") {
        if (data[i].currentObj) {
          let lnglat = data[i].currentObj.getPosition();
          dataOption.push({ weight: 1, lnglat: [lnglat.lng, lnglat.lat], bMarker: data[i] });
        }
        else if (data[i]) {
          let lnglat = data[i].getPosition();
          dataOption.push({ weight: 1, lnglat: [lnglat.lng, lnglat.lat], bMarker: data[i] });
        }
      }
    }
    return dataOption;
  },
  //清除所有点聚合
  clearClusterMarkers(config) {
    config.obj.currentObj.setData([]);
  },
  //移除单个聚合点
  removeClusterMarker(data) {
    throw new Error("高德地图2.0版本不支持方法 removeClusterMarker，推荐使用 setClusterData");
  },

  setClusterData(data) {
    let points = this.formateClusterData(data.markers);
    data.obj.currentObj.setData(points);
    data.obj.clusterMarkers = data.obj.clusterMarkers.concat(data.markers);
  },
  //移除聚合点(批量)
  removeClusterMarkers(data) {
    throw new Error("高德地图2.0版本不支持方法 removeClusterMarker，推荐使用 setClusterData");
  },
  //获取聚合数量
  getClustersCount(data) {
    if (data.obj) {
      return data.obj.currentObj.getClustersCount();
    } else {
      throw new Error("getClustersCount参数不正确");
    }
  },
  infoWindow(data) {
    let infoWindow = new AMap.InfoWindow({
      content: data.content ? data.content : "内容为空",
      offset: data.offset ? data.offset : new AMap.Pixel(20, -20),
      autoMove: data.autoMove === undefined ? false : data.autoMove,
      isCustom: data.isCustom === undefined ? false : data.autoMove
    })
    infoWindow.on("close", () => {
      if (data.closecallback) {
        data.closecallback();
      }
    })
    infoWindow.on("open", () => {
      if (data.opencallback) {
        data.opencallback();
      }
    })
    infoWindow.on("change", () => {
      if (data.changecallback) {
        data.changecallback();
      }
    })
    let obj = {};
    obj.currentObj = infoWindow;
    return obj;
  },
  simpleInfoWindow(callback) {
    //加载SimpleInfoWindow，loadUI的路径参数为模块名中 'ui/' 之后的部分
    AMapUI.loadUI(['overlay/SimpleInfoWindow'], function (SimpleInfoWindow) {
      var infoWindow = new SimpleInfoWindow({
        infoTitle: '<strong>这里是标题</strong>',
        infoBody: '<p>这里是内容。</p>'
      });
      //显示在map上
      infoWindow.open(map, map.getCenter());
      var obj = {};
      obj.currentObj = infoWindow;
      if (callback) {
        callback(obj);
      }
    });
  },
  //打开信息窗体
  openInfoWindow(data) {
    let amap = this.getMapObj(data).currentObj;
    if (data.position.currentObj) {
      data.obj.currentObj.open(amap, data.position.currentObj);
    } else if (Array.isArray(data.position)) {
      let position = new AMap.LngLat(data.position[0], data.position[1], true);
      data.obj.currentObj.open(amap, position);
    }
  },
  closeInfoWindow(obj) {
    obj.currentObj.close();
  },
  setInfoContent(data) {
    data.obj.currentObj.setContent(data.content);
  },
  getInfoContent(obj) {
    return obj.currentObj.getContent()
  },
  setInfoPosition(data) {
    var lnglat = data.lnglat.currentObj ? data.lnglat.currentObj : data.lnglat;
    data.obj.currentObj.setPosition(lnglat);
  },
  getInfoPosition(obj) {
    return { currentObj: obj.currentObj.getPosition() }
  },
  //自定义图层对象
  customLayer(data) {
    //创建图层对象
    let amap = this.getMapObj(data).currentObj;
    var options = data.options ? data.options : {};
    var customLayer = new AMap.CustomLayer(data.drawObj, {
      map: options.map ? options.map : amap,
      zooms: options.zooms ? options.zooms : [3, 20],
      alwaysRender: options.alwaysRender !== undefined ? options.alwaysRender : true,//缩放过程中是否重绘，复杂绘制建议设为false
      zIndex: options.zIndex ? options.zIndex : 120
    });
    var instance = { currentObj: customLayer };
    instance.setMap = function (map) {
      //map对象
      amap.add(instance.currentObj);
    }
    instance.hide = function () {
      instance.currentObj.hide();
    }
    instance.show = function () {
      instance.currentObj.show();
    }
    instance.setzIndex = function (number) {
      instance.currentObj.setzIndex(number);
    }
    instance.getzIndex = function () {
      return instance.currentObj.getzIndex();
    }
    instance.setOpacity = function (number) {
      instance.currentObj.setOpacity(number);
    }
    instance.setRender = function (fn) {
      if (typeof fn == "function") {
        instance.currentObj.render = fn;
      } else {
        console.error("参数类型应为 Function");
      }
    }
    return instance;
  },
  //轨迹纠偏服务 绑路
  graspRoad() {
    let grasp = new AMap.GraspRoad()
    let obj = {
      currentObj: grasp,
    }
    obj.driving = function (originPath, callback) {
      if (!error) {
        var newPath = result.data.points;//纠偏后的轨迹
        var distance = result.data.distance;//里程
        callback(newPath, distance);
      } else {
        callback(null, null);
      }
    }
    return obj;
  },
  //定位对象
  getGeoLocation(data) {
    var geolocation = new AMap.Geolocation({
      enableHighAccuracy: data.enableHighAccuracy ? data.enableHighAccuracy : true,//是否使用高精度定位，默认:true
      timeout: data.timeout ? data.timeout : 4000,          //超过10秒后停止定位，默认：5s
      buttonPosition: 'RB',    //定位按钮的停靠位置
      buttonOffset: new AMap.Pixel(10, 20),//定位按钮与设置的停靠位置的偏移量，默认：Pixel(10, 20)
      zoomToAccuracy: data.zoomToAccuracy ? data.zoomToAccuracy : true   //定位成功后是否自动调整地图视野到定位点
    });
    var obj = {
      currentObj: geolocation,
      getCurrentPosition: (callback) => {
        geolocation.getCurrentPosition(function (status, result) {
          if (status == 'complete') {
            var param = {
              info: "成功",
              message: "",
              position: [result.position.lng, result.position.lat],
            }
          } else {
            var param = {
              info: "失败",
              message: result.message,
              position: null
            }
          }
          if (callback) {
            callback(param)
          }
        });
      }
    }
    return obj;
  },
  //调整地图视野
  setFitView(id, data) {
    var map = this.getMapObj({ id: id }).currentObj;
    if (data) {
      let param = null;
      if (Array.isArray(data)) {
        param = this.serArrParam(data);
      } else {
        param = data.currentObj ? data.currentObj : data
      }
      map.setFitView(param);
    } else {
      map.setFitView();
    }

  },
  getHeatmap(data) {
    var amap = this.getMapObj(data).currentObj;
    var obj = {};
    var heatmap = new AMap.HeatMap(amap, {});
    obj.currentObj = heatmap;
    //radius gradient opacity zooms
    //设置热力图属性
    obj.setOptions = (option) => {
      obj.currentObj.setOptions(option);
    }
    //设置热力图展现的数据集
    obj.setDataSet = (dataset) => {
      obj.currentObj.setDataSet(dataset);
    }
    //设置热力图展现的数据集
    obj.setMap = (map) => {
      if (map) {
        map = map.currentObj ? map.currentObj : map;
      }
      map.add(obj.currentObj);
    }
    //
    obj.show = () => {
      obj.currentObj.show();
    }
    //
    obj.hide = (dataset) => {
      obj.currentObj.hide();
    }
    return obj;
  },
  setObject3DLayer(data) {
    var amap = this.getMapObj(data).currentObj;
    var obj = {};
    var object3Dlayer = new AMap.Object3DLayer();
    obj.currentObj = object3Dlayer;
    obj.add = (wall) => {
      obj.currentObj.add(wall);
    }
    amap.add(object3Dlayer);
    return obj;
  },
  setObject3DWall(data) {
    //data :object3Dlayer
    var obj = {};
    var wall = new AMap.Object3D.Wall({
      path: data.options.bounds,
      height: data.options.height,
      color: data.options.color
    });
    obj.currentObj = wall;
    data.object3Dlayer.currentObj.add(obj.currentObj);
    return obj;
  },
  //设置省级图层
  getDistrictLayerProvince(data) {
    var obj = {};
    var disProvince = new AMap.DistrictLayer.Province(data);
    obj.currentObj = disProvince;
    obj.setMap = (map) => {
      if (map.currentObj) {
        map.currentObj.add(obj.currentObj);
      } else {
        map.add(obj.currentObj);
      }
    };
    return obj;
  },
  getDistrictLayerCountry(data) {
    var obj = {};
    var disCountry = new AMap.DistrictLayer.Country(data);
    obj.currentObj = disCountry;
    obj.setMap = (map) => {
      if (map.currentObj) {
        map.currentObj.add(obj.currentObj);
      } else {
        map.add(obj.currentObj);
      }
    };
    return obj;
  },
  getAMapBounds(southWest, northEast) {
    southWest = southWest.currentObj ? southWest.currentObj : southWest;
    northEast = northEast.currentObj ? northEast.currentObj : northEast;
    var myBounds = new AMap.Bounds(southWest, northEast);
    var obj = {
      currentObj: myBounds
    };
    obj.contains = function (lnglat) {
      lnglat = lnglat.currentObj ? lnglat.currentObj : lnglat
      return obj.currentObj.contains(lnglat);
    }
    obj.getCenter = function () {
      return obj.currentObj.getCenter();
    }
    obj.getNorthEast = function () {
      return obj.currentObj.getNorthEast();
    }
    obj.getSouthWest = function () {
      return obj.currentObj.getSouthWest();
    }
    obj.toString = function () {
      return obj.currentObj.toString();
    }
    return obj;
  },
  //创建第三方标准图层之切片图层类 
  createTileLayer({ map, tileUrl, errorUrl, getTileUrl, complete, zIndex }) {
    let param = {};

    if (map) param.map = map;
    if (getTileUrl) param.tileUrl = getTileUrl;
    if (tileUrl) param.tileUrl = tileUrl;
    if (errorUrl) param.errorUrl = errorUrl;
    let lay = new AMap.TileLayer(param);
    param.zIndex = zIndex ? zIndex : 0;
    let ret = {
      currentObj: lay
    };
    ret.setMap = (fMap) => {
      if (fMap.currentObj)
        fMap.currentObj.add(ret.currentObj);
      else fMap.add(ret.currentObj);
    };
    ret.hide = ret.currentObj.hide;
    ret.show = ret.currentObj.show;
    ret.setTileUrl = ret.currentObj.setTileUrl;
    ret.reload = ret.currentObj.reload;
    ret.setOpacity = ret.currentObj.setOpacity;
    ret.currentObj.on("complete", () => {
      if (complete) complete();
    });
    return ret;
  },
  /**
 *  传入的参数为数组时，把自定义格式处理成地图适用的格式
 * @param {any} arr
 */
  serArrParam(arr) {
    let retArr = [];
    for (let i = 0; i < arr.length; i++) {
      if (arr[i].currentObj) retArr[i] = arr[i].currentObj;
      else retArr[i] = arr[i];
    }
    return retArr;
  }
}
export default obj;
