/**
 *
 * 所有接口和返回内容都按照高德地图接口和类的标准封装。接口和对象定义请参考 高德官方文档和（amapWeb.js）
 * */
   

//地图实例配置对象
const instanceConfig = {
  protocol: 'https',
  hostAndPath: 'api.map.baidu.com/api',
  callback: 'initialize',
  //ak
  ak: '1QPHlsvcuEeSvAvfB8nOLTEG5tin3pZ6',
  v: '3.0',
};

let obj = {
  bmap: {},

  create(data) {
    window.onload = () => {
    this.loadAMapScript((resData) => {
      if (resData.Code !== 0) {
        console.error("加载Js Api 失败");
        return;
      }
      let $this = this;
      let mapData = {};
      if (data.zooms) {
        mapData["minZoom"] = data.zooms[0];
        mapData["maxZoom"] = data.zooms[1];
      } else {
        mapData["minZoom"] = 3;
        mapData["maxZoom"] = 18;
      }
      if (data.viewMode)   mapData.viewMode == data.viewMode;
      this.initMapObj(data, mapData);
    });
    }
  },
  //地图初始化时需要执行的
  initMapObj(data, mapData, area) {
    if (mapData.viewMode == '3D') {

    } else {
      this.bmap[data.id] = new BMap.Map(data.id, mapData);
    }
    let loc = new BMap.LocalCity;
    loc.get((res) => {
      let point = (116.404, 39.915);
      if (res) {
        point= new BMap.Point(res.center.lng, res.center.lat);
      }
      this.bmap[data.id].centerAndZoom(point, 15);
    });
    this.bmap[data.id].enableScrollWheelZoom();
    if (data.callback) {
      data.callback(area)
    }
  },
  //添加地图引擎script标签
  loadAMapScript(callback) {
        const script = document.createElement('script');
        script.type = 'text/javascript';

        script.src = this._getScriptSrc();
        //异步高德地图JS API 加载回调
      if (!window['initialize']) {
        window['initialize'] = () => {
            if (callback) callback({ Code: 0 });
          };
        }
        document.head.appendChild(script);

    //let script = document.createElement("script");
    //script.src = `http://api.map.baidu.com/api?v=2.0&ak=${instanceConfig.}&callback=initialize`;
    //document.body.appendChild(script);  
  },
 
  _getScriptSrc() {
    return `${instanceConfig.protocol}://${instanceConfig.hostAndPath}?v=${instanceConfig.v}&ak=${instanceConfig.ak}&callback=initialize`;
  },


  //获取地图对象
  getMapObj(data) {
    let instance = null;
    if (data && data.id !== undefined) {
      instance = {
        currentObj: this.bmap[data.id]
      };
    } else {
      instance = {
        currentObj: this.bmap["mapBox"]
      };
    }
    instance.getSize = function () {
      let size = instance.currentObj.getSize();
      return {
        getWidth: () => { return size.width },
        getHeight: () => { return size.height  }
      }
    }
    instance.getBounds = function () {
      let bounds = instance.currentObj.getBounds();
      let obj = {
        currentObj: bounds,
        northeast: bounds.getNorthEast(),
        southwest: bounds.getSouthWest()
      }
      return obj;
    }
    //instance.setBounds = function (bounds) {
    //  //bounds:Array<Point>
    //  let b = bounds.currentObj ? bounds.currentObj : bounds;
    //  instance.currentObj.setBounds(b);
    //}
    //instance.lngLatToContainer = function (lnglat) {
    //  let pos = null;
    //  if (lnglat.currentObj) {
    //    pos = instance.currentObj.lngLatToContainer(lnglat.currentObj);
    //  } else {
    //    pos = instance.currentObj.lngLatToContainer(lnglat);
    //  }
    //  return pos;
    //}
    /**
   * 设置地图上显示的元素种类，支持bg（地图背景）、point（兴趣点）、
        road（道路）、building（建筑物）
   * @param {any} arr
   */
    //instance.setFeatures = function (arr) {
    //  instance.currentObj.setFeatures(arr);
    //}

    //instance.getFeatures = function () {
    //  return instance.currentObj.getFeatures();
    //}
    return instance;
  },

  //注册地图事件
  on(data) {
    let $this = this;
    data.obj.currentObj.addEventListener(data.eventName, (e) => {
      let obj = {};
      //if (data.eventName == "click") {
      //  obj.target = {};
      //  obj.target.currentObj = e.target;
      //  obj.type = e.type;
      //  obj.lnglat = $this.lngLat(e.lnglat.lng, e.lnglat.lat);
      //} else if (data.eventName == "zoomend") {
      //  obj.target = {};
      //  obj.target.currentObj = e.target;
      //}
      if (data.callback) {
        data.callback(obj);
      }
    }); 
  },

  setCenter(center, id) {
    let data = { id: id };
    let map = this.getMapObj(data).currentObj;
    if (!center.currentObj) {
      center = this.lngLat(center[0],center[1]);
    }
    map.setCenter(center.currentObj);
  },

  //获取地图中心
  getCenter(id) {
    let data = { id: id };
    let map = this.getMapObj(data).currentObj;
    let currentObj = map.getCenter();
    let ret = { currentObj };
    ret.lng = currentObj.lng;
    ret.lat = currentObj.lat;
    return ret;
  },
  ////获取地图zoom
  getZoom(id) {
    let data = { id: id };
    let map = this.getMapObj(data).currentObj;
    return map.getZoom();
  },
  setZoom(num, id) {
    let data = { id: id };
    let map = this.getMapObj(data).currentObj;
    map.setZoom(num);
  },
  //创建坐标点
  lngLat(lng, lat) {
    let pnt = {};
    let currentObj = new BMap.Point(lng, lat);
    pnt.currentObj = currentObj;
    pnt.lng = currentObj.lng;
    pnt.lat = currentObj.lat;
    return pnt;
  },
  newPixel(data) {
    let pixel = {}
    let currentObj = new BMap.Pixel(data.x, data.y);
    pixel.currentObj = currentObj;
    pixel.x = currentObj.x;
    pixel.y = currentObj.y;
    return pixel;
  },
  setFeatures(arr, id) {
    let map = this.getMapObj({ id: id }).currentObj;
    this.features = arr;
    let mStyle = { features: arr };
    if (this.mapStyle) mStyle.style = this.mapStyle;
    map.setMapStyle(mStyle);
  },
  getFeatures(id) {
    throw new Error("百度地图不支持获取地图配置");
  },
  //设置俯仰角,3D视图有效
  setPitch(Number, id) {
    var map = this.getMapObj({ id: id }).currentObj;
    map.enableScrollWheelZoom(true);
    map.setHeading(64.5);
    map.setTilt(73);
  },
  distance(data) {
    let p = [];
    for (let i = 0; i < data.length; i++) {
      if (!data[i].lng || !data[i].lat) {
        //[]
        p.push(data[i]);
      } else {
        //经纬度对象
        p.push([data[i].lng, data[i].lat]);
      }
    }
    let map = this.getMapObj({ id: id }).currentObj;
    let dis = map.getDistance(p[0], p[1]);
    return dis;
  },

};
export default obj;
