let markLayer = class {
    constructor(map, width, height, styles, config) {
        this.map = map;
        let canvas = this.canvas = document.createElement('canvas');
        canvas.width = width;
        canvas.height = height;
        this.drawType = config.drawType;
        this.maxZoom = 17;
        this.collStatus = 0;
        this.shinSize = 30;
        this.thinStatus = 1; //1 = 抽稀 2=关闭抽稀释
        if (config.maxZoom) this.maxZoom = config.maxZoom;
        if (config.collStatus) this.collStatus = config.collStatus;
        if (config.thinStatus) this.thinStatus = config.thinStatus;
        if (config.shinSize) this.shinSize = config.shinSize;
        this.gridSize = 70;
        this.context = canvas.getContext('2d')
        //层级需要改小，否则出现测距的删除按钮被遮盖，无法点击
        var customLayer = new AMap.CustomLayer(canvas, {
            zIndex: 100
        });
        this.zoomOnClick = true;
        this.showLable = true;
        if (config.zoomOnClick === false) this.zoomOnClick = false;
        if (config.showLable === false) this.showLable = false;
        this.imgDic = {};
        this.events = {};
        this.collArray = [];//已绘制在界面上的所有元素
        this.visible = true;//true=显示图层
        this.debugGridVisible = false;//辅助调试网格
        this.marks = [];
        this.m1 = null;//10个以内的聚合图标
        this.m2 = null;//100个以内的聚合图标
        this.m3 = null;//1000个以内的聚合图标
        this.m4 = null;//10000个以内的聚合图标
        this.m5 = null;//10000个以上的聚合图标
        //drawType=1默认绘制图片，空或者0绘默认制圆
        this.defIcons = {
            Lev1: { src: require("../images/point/m1.png") },
            Lev2: { src: require("../images/point/m2.png") },
            Lev3: { src: require("../images/point/m3.png") },
            Lev4: { src: require("../images/point/m4.png") },
            Lev5: { src: require("../images/point/m5.png") }
        }
        if (this.drawType == 1) {
            if (styles) {
                //如果绘制类型为绘制图标的话，未传入的配置采用默认图标
                if ((!styles.Lev1 || !styles.Lev1.src)) styles.Lev1 = this.defIcons.Lev1;
                if ((!styles.Lev2 || !styles.Lev2.src)) styles.Lev2 = this.defIcons.Lev2;
                if ((!styles.Lev3 || !styles.Lev3.src)) styles.Lev3 = this.defIcons.Lev3;
                if ((!styles.Lev4 || !styles.Lev4.src)) styles.Lev4 = this.defIcons.Lev4;
                if ((!styles.Lev5 || !styles.Lev5.src)) styles.Lev5 = this.defIcons.Lev5;
            } else {
                styles = this.defIcons;
            }
        }
        this.setCollectStyles(styles, () => {
            console.log("样式初始化完成");
            customLayer.render = () => this.refresh();
            customLayer.setMap(map);
            map.on('click', (e) => this.mapClick(e));
            map.on('zoomend', () => {
                this.bounding = false;
            });
        });
    }

    //注册一个事件
    on(eventName, eventFunc) {
        let eventAry = this.events[eventName];
        if (!eventAry) {
            eventAry = [];
            this.events[eventName] = eventAry;
        }
        eventAry.push(eventFunc);
    }

    raiseEvent(eventName, arg) {
        let eventAry = this.events[eventName];
        if (!eventAry) return;
        for (let i = 0; i < eventAry.length; i++) {
            const regedEvent = eventAry[i];
            regedEvent(arg);
        }
    }

    //刷新图层内容
    refresh() {
        let context = this.context;
        let canvas = this.canvas;
        let map = this.map;
        let zoom = this.map.getZoom();
        console.log(zoom);
        context.clearRect(0, 0, canvas.width, canvas.height);
        if (!this.visible || this.bounding) return;
        let splitSize = this.gridSize;
        this.drawDebugGrid(splitSize);
        this.drawMarks(this.marks, splitSize, this.thinStatus, this.shinSize);
    }

  drawMarks(marks, splitSize, thinStatus, shinSize) {
        if (!splitSize) splitSize = 80;
        if (!shinSize) shinSize =40;
       if (!thinStatus) thinStatus =1;//1=抽稀 
        let context = this.context;
        let map = this.map;
        // console.log("startRender=======")
        let bounds = map.getBounds();
        let left = bounds.southwest.lng;
        let bottom = bounds.southwest.lat;
        let top = bounds.northeast.lat;
        let right = bounds.northeast.lng;
        //if (left < 0) left += 360;
        if (right < 0) right += 360;
        let renderCount = 0;
        let slng = this.calcSlngSLat(splitSize);
        let thLng = this.calcSlngSLat(shinSize);
        let hCount = Math.floor(360 / slng);
        let thCount = Math.floor(360 / thLng);
        let zoom = map.getZoom();
        let coll = {};
        let thinColl = {};
        let joinTrig = slng / 2;
        let collArray = [];
        //整理所有元素,合并和计数
        //console.log(zoom);
        for (let index = 0; index < marks.length; index++) {
            let mark = marks[index];
            mark.lat = parseFloat(mark.lat);
            mark.lng = parseFloat(mark.lng);
            //如果点本身不在视野范围内就直接跳过
            let inView = mark.lng >= left && mark.lng < right && mark.lat >= bottom && mark.lat < top;
          if (!inView) continue;
          //计算点抽稀
          let thRow = Math.floor((90 - mark.lat) / thLng);
          let thCol = Math.floor((mark.lng + 180) / thLng);
          let thIdx = thCount * thRow + thCol;
          //console.log(thIdx, zoom);
          if (thinStatus ==1&&zoom <= this.maxZoom && this.collStatus == 1 && thinColl[thIdx]) {
             /*console.log(thIdx, zoom);*/
             continue;
          }
           //计算点所在的格子索引
            let row = Math.floor((90 - mark.lat) / slng);

            let col = Math.floor((mark.lng + 180) / slng);
            let detIdx = hCount * row + col;

            //如果等级大于17 则直接绘制元素
          if (zoom > this.maxZoom || this.collStatus == 1) {
                let colItem = {};
                thinColl[thIdx] = colItem
                collArray.push(colItem);
                colItem.count = 1;
                colItem.row = row;
                colItem.col = col;
                colItem.detIdx = detIdx;
                colItem.lng = mark.lng;
                colItem.lat = mark.lat;
                colItem.mark = mark;
                colItem.markItems = [mark];
                colItem.left = mark.lng - joinTrig;
                colItem.right = mark.lng + joinTrig;
                colItem.top = mark.lat - joinTrig;
                colItem.bottom = mark.lat + joinTrig;
                this.drawCollItem(colItem);
                continue
            }

            let colItem = coll[detIdx];
            //对格子索引的点数量计数
            if (!colItem) {
                colItem = {};
                coll[detIdx] = colItem;
                colItem.count = 0;
                colItem.row = row;
                colItem.col = col;
                colItem.detIdx = detIdx;
                colItem.lng = mark.lng;
                colItem.lat = mark.lat;
                colItem.mark = mark;
                colItem.markItems = [];
                colItem.left = mark.lng - joinTrig;
                colItem.right = mark.lng + joinTrig;
                colItem.top = mark.lat - joinTrig;
                colItem.bottom = mark.lat + joinTrig;
            }
            colItem.markItems.push(mark);
            colItem.count++;
            renderCount++;
        }
        // console.log("结果1", coll);
        //再次整理,合并距离较近的聚合点
        let collKeys = Object.keys(coll);
        for (let keyIdx = 0; keyIdx < collKeys.length; keyIdx++) {
            const key1 = collKeys[keyIdx];
            let col1 = coll[key1];
            if (!col1) continue;
            for (let othKeyIdx = keyIdx + 1; othKeyIdx < collKeys.length; othKeyIdx++) {
                const key2 = collKeys[othKeyIdx];
                let col2 = coll[key2];
                if (!col2) continue;
                let dis = this.distance(col1.lng, col1.lat, col2.lng, col2.lat);
                if (dis <= joinTrig) {
                    //合并两个聚合点
                    if (col2.count > col1.count) {
                        col2.count += col1.count;
                        col2.lng += (col1.lng - col2.lng) / 2;
                        col2.lat += (col1.lat - col2.lat) / 2;
                        delete coll[key1]
                    }
                    else {
                        col1.count += col2.count;
                        col1.lng += (col2.lng - col1.lng) / 2;
                        col1.lat += (col2.lat - col1.lat) / 2;
                        delete coll[key2]
                    }
                }
            }
        }
        // console.log("结果2", coll);
        //绘制合并后的结果
        for (let key in coll) {
            let colItem = coll[key];
            collArray.push(colItem);
            this.drawCollItem(colItem);
        }
        this.collArray = collArray;
    }

    drawCollItem(colItem) {
        let context = this.context;
        let pix = this.toPixl(colItem.lng, colItem.lat);
        let icon = this.m5;
        let r = 22;
        let color = "#B700FF";
        if (Math.floor(colItem.count / 10) == 0) {
            icon = this.m1;
            r = 12;
            color = "#008CFF"
        }
        else if (Math.floor(colItem.count / 100) == 0) {
            icon = this.m2;
            r = 14;
            color = "#FFBF00"
        }
        else if (Math.floor(colItem.count / 1000) == 0) {
            icon = this.m3;
            r = 16;
            color = "#FF0000"
        }
        else if (Math.floor(colItem.count / 10000) == 0) {
            icon = this.m4;
            r = 19;
            color = "#FF00F2"
        }
        let textColor = ""
        let font = ""
        //只有一个标注元素时，如果标注元素有图标，这使用标注元素的图标
        if (colItem.count == 1 && colItem.mark.img)
            icon = colItem.mark;
        if (icon) {
            if (colItem.count > 1)
                this.drawIcon(icon, pix.x, pix.y);
            else
                this.showmark(colItem.mark);
            let width = icon.width;
            let height = icon.height;
            if (!width) width = icon.img.width;
            if (!height) height = icon.img.height;
            colItem.width = width;
            colItem.height = height;
            textColor = icon.textColor
            font = icon.font
        }
        else {
            if (colItem.count > 1)
                this.drawCircle(pix.x, pix.y, r, color)
            else
                this.showmark(colItem.mark);
            colItem.width = r * 2;
            colItem.height = r * 2;
        }
        if (colItem.count > 1)
            this.drawText(colItem.count + "", pix.x, pix.y, font, textColor, "middleCenter");
    }

    //在界面上呈现指定图标
    showmark(mark) {
        let context = this.context;
        let pix = this.toPixl(mark.lng, mark.lat);
        let img = mark.img;
        if (img)
            this.drawIcon(mark, pix.x, pix.y);
        else
            this.drawCircle(pix.x, pix.y, 5, "#FF7171")
        if (mark.text) {
            let fontStyle = mark.fontStyle;
            let font = mark.font;
            if (!font) font = "12px 宋体";
            if (!fontStyle) fontStyle = "#000000";
            let height = mark.height;
            if (!height) {
                if (img)
                    height = img.height
                else
                    height = 5
            }
            if (this.showLable)
            this.drawText(mark.text, pix.x, pix.y + height / 2, font, fontStyle, "middleTop", true, mark.textOffsetX, mark.textOffsetY)
        }
    }

    //绘制辅助格子
    drawDebugGrid(size) {
        if (!this.debugGridVisible) return;
        let context = this.context;
        let canvas = this.canvas;
        let map = this.map;

        let bounds = map.getBounds();
        let left = bounds.southwest.lng;
        let bottom = bounds.southwest.lat;
        let top = bounds.northeast.lat;
        let right = bounds.northeast.lng;
        //if (left < 0) left += 360;
        if (right < 0) right += 360;
        let slng = this.calcSlngSLat(size);

        //绘制竖线
        let bLng = 180;
        let endLng = -1;
        while (bLng > -180) {
            if (bLng >= left && bLng < right) {
                let p1 = this.toPixl(bLng, bottom);
                let p2 = this.toPixl(bLng, top);
                this.drawLine(p1.x, p1.y, p2.x, p2.y);
                endLng = bLng;
            }
            bLng -= slng;
        }
        let hCount = Math.floor(360 / slng);
        let osize = size / 2;
        //绘制横线
        let blat = 90;
        while (blat > -90) {
            if (blat >= bottom && blat < top) {
                let p1 = this.toPixl(left, blat);
                let p2 = this.toPixl(right, blat);
                this.drawLine(p1.x, p1.y, p2.x, p2.y);

                //绘制格子的Id
                bLng = endLng;
                let row = Math.floor((90 - blat) / slng);
                while (bLng < right) {
                    let txtPix = this.toPixl(bLng, blat);
                    this.drawCircle(txtPix.x, txtPix.y, 3);
                    let col = Math.floor((bLng + 180) / slng);
                    let detIdx = hCount * row + col;
                    this.drawText(detIdx + "", txtPix.x + osize, txtPix.y + osize, null, "#0000FF", "middleCenter");
                    bLng += slng;
                }
            }
            blat -= slng;
        }
    }

    drawLine(x1, y1, x2, y2, style) {
        let context = this.context;
        context.beginPath();   //开始一个新的绘制路径
        context.moveTo(x1, y1);   //定义直线的起点坐标为(10,10)
        context.lineTo(x2, y2);   //定义直线的终点坐标为(200,10)
        if (style)
            context.strokeStyle = style;
        else
            context.strokeStyle = "#9c9c9c";
        context.stroke();   //沿着坐标点的路径顺序绘制直线
        context.closePath();   //关闭当前的绘制路径
    }

    drawCircle(x, y, r, fillStyle, strokeStyle) {
        let context = this.context;
        //3.绘制
        context.beginPath();//开始绘制
        context.arc(x, y, r, 0, 2 * Math.PI);//arc 的意思是“弧”
        if (fillStyle)
            context.fillStyle = fillStyle;//设置填充颜色
        else
            context.fillStyle = "#ff0";//设置填充颜色
        context.fill();//开始填充
        if (strokeStyle)
            context.strokeStyle = strokeStyle;//将线条颜色设置为蓝色
        else
            context.strokeStyle = "#BFBFBF";//将线条颜色设置为蓝色
        context.stroke();//stroke() 方法默认颜色是黑色（如果没有上面一行，则会是黑色）。
    }

    //在地图上绘制一个图标,格式参考类型 collIcon 
    drawIcon(icon, x, y, align) {
        let context = this.context;
        let width = icon.width;
        let height = icon.height;
        if (!width) width = icon.img.width;
        if (!height) height = icon.img.height;
        if (!align) align = "middleCenter";
        let rx = x, ry = y;
        if (icon.offsetX) x += icon.offsetX
        if (icon.offsetY) y += icon.offsetY
        switch (align) {
            case "leftTop": rx = x; ry = y; break;
            case "leftMiddle": rx = x; ry = y - height / 2; break;
            case "leftBottom": rx = x - width / 2; ry = y - height / 2; break;
            case "rightTop": rx = x - width; ry = y; break;
            case "rightMiddle": rx = x - width; ry = y - height / 2; break;
            case "rightBottom": rx = x - width; ry = y - height; break;
            case "middleTop": rx = x - width / 2; ry = y; break;
            case "middleCenter": rx = x - width / 2; ry = y - height / 2; break;
            case "middleBottom": rx = x - width / 2; ry = y - height; break;
        }
        if (!icon.angle)
            context.drawImage(icon.img, rx, ry, width, height);
        else {
            context.save();
            let tx = width / 2;
            let ty = height / 2;
            context.translate(rx + tx, ry + ty);
            context.rotate((icon.angle * Math.PI) / 180);
            context.drawImage(icon.img, -tx, -ty, width, height);
            context.restore();
        }
    }

    drawText(txt, x, y, font, fillStyle, align, backGround, textOffsetX, textOffsetY) {
        if (!txt) return;
        let context = this.context;
        let txtSize = context.measureText(txt);
        let fontHeight = txtSize.actualBoundingBoxAscent + 1;
        let fontWidth = txtSize.width;
        if (!fillStyle) fillStyle = "#000000";
        if (!font) font = "12px Arial";
        let rx = x;
        let ry = y;

        switch (align) {
            case "leftTop": rx = x; ry = y + fontHeight; break;
            case "leftMiddle": rx = x; ry = y + fontHeight / 2; break;
            case "leftBottom": rx = x; ry = y; break;
            case "rightTop": rx = x - fontWidth; ry = y + fontHeight; break;
            case "rightMiddle": rx = x - fontWidth; ry = y + fontHeight / 2; break;
            case "rightBottom": rx = x - fontWidth; ry = y; break;
            case "middleTop": rx = x - fontWidth / 2; ry = y + fontHeight; break;
            case "middleCenter": rx = x - fontWidth / 2; ry = y + fontHeight / 2; break;
            case "middleBottom": rx = x - fontWidth / 2; ry = y; break;
            default: rx = x, y + fontHeight; break;
        }
        if (textOffsetX) rx += textOffsetX;
        if (textOffsetY) ry += textOffsetY;
        if (backGround) {
            context.fillStyle = "#FFFFFF";
            context.strokeStyle = "#000000";
            context.strokeRect(rx - 2, ry - fontHeight - 1, fontWidth + 4, fontHeight + 5);
            context.fillRect(rx - 2, ry - fontHeight - 1, fontWidth + 4, fontHeight + 5);
        }
        if (font) context.font = font;
        if (fillStyle) context.fillStyle = fillStyle;

        context.fillText(txt, rx, ry);
    }

    mapClick(e) {
        if (!this.visible) return;
        let colItem = this.selItem(e.lnglat.lng, e.lnglat.lat);
        if (!colItem) return;
        let canvas = this.canvas;
        this.raiseEvent("click", colItem)
        //关闭点击聚合点时，散开
        if (!this.zoomOnClick) { return }
        if (colItem.count > 1) {//点击的是聚合效果时放大地图视野
            this.bounding = true;
            this.context.clearRect(0, 0, canvas.width, canvas.height);
            //let bounds = new AMap.Bounds(new AMap.LngLat(colItem.left, colItem.bottom), new AMap.LngLat(colItem.right, colItem.top))
            //this.map.setBounds(bounds);
            //设置边界改成设置层级多度更顺滑
            this.map.setCenter(e.lnglat);
            let zoom = this.map.getZoom();
            if (zoom <= 18) {
                this.map.setZoom(zoom + 2);
            }
        } else {
            this.showmark(colItem.mark);
        }
    }
    setCollectStyles(styles, callback) {
        let fn = [];
        let count = 0;
        if (!styles) { callback(); return; }

        if (styles.Lev1 && styles.Lev1.src) {
            if (!this.m1) this.m1 = new collIcon(styles.Lev1);
            if (!this.m1.img) this.m1.img = new Image();
            fn.push(this.m1.img.onload)
            this.m1.img.src = styles.Lev1.src;
        }
        if (styles.Lev2 && styles.Lev2.src) {
            if (!this.m2) this.m2 = new collIcon(styles.Lev2);
            if (!this.m2.img) this.m2.img = new Image();
            fn.push(this.m2.img.onload)
            this.m2.img.src = styles.Lev2.src;
        }
        if (styles.Lev3 && styles.Lev3.src) {
            if (!this.m3) this.m3 = new collIcon(styles.Lev3);
            if (!this.m3.img) this.m3.img = new Image();
            fn.push(this.m3.img.onload)
            this.m3.img.src = styles.Lev3.src;
        }
        if (styles.Lev4 && styles.Lev4.src) {
            if (!this.m4) this.m4 = new collIcon(styles.Lev4);
            if (!this.m4.img) this.m4.img = new Image();
            fn.push(this.m4.img.onload)
            this.m4.img.src = styles.Lev4.src;
        }
        if (styles.Lev5 && styles.Lev5.src) {
            if (!this.m5) this.m5 = new collIcon(styles.Lev5);
            if (!this.m5.img) this.m5.img = new Image();
            fn.push(this.m5.img.onload)
            this.m5.img.src = styles.Lev5.src;
        }
        Promise.all(fn).then(() => {
            this.refresh();
            if (callback) callback();
        })
        //可以支持实时更新车辆图标、支持实时添加点，移除点，隐藏点、支持动态设置图标大小和样式、

    }
    setGridSize(num) {
        this.gridSize = num;
        this.refresh();
    }
    setShinSize() {
     this.shinSize = num;
     this.refresh();
    }
    setCollStatus(num) {
        this.collStatus = num;
        this.refresh();
    }
    setshowLable(val) {
        this.showLable = val;
        this.refresh();
    }
    setZoomOnClick(val) {
        this.zoomOnClick = val;
    }
    setMarkItems(marks) {
        this.marks = [].concat(marks);
        this.refresh();
    }
    addMrakItems(marks) {
        this.marks = this.marks.concat(marks);
        this.refresh();
    }
    selItem(lng, lat) {
        let colArray = this.collArray;
        if (!colArray) return;
        let p1 = this.toPixl(lng, lat);
        for (let i = 0; i < colArray.length; i++) {
            const colItem = colArray[i];
            if (lng >= colItem.left && lng < colItem.right && lat >= colItem.top && lat < colItem.bottom) {
                let p2 = this.toPixl(colItem.lng, colItem.lat);
                //长宽相等视为一个圆形图标
                if (colItem.width == colItem.height) {
                    let dis = this.distance(p1.x, p1.y, p2.x, p2.y);
                    if (dis < colItem.width / 2)
                        return colItem;
                }
                else {
                    let hw = colItem.width / 2;
                    let hh = colItem.height / 2;
                    //转换为正方形，降低用户点击精度
                    if (hh < hw) hh = hw;
                    if (hw < hh) hw = hh;
                    let ox = colItem.mark.offsetX;
                    let oy = colItem.mark.offsetY;
                    let left = p2.x - hw +ox;
                    let right = p2.x + hw + ox;
                    let top = p2.y - hh + oy;
                    let bottom = p2.y + hh + oy;
                    if (p1.x >= left && p1.x < right && p1.y >= top && p1.y < bottom)
                        return colItem;
                }
            }
        }
        return null;
    }
    //将一个像素坐标长度，转换为经纬度坐标长度
    calcSlngSLat(size) {
        let map = this.map;
        let c1 = map.getCenter();
        let pix = this.toPixl(c1.lng, c1.lat);
        //计算两条竖线之间的经度间隔
        let clng = this.toLngLat(pix.x + size, pix.y);
        let slng = clng.lng - c1.lng; if (slng < 0) slng = -slng;
        return slng;
        // //计算横线之间的纬度间隔
        // let clat = this.toLngLat(pix.x, pix.y + size);
        // let slat = clat.lat - c1.lat; if (slat < 0) slat = -slat;
        // return {
        //     slng: slng,
        //     slat: slng
        // }
    }

    //将经纬度转换为像素坐标
    toPixl(lng, lat) {
        var pixel = this.map.lngLatToContainer([lng, lat]);
        return pixel;
    }

    //将像素坐标转换为经纬度
    toLngLat(x, y) {
        let arg = this.toLngLatArg;
        if (!arg) arg = this.toLngLatArg = new AMap.Pixel(1, 1);
        arg.x = x;
        arg.y = y;
        return this.map.containerToLngLat(arg);
    }

    distance(x1, y1, x2, y2) {
        let dx = x1 - x2;
        let dy = y1 - y2;
        return Math.sqrt(dx * dx + dy * dy);
    }
}

let markItem = class {
    constructor(config) {
        if (!config) config = {}
        this.img = config.img ? config.img : null;//图标元素
        this.width = config.width ? config.width : 16;//宽度
        this.height = config.height ? config.height : 16;//高度
        this.angle = config.angle > 0 ? config.angle : 0;//旋转角度
        this.lng = config.lng ? config.lng : 118;//经度
        this.lat = config.lat ? config.lat : 24;//纬度
        this.text = config.text ? config.text : null;
        this.offsetX = config.offsetX ? config.offsetX : 0;
        this.textOffsetX = config.textOffsetX ? config.textOffsetX : 0;
        this.textOffsetY = config.textOffsetY ? config.textOffsetY : 0;
        this.offsetY = config.offsetY ? config.offsetY : 0;
        this.text = config.text ? config.text : null;
        this.font = config.font ? config.font : null;
        this.ext = config.ext ? config.ext : null;
        this.key = config.key ? config.key : null;
        this.name = config.name ? config.name : null;
        this.fontStyle = config.fontStyle ? config.fontStyle : null;
    }
}

let collIcon = class {
    constructor(style) {
        if (!style) style = {};
        this.img = style.img
        this.width = style.width;//宽度
        this.height = style.height;//高度
        this.offsetX = style.imgX;//偏移x
        this.offsetY = style.imgY;//偏移y
        this.angle = style.angle;//旋转角度
        this.font = style.font;//旋转角度
        this.textColor = style.textColor;//旋转角度
    }
}

export { collIcon, markItem, markLayer }
