import Vue from 'vue';
import core from '../core';
import mapFunc from './map';
const fillColor = "#1791fc"; //填充色 默认值"#1791fc"
const fillOpacity = 0.35;//填充透明度 默认值0.35
const strokeWeight = 2;//线宽 默认值2
const strokeColor = "#1791fc";//线颜色 默认值"#1791fc"
const strokeOpacity = 0.9;//线透明度 默认值0.9
let obj = {
  amap: {},
  time_id: null,//轨迹动画定时
  geocoder: null,
  mousetool: null,
  rangtool: null,
  dragroute: null,
  satellite: null,
  roadNet: null,
  traffic: null,
  create(data)
  {
    var $this = this;
    epgis.tokenTask.login('7a5faf5e76a83f5ba7a866545b4be9b0', "044f933e21ae32b8b2464bfbea836094").then(() =>
    {
      this.amap[data.id] = new epgis.Map({
        // 地图绑定的DOM元素ID
        container: data.id,
        // 地图样式
        style: "aegis://styles/aegis/Streets",
        // 默认缩放层级
        zoom: 11,
        // 地图中心点
        center: [116.397428, 39.90923],
        // 地图默认字体
        localIdeographFontFamily: 'Microsoft YoHei'
      });
      this.district = new epgis.DistrictTask();
      if(data.callback)
      {
        data.callback()
      }
    }).
      catch((res) =>
      {
        throw new Error(res);
      })
  },
  //设置地图风格
  setMapStyle(style, id)
  {
    var map = this.getMapObj({ id: id }).currentObj;
    let styleStr = 'amap://styles/' + style;
    map.setMapStyle(styleStr);
  },
  zoomIn(id)
  {
    var map = this.getMapObj({ id: id }).currentObj;
    map.zoomIn();
  },
  zoomOut(id)
  {
    var map = this.getMapObj({ id: id }).currentObj;
    map.zoomOut();
  },
  //卫星图层
  satelliteShow(data)
  {
    this.satellite.show();
  },
  //路网图层
  roadNetShow(data)
  {
    this.roadNet.show();
  },
  //路况图层
  trafficShow(data)
  {
    this.traffic.show();
  },
  //卫星图层
  satelliteHide(data)
  {
    this.satellite.hide();
  },
  //路网图层
  roadNetHide(data)
  {
    this.roadNet.hide();
  },
  //路况图层
  trafficHide(data)
  {
    this.traffic.hide();
  },

  //数组path处理
  getPoints(data)
  {
    var lnglat = [];
    if(core.isNull(data.length) && data.currentObj)
    {
      return [data.currentObj];
    }
    for(let i = 0; i < data.length; i++)
    {
      if(Array.isArray(data[i]))
      {
        lnglat.push(this.lngLat(data[i][0], data[i][1]).currentObj);
      } else if(typeof (data[i]) == "object")
      {
        if(data[i].currentObj)
        {
          lnglat.push(data[i].currentObj);
        } else
        {
          lnglat.push(data[i]);
        }
      } else if(typeof (data[i]) != "object")
      {
        lnglat = [this.lngLat(data[0], data[1]).currentObj];
        break;
      }
    }
    return lnglat;
  },
  setPoints(arr)
  {
    var poi = []
    for(var i = 0; i < arr.length; i++)
    {
      poi.push({
        currentObj: arr[i],
        lng: arr[i].lng,
        lat: arr[i].lat,
      })
    }
    return poi;
  },
  convertFrom(gps, callback)
  {
    AMap.convertFrom(gps, 'gps', function (status, result)
    {
      if(result.info === 'ok')
      {
        var lnglats = result.locations; // Array.<LngLat>
        if(callback)
        {
          callback()
        }
      }
    });
  },
  getMapObj(data)
  {
    if(data && data.id !== undefined)
    {
      return { currentObj: this.amap[data.id] };
    } else
    {
      return { currentObj: this.amap["mapBox"] };
    }
  },
  //摧毁地图
  destroy(id)
  {
    var map = this.getMapObj({ id: id }).currentObj;
    if(map)
    {
      map.remove();
    }
  },

  clearMap(id)
  {
    var map = this.getMapObj({ id: id }).currentObj;
    console.log("暂无此功能")
    //map.clearMap();
  },

  add(data, id)
  {
    console.log(data);
    if(data)
    {
      var map = this.getMapObj({ id: id }).currentObj;
      data.currentObj.addTo(map)
      //map.setFitView();
    } else
    {
      throw new Error("data is undefined");
      return
    }
  },
  //移除覆盖物数组
  remove(arr, id)
  {
    var overlay = [];
    var data = { id: id };
    var map = this.getMapObj(data).currentObj;
    if(Array.isArray(arr))
    {
      for(var i = 0; i < arr.length; i++)
      {
        if(arr[i].currentObj)
        {
          overlay.push(arr[i].currentObj);
        } else
        {
          overlay.push(arr[i]);
        }
      }
    } else
    {
      if(arr.currentObj)
      {
        overlay = arr.currentObj;
      } else
      {
        overlay = arr;
      }
    }
    overlay.remove();
  },

  //设置中心城市
  setCity(name, callback, id)
  {
    var data = { id: id };
    var map = this.getMapObj(data).currentObj;
    map.setCity(name, () =>
    {
      if(callback)
      {
        callback();
      }
    });
  },

  //设置中心点
  zoomAndCenter(zoom, center, id)
  {
    var data = { id: id };
    var map = this.getMapObj(data).currentObj;
    if(center.currentObj)
    {
      map.setZoom(zoom);
      map.setCenter([center.lng, center.lat,]);
    } else
    {
      map.setZoom(zoom);
      map.setCenter(center);
    }
  },
  setZoom(num, id)
  {
    var data = { id: id };
    var map = this.getMapObj(data).currentObj;
    map.setZoom(num);
  },
  setCenter(center, id)
  {
    var data = { id: id };
    var map = this.getMapObj(data).currentObj;
    if(center.currentObj)
    {
      map.setCenter([center.lng, center.lat,]);
    } else
    {
      map.setCenter(center);
    }
  },

  
  //坐标偏移误差转换
  transferLnglat(coords,callback){
    let convertTask = new epgis.ConvertTask();
    convertTask.convertCoord([coords],{
      from:1
    }).then(function(result){
      callback(result);
    });
  },
  
  //获取地址
  getAddress(lnglat, callback)
  {
    let geocodingTask = new epgis.GeocodingTask();
    var lnglat = lnglat.currentObj ? [lnglat.currentObj.lng, lnglat.currentObj.lat] : lnglat;
    geocodingTask.getAddress({
      location: lnglat,
      coordtype: "gps"
    }).then(function (res)
    {
      if(callback)
      {
        var data = {
          info: "ok",
          regeocode: res,
        };
        callback("complete", data);
      }
    });
  },
  //获取地址
  // getAddress(lnglat, callback)
  // {
  //   let districtTask = new epgis.DistrictTask();
  //   var poi = lnglat.currentObj ? [lnglat.currentObj.lng, lnglat.currentObj.lat] : lnglat;
  //   districtTask.searchByLocation({
  //     coords: poi,
  //     coordtype: "sg"
  //   }).then(function (res)
  //   {
  //     if(callback)
  //     {
  //       var data = {
  //         info: "ok",
  //         all: res,
  //         regeocode: {
  //           formattedAddress: "",
  //           addressComponent: {
  //             adcode: res[0].adcode,
  //             province: res[0].name,
  //             city: res[1].name,
  //             citycode: res[1].adcode,
  //             district: res[2].name,
  //           },
  //         }
  //       };
  //       callback("complete", data);
  //     }
  //   });
  // },

  //获取地图中心
  getCenter(id)
  {
    var data = { id: id };
    var map = this.getMapObj(data).currentObj;
    return { currentObj: map.getCenter() };
  },
  //获取地图zoom
  getZoom(id)
  {
    var data = { id: id };
    var map = this.getMapObj(data).currentObj;
    return map.getZoom();
  },
  //创建坐标点
  lngLat(lng, lat)
  {
    let pnt = {};
    var currentObj = new epgis.LngLat(lng, lat);
    pnt.currentObj = currentObj;
    pnt.lng = currentObj.lng;
    pnt.lat = currentObj.lat;
    return pnt;
  },
  newPixel(data)
  {
    var pixel = {}
    var currentObj = new AMap.Pixel(data.x, data.y);
    pixel.currentObj = currentObj;
    pixel.x = currentObj.x;
    pixel.y = currentObj.y;
    return pixel;
  },

  setElementObj(type, value)
  {
    if(type == "icon")
    {
      let markDom = document.createElement("img");
      markDom.src = value;
      return markDom;
    } else if(type == "content")
    {
      let markDom = document.createElement("div");
      markDom.innerHTML = value;
      return markDom.childNodes[0];
    }
  },

  //创建mark
  newMark(data)
  {
    var $this = this;
    var mark = {};
    var currentObj = new epgis.Marker({
      //position: $this.lngLat(data.lng, data.lat).currentObj,
      element: data.icon ? $this.setElementObj("icon", data.icon) : data.content ? $this.setElementObj("content", data.content) : "",
      offset: [0, 0],
    });
    currentObj.setLngLat([data.lng, data.lat]);
    mark.currentObj = currentObj;
    mark.getOffset = function ()
    {
      var offset = {}
      var offsetObj = currentObj.getOffset();
      offset.currentObj = offsetObj;
      offset.x = offsetObj.x;
      offset.y = offsetObj.y;
      return offset;
    }
    mark.setOffset = function ()
    {
      currentObj.setOffset();
    }
    mark.getPosition = function ()
    {
      var poi = {};
      var poiObj = currentObj.getLngLat();
      poi.currentObj = poiObj;
      poi.lng = poiObj.lng;
      poi.lat = poiObj.lat;
      return poi;
    }
    mark.setPosition = function (data)
    {
      var lnglat = [data.lng, data.lat];
      currentObj.setLngLat(lnglat)
    }
    //mark.setLabel
    //mark.getLabel
    //mark.setIcon
    //mark.getIcon
    //mark.getContent
    //mark.setContent
    //mark.moveAlong
    //mark.moveTo
    //mark.stopMove
    //mark.pauseMove
    //mark.resumeMove
    //mark.setMap
    //mark.getMap
    //mark.setTitle
    //mark.getTitle
    return mark;
  },

  setMarkPosition(data)
  {
    var lnglat = data.lnglat.currentObj ? data.lnglat.currentObj : this.lngLat(data.lnglat).currentObj;
    data.obj.currentObj.setPosition(lnglat);
  },
  getMarkPosition(obj)
  {
    return { currentObj: obj.currentObj.getPosition() };
  },
  getMarkLabel(obj)
  {
    return obj.currentObj.getLabel();
  },
  setMarkLabel(data)
  {
    data.obj.currentObj.setLabel(data.label);
  },
  setMarkOffset(data)
  {
    data.obj.currentObj.setOffset(data.pixel);
  },
  //轨迹展示UI组件
  pathSimplifierIns(data, callback)
  {
    var map = this.getMapObj(data).currentObj;
    var orginData = [];
    var _this = this;
    if(data.data)
    {
      for(let i = 0; i < data.data.length; i++)
      {
        orginData.push({
          name: data.data[i].name,
          path: _this.getPoints(data.data[i].path)
        })
      }
    } else
    {
      orginData = [];
    }
    var renderOptions = {};
    if(data.renderOptions)
    {
      if(data.renderOptions.pathLineStyle)
      {
        renderOptions.pathLineStyle = data.renderOptions.pathLineStyle;
      } else
      {
        renderOptions.pathLineStyle = {
          strokeStyle: 'red',
          lineWidth: 6,
          dirArrowStyle: true
        }
      }
    } else
    {
      renderOptions.pathLineStyle = {
        strokeStyle: 'red',
        lineWidth: 6,
        dirArrowStyle: true
      }
    }
    AMapUI.load(['ui/misc/PathSimplifier'], function (PathSimplifier)
    {

      if(!PathSimplifier.supportCanvas)
      {
        alert('当前环境不支持 Canvas！');
        return;
      }
      //启动页面
      var pathSimplifierIns = new PathSimplifier({
        zIndex: 100,
        map: map,
        data: orginData,
        getPath: function (pathData, pathIndex)
        {
          if(data.getPath)
          {
            data.getPath(pathData, pathIndex);
          } else
          {
            return pathData.path
          }
        },
        getHoverTitle: function (pathData, pathIndex, pointIndex)
        {
          if(data.getHoverTitle)
          {
            return data.getHoverTitle(pathData, pathIndex, pointIndex);
          }
          else
          {
            //返回鼠标悬停时显示的信息
            if(pointIndex >= 0)
            {
              //鼠标悬停在某个轨迹节点上
              return pathData.name + '，点:' + pointIndex + '/' + pathData.path.length;
            }
            //鼠标悬停在节点之间的连线上
            return pathData.name + '，点数量' + pathData.path.length;
          }
        },
        renderOptions: {
          //轨迹线的样式
          renderAllPointsIfNumberBelow: 100,
          pathLineStyle: renderOptions.pathLineStyle,
        }
      });
      var pathObj = {
        currentObj: pathSimplifierIns,
        createPathNavigator: function (pathIndex, config)
        {
          var index = pathIndex ? pathIndex : 0;
          var nav = pathSimplifierIns.createPathNavigator(index, {
            loop: config.loop ? config.loop : false,
            speed: config.speed ? config.speed : 1000,
            pathNavigatorStyle: {
              pathLinePassedStyle: {
                lineWidth: 6,
                strokeStyle: '#3CA81B',
                dirArrowStyle: {
                  stepSpace: 15,
                  strokeStyle: 'red'
                }
              }
            }
          })
          return {
            currentObj: nav,
            start: function (pointIndex)
            {
              nav.start(pointIndex);
            },
            pause: function ()
            {
              nav.pause();
            },
            resume: function ()
            {
              nav.resume();
            },
            destroy: function ()
            {
              nav.destroy();
            },
            getPosition: function ()
            {
              var p = nav.getPosition();
              var position = _this.lngLat(p.lng, p.lat);
              return position;
            },
            setSpeed: function (speed)
            {
              nav.setSpeed(speed);
            },
            on: function (eventName, handler)
            {
              nav.on(eventName, handler);
            },
            off: function (eventName, handler)
            {
              nav.off(eventName, handler);
            },
          }
        },
        setData: function (data)
        {
          var arr = [];
          for(let i = 0; i < data.length; i++)
          {
            arr.push({
              name: data[i].name,
              path: _this.getPoints(data[i].path)
            })
          }
          pathSimplifierIns.setData(data);
        },
        on: function (eventName, handel)
        {
          pathSimplifierIns.on(eventName, handel);
        },
        clearPathNavigators: function ()
        {
          pathSimplifierIns.clearPathNavigators();
        },
        getSelectedPathIndex: function ()
        {
          return pathSimplifierIns.getSelectedPathIndex();
        },
        getSelectedPathData: function ()
        {
          pathSimplifierIns.getSelectedPathData();
        },
        hide: function ()
        {
          pathSimplifierIns.hide();
        },
        show: function ()
        {
          pathSimplifierIns.show();

        }
      };
      if(callback)
      {
        callback(pathObj);
      }
    });


  },
  //轨迹回放
  //移动
  moveAlong(data, callback)
  {
    //data参数{ lineArr为轨迹点数组，autoPlay是否为自动播放，obj巡航器对象(覆盖物对象),speed车速 单位km/h,callback回调函数}
    var
      $this = this,
      chunkInfos = null,
      map = this.getMapObj(data).currentObj,
      //是否自动播放
      autoPlay = data.autoPlay,
      // 坐标索引
      i = 0,
      // 路片段索引
      j = 0,
      // 上一个坐标点
      prevPoint = null,
      // 上一个路片段
      prevPoingInfo = null,
      //巡航器对象
      movePointMarker = data.obj,
      //速度，单位km/h
      speed = data.speed,
      // 路线轨迹
      lineArr = {
        "type": "Feature",
        "properties": {},
        "geometry": {
          "type": "LineString",
          "coordinates": data.lineArr
        }
      },
      //轨迹回放经过线段对象
      geojson = {
        "type": "FeatureCollection",
        "features": [{
          "type": "Feature",
          "geometry": {
            "type": "LineString",
            "coordinates": []
          }
        }]
      },
      aryInfo = data.lineArr;
    if(map.getSource('line-animation') && map.getSource('route'))
    {
      //停止轨迹动画效果
      cancelAnimationFrame($this.time_id);
      //清空经过的轨迹线
      geojson.features[0].geometry.coordinates = [];
      map.getSource('line-animation').setData(geojson);
      //重新加载轨迹线
      map.getSource('route').setData(lineArr);
      movePointMarker.currentObj.setLngLat(data.lineArr[0]);
      if(autoPlay)
      {
        chunkLine()
      }
    } else
    {
      $this.add(movePointMarker, data.id);
      //轨迹线
      map.loadImage("https://maps.epgis.com/products/docs/assets/images/roadFragment.jpg", (error, image) =>
      {
        //添加图片到map，可以设置图片id
        map.addImage("poi", image);
        map.addLayer({
          "id": "route",
          "type": "line",
          "source": {
            "type": "geojson",
            "data": lineArr
          },
          "layout": {
            "line-join": "round",
            "line-cap": "round"
          },
          "paint": {
            "line-color": "#FF0000",
            "line-width": 6,
            "line-pattern": "poi"
          }
        });
        //经过的轨迹线
        map.addLayer({
          'id': 'line-animation',
          'type': 'line',
          'source': {
            'type': 'geojson',
            'data': geojson
          },
          'layout': {
            'line-cap': 'round',
            'line-join': 'round'
          },
          'paint': {
            'line-color': '#7FD53E',
            'line-width': 7,
            'line-opacity': 1
          }
        });
        if(autoPlay)
        {
          chunkLine()
        }
      })
    }
    // 以路线居中视图
    var bounds = aryInfo.reduce(function (bounds, coord)
    {
      return bounds.extend(coord);
    }, new epgis.LngLatBounds(aryInfo[0], aryInfo[0]));

    map.fitBounds(bounds, {
      padding: 30
    });
    //线段
    function chunkLine()
    {
      var chunk;
      if(j < aryInfo.length)
      {
        if(j != 0)
        {
          chunk = turf.lineChunk(
            turf.lineString([prevPoingInfo, aryInfo[j]]),
            speed / 60 / 3600 || .01, {
              units: "kilometers"
            }
          );
          prevPoingInfo = aryInfo[j];
          // 开始执行动画
          i = 0;
          j++;
          chunkInfos = chunk.features;
          moveAction(chunk.features)
        } else
        {
          prevPoingInfo = aryInfo[j];
          j++;
          chunkLine()
        }
      } else
      {
        i = 0;
        j = 0;
        geojson.features[0].geometry.coordinates = [];
        map.getSource('line-animation').setData(geojson);
        chunkLine()
      }
    }
    //动画效果
    function moveAction(chunkInfo)
    {
      if(i < chunkInfo.length)
      {
        var currentPoint = chunkInfo[i].geometry.coordinates[1];

        geojson.features[0].geometry.coordinates.push(currentPoint);
        if(prevPoint)
        {
          var bearing = turf.bearing(prevPoint, turf.point(currentPoint));

          bearing && movePointMarker.currentObj.setLngLat(currentPoint).setRotate((bearing - 90))
        }
        prevPoint = turf.point(currentPoint);
        map.getSource('line-animation').setData(geojson);
        i++;
        $this.time_id = requestAnimationFrame(moveAction.bind(this, chunkInfo))
        //time_id = setTimeout(()=>{moveAction(chunkInfo)},17)
      } else
      {
        cancelAnimationFrame($this.time_id)
        chunkLine()
      }
    }
    if(callback)
    {
      let moveobj = {
        //播放
        play: function ()
        {
          if(autoPlay)
          {
            moveAction(chunkInfos)
          } else
          {
            autoPlay = true;
            chunkLine();
          }
        },
        //暂停移动
        pause: function ()
        {
          cancelAnimationFrame($this.time_id)
        },
        changeSpeed: function (val)
        {
          speed = val;
        }
      };
      callback(moveobj)
    }
  },
  ////停止移动
  //stopMove(data)
  //{
  //  data.obj.currentObj.stopMove();
  //},
  ////暂停移动
  //pauseMove(data)
  //{
  //  data.obj.currentObj.pauseMove();
  //},
  ////继续移动
  //resumeMove(data)
  //{
  //  data.obj.currentObj.resumeMove();
  //},
  //注册地图事件
  on(data)
  {
    var $this = this;
    if(data.obj.currentObj.style)
    {
      if(data.eventName == "complete")
      {
        data.eventName = "load";
      };
      data.obj.currentObj.on(data.eventName, (e) =>
      {
        if(data.callback)
        {
          data.callback(e);
        }
      });
    }
    else
    {
      let markObj = data.obj.currentObj;
      let markDom = markObj.getElement();
      markDom.addEventListener('mouseenter', function ()
      {
        markDom.style.cursor = 'pointer'
      });
      markDom.addEventListener('mouseleave', function ()
      {
        markDom.style.cursor = ''
      });
      markDom.addEventListener(data.eventName, function ()
      {
        var obj = {};
        obj.target = {};
        obj.target.currentObj = markObj;
        obj.lnglat = markObj.getLngLat();
        obj.type = data.eventName;
        if(data.callback)
        {
          data.callback(obj);
        }
      });
    }
  },
  //poi搜索
  search(id, callback)
  {
    //加载PoiPicker
    AMapUI.loadUI(['misc/PoiPicker'], function (PoiPicker)
    {
      var auto = new PoiPicker({
        input: id,//输入框id
      });
      auto.on("poiPicked", (res) =>
      {
        var data = {};
        data.location = res.item.location;
        data.adcode = res.item.adcode;
        data.district = res.item.district;
        data.address = res.item.address;
        data.name = res.item.name;
        if(callback)
        {
          callback(data);
        }
      })

    });
  },

  //关闭测距
  closeRangTool()
  {
    this.rangtool.turnOff();
  },

  //测距
  openRangTool(data)
  {
    this.rangtool.turnOn();
    this.rangtool.on("addnode", (res) =>
    {
      if(data.addcallback)
      {
        data.addcallback(res)
      }
    });
    this.rangtool.on("removenode", (res) =>
    {
      if(data.delcallback)
      {
        data.delcallback(res)
      }
    });
    this.rangtool.on("end", (res) =>
    {
      if(data.endcallback)
      {
        data.endcallback(res)
      }
    })

  },

  //开启鼠标工具
  openMouseTool(data)
  {
    var _this = this;
    this.mousetool.on("draw", (e) =>
    {
      let drawData = {
        radius: 0,
        points: null,
        type: data.type,
        target: e.obj
      }
      switch(e.obj.CLASS_NAME)
      {
        case "AMap.Circle":
          drawData.radius = e.obj.getRadius();
          let position = e.obj.getCenter();
          drawData.points = {
            currentObj: position,
            lng: position.lng,
            lat: position.lat,

          };
          break;
        case "AMap.Polygon":
          drawData.points = _this.setPoints(e.obj.getPath());
          drawData.area = e.obj.getArea();
          break;
        case "AMap.Polyline":
          drawData.points = _this.setPoints(e.obj.getPath());
          drawData.length = e.obj.getLength();
      }
      if(data.callback)
      {
        data.callback(drawData);
      }
    })

    //开启鼠标操作
    switch(data.type)
    {
      case 'marker':
        this.mousetool.marker();
        break;
      case 'polyline':
        this.mousetool.polyline();
        break;
      case 'polygon':
        this.mousetool.polygon();
        break;
      case 'rectangle':
        this.mousetool.rectangle();
        break;
      case 'circle':
        this.mousetool.circle();
        break;
      case 'rule':
        this.mousetool.rule();
        break;
      case 'measureArea':
        this.mousetool.measureArea();
        break;
      case 'rectZoomIn':
        this.mousetool.rectZoomIn();
        break;
      case 'rectZoomOut':
        this.mousetool.rectZoomOut();
        break;
    }
  },

  //关闭鼠标工具
  closeMouseTool()
  {
    this.mousetool.close();
  },

  //覆盖物
  overLay: function (data)
  {
    var map = this.getMapObj(data).currentObj;
    var shape = null;
    var $this = this;
    var lnglat = this.getPoints(data.points);
    if(data.type == "circle")
    {
      shape = new AMap.Circle({
        title: data.title ? data.title : "特殊",
        map: map,
        center: lnglat[0],  // 圆心位置
        radius: data.radius, // 圆半径
        strokeColor: strokeColor, //线颜色
        strokeOpacity: strokeOpacity, //线透明度
        strokeWeight: strokeWeight,    //线宽
        fillColor: fillColor, //填充色
        fillOpacity: fillOpacity//填充透明度
      });
      map.setZoomAndCenter(12, lnglat[0]);
    }
    else if(data.type == "ellipse")
    {
      shape = new AMap.Ellipse({
        map: map,
        center: lntlat[0],  // 圆心位置
        radius: data.radius, // 圆半径
        strokeColor: "#1791fc", //线颜色
        strokeOpacity: 0.9, //线透明度
        strokeWeight: 2,    //线宽
        fillColor: "#1791fc", //填充色
        fillOpacity: 0.35//填充透明度
      });
      map.setZoomAndCenter(12, lnglat[0]);
    }
    else
    {
      var config = {
        map: map,
        path: lnglat,
        strokeColor: "#1791fc", //线颜色
        strokeOpacity: 0.9, //线透明度
        strokeWeight: 2,    //线宽
        fillColor: "#1791fc", //填充色
        fillOpacity: 0.35//填充透明度
      }
      map.setZoomAndCenter(12, lnglat[0]);
    }

    if(data.type == "polygon" || data.type == "rectangle")
    {
      shape = new AMap.Polygon(config);
    }
    else if(data.type == "polyline")
    {
      shape = new AMap.Polyline(config);
    }
    var obj = {};
    obj.currentObj = shape;
    return shape;
  },

  //绘制线路
  drawRoute(data)
  {
    var amap = this.getMapObj(data).currentObj;
    if(data.dragroute === undefined)
    {
      //路线规划没有传递对象时，默认只有一条数据，操作的都是一条数据
      var $this = this;
      var mapData = {};
      if(this.dragroute)
      {
        this.dragroute.destroy();
      }
      if(data.path)
      {
        if(data.path.length == 0)
        {
          path = [];
          mapData.points = [];
          mapData.routeObj = null;
          mapData.start = null;
          mapData.end = null;
          mapData.waypoints = [];
          if(data.completecallback)
          {
            data.completecallback(mapData);
          }
        }
      }
      let copyPath = core.clone(data.path);
      this.dragroute = new AMap.DragRoute(amap, copyPath, data.type);
      this.dragroute.search();

      this.dragroute.on("complete", function (obj)
      {
        mapData.points = mapFunc.getPath(obj.data.start.location, obj.data.waypoints, obj.data.end.location);
        mapData.routeObj = obj.target;
        mapData.start = obj.data.start.location;
        mapData.end = obj.data.end.location;
        var way = obj.data.waypoints;
        mapData.waypoints = [];
        for(var i = 0; i < way.length; i++)
        {
          mapData.waypoints.push(way[i].location);
        }
        if(data.completecallback)
        {
          data.completecallback(mapData);
        }
      })
      this.dragroute.on("addway", function (obj)
      {
        if(data.addwaycallback)
        {
          data.addwaycallback(obj);
        }
      })
    }
    else
    {
      //参数包括对象时，处理的是参数的路线对象
    }
  },

  //摧毁路线
  destroyRoute(data)
  {
    if(data.obj)
    {
      data.obj.destroy();
    } else
    {
      throw new Error("dstory 的参数不正确");
    }
  },
  //绘制行政区划
  drawDistrict(data)
  {
    var $this = this;
    var amap = this.getMapObj(data).currentObj;
    //this.district.setLevel = data.level;
    this.district.searchDistrict({
      keyword: data.name,
      subdistrict: 1,
      extension: true,
      start: "",
      limit: ""
    }).then(result =>
    {
      var features = [];
      for(var o in result[0].sub_districts)
      {
        features.push({
          'type': 'Feature',
          'geometry': result[0].sub_districts[o].shape
        })
        for(var i in result[0].sub_districts[o].sub_districts)
        {
          features.push({
            'type': 'Feature',
            'geometry': result[0].sub_districts[o].sub_districts[i].shape
          })
        }
      }

      if(amap.getSource('power-polygon'))
      {
        amap.getSource('power-polygon').setData({
          type: "FeatureCollection",
          features: features
        });
      } else
      {
        //重设面数据
        amap.addSource("power-polygon", {
          type: 'geojson',
          data: {
            'type': 'FeatureCollection',
            'features': features
          }
        });

        //画面
        amap.addLayer({
          "id": "power-polygon",
          'type': 'fill',
          'source': "power-polygon",
          'paint': {
            'fill-color': '#409EFF',
            'fill-opacity': .5,
            "fill-outline-color": "#409EFF"
          }
        });
      }
      if(data.callback)
      {
        var shape = {};
        let district = amap.getSource('power-polygon');
        shape[data.name] = district;
        shape.setMap = function ()
        {
          district.setData({
            type: "FeatureCollection",
            features: []
          });
        };
        data.callback("complete", result, shape);
      }
    });

  },
  //点聚合
  markerClusterer(data)
  {
    var amap = this.getMapObj(data).currentObj;
    var $this = this;
    var points = this.getPoints(data.markers);
    var cluster = new AMap.MarkerClusterer(amap, data.markers, {
      gridSize: data.gridSize ? data.gridSize : 60,
    });
    cluster.on("click", (data) =>
    {
      if(data.clickcallback)
      {
        data.clickcallback(data);
      }
    });
    amap.setFitView();
    obj = {};
    obj.currentObj = cluster;
    return obj;
  },
  //点聚合添加点
  addClusterMarker(data)
  {
    data.obj.currentObj.addMarker(data.marker.currentObj);
    var amap = this.getMapObj(data).currentObj;
    amap.setFitView();

  },
  //清除所有点聚合
  clearClusterMarkers(config)
  {
    config.obj.currentObj.clearMarkers();
  },
  //移除单个聚合点
  removeClusterMarker(data)
  {
    if(data.obj)
    {
      data.obj.currentObj.removeMarker(data.marker.currentObj);
    } else
    {
      throw new Error("removeClusterMarker参数不正确");
    }
  },
  //获取聚合数量
  getClustersCount(data)
  {
    if(data.obj)
    {
      return data.obj.currentObj.getClustersCount();
    } else
    {
      throw new Error("getClustersCount参数不正确");
    }
  },
  //信息窗体
  infoWindow(data)
  {
    var infoWindow = new AMap.InfoWindow({
      content: data.content ? data.content : "内容为空",
      offset: data.offset ? data.offset : new AMap.Pixel(40, 15),
    })
    infoWindow.on("close", () =>
    {
      if(data.closecallback)
      {
        data.closecallback();
      }
    })
    infoWindow.on("open", () =>
    {
      if(data.opencallback)
      {
        data.opencallback();
      }
    })
    infoWindow.on("change", () =>
    {
      if(data.changecallback)
      {
        data.changecallback();
      }
    })
    var obj = {};
    obj.currentObj = infoWindow;
    return obj;
  },
  simpleInfoWindow(callback)
  {
    //加载SimpleInfoWindow，loadUI的路径参数为模块名中 'ui/' 之后的部分
    AMapUI.loadUI(['overlay/SimpleInfoWindow'], function (SimpleInfoWindow)
    {
      var infoWindow = new SimpleInfoWindow({
        infoTitle: '<strong>这里是标题</strong>',
        infoBody: '<p>这里是内容。</p>'
      });
      //显示在map上
      infoWindow.open(map, map.getCenter());
      var obj = {};
      obj.currentObj = infoWindow;
      if(callback)
      {
        callback(obj);
      }
    });
  },
  //打开信息窗体
  openInfoWindow(data)
  {
    var amap = this.getMapObj(data).currentObj;
    if(data.position.currentObj)
    {
      var position = data.position.currentObj
    } else if(Array.isArray(data.position))
    {
      var position = this.lngLat(data.position[0], data.position[1]).currentObj;
    }
    data.obj.currentObj.open(amap, position);
  },
  closeInfoWindow(obj)
  {
    obj.currentObj.close();
  },
  setInfoContent(data)
  {
    data.obj.currentObj.setContent(data.content);
  },
  getInfoContent(obj)
  {
    return obj.currentObj.getContent()
  },
  setInfoPosition(data)
  {
    var lnglat = data.lnglat.currentObj ? data.lnglat.currentObj : this.lngLat(data.lnglat).currentObj;
    data.obj.currentObj.setPosition(lnglat);
  },
  getInfoPosition(obj)
  {
    return { currentObj: obj.currentObj.getPosition() }
  }
}
export default obj;
